#include <BSLinearCamera.h>
#include <BSVector3f.h>
#include <BSRainbow.h>
#include <BSRainbowSelective.h>
#include <BSRefractionConst.h>
#include <BSRefractionPower.h>
#include <BSRandom.h>
#include <BSObjectManager.h>

#include <cstdio>
#include <cstdlib>
#include <ctime>

bool isEmpty(char *buffer)
{
	if (*buffer == 0)
		return true;
	else if (*buffer <= ' ')
		return isEmpty(buffer+1);
	else
		return false;
}

int readInt(int defaultValue, int max)
{
	char buffer[256];
	int r;
	gets(buffer);

	if (isEmpty(buffer))
		return defaultValue;
	r = atoi(buffer);
	if (r < 0 || r > max)
		return defaultValue;

	return r;
}

int readInt(int defaultValue)
{
	char buffer[256];
	int r;
	gets(buffer);

	if (isEmpty(buffer))
		return defaultValue;
	r = atoi(buffer);
	return r;
}

float readFloat(float defaultValue)
{
	char buffer[256];
	float r;
	gets(buffer);

	if (isEmpty(buffer))
		return defaultValue;
	r = atof(buffer);
	return r;
}

unsigned long long readBinary(unsigned long long defaultValue)
{
	char buffer[256];
	unsigned long long r;
	gets(buffer);

	if (isEmpty(buffer))
		return defaultValue;
	r = 0;
	for (char *c = buffer; *c; c++)
	{
		r <<= 1;
		if (*c != '0')
			r |= 1;
	}
	return r;
}

BlueSky::Refraction *selectRefraction(int defaultValue)
{
	printf("Select refractive index:\n"\
		"0) Water\n"\
		"1) Constant refractive index\n"\
		"2) Power function\n");
	printf("[%d]> ", defaultValue);

	int ans = readInt(defaultValue, 2);

	switch (ans)
	{
	case 0:
		return new BlueSky::RefractionPower(BlueSky::RefractionPower::WATER);
	case 1:
		printf("Refractive index [1.333]: ");
		return new BlueSky::RefractionConst(readFloat(1.333f));
	case 2:
		{
			printf("Shift [1.32269]: ");
			float shift = readFloat(1.32269f);
			printf("Scale [1126.42]: ");
			float scale = readFloat(1126.42f);
			printf("Power [-1.83538]: ");
			float power = readFloat(-1.83538);
			return new BlueSky::RefractionPower(shift, scale, power);
		}
	default:
		return NULL;
	}
}

BlueSky::Photon ph0;

BlueSky::Photon emitRGB()
{
	switch (rand()%3)
	{
	case 0: // R
		return BlueSky::Photon(
			BlueSky::Vector3f::UNIT_X,
			BlueSky::Vector3f(1.0f, 0.0f, 0.0f),
			650.0f);
	case 1: // G
		return BlueSky::Photon(
			BlueSky::Vector3f::UNIT_X,
			BlueSky::Vector3f(0.0f, 1.0f, 0.0f),
			510.0f);
	case 2: // B
		return BlueSky::Photon(
			BlueSky::Vector3f::UNIT_X,
			BlueSky::Vector3f(0.0f, 0.0f, 1.0f),
			440.0f);
	}
	return ph0;
}

BlueSky::Photon emitPh0()
{
	return ph0;
}

int main(int argc, char **argv)
{
	int ans, resolution, mode;
	BlueSky::ObjectManager mgr;
	BlueSky::LinearCamera *camera = NULL;
	BlueSky::Action       *action = NULL;

	srand(time(NULL));

	printf("Select camera\n"\
		"0) Linear camera\n"\
		"1) Angle vs param\n"\
		"2) Angle vs sqrt(param) (excentricity)\n");
	printf("[0]> ");

	ans = readInt(0, 2);

	switch (ans)
	{
	case 0: // Linear camera
		printf("Resolution [1024]: ");
		camera = new BlueSky::LinearCamera(readInt(1024));
		mgr.manage(camera);
		break;
	case 1:
		printf("Resolution [1024]: ");
		resolution = readInt(1024);
		camera = NULL;
		mode = 0;
		break;	
	case 2:
		printf("Resolution [1024]: ");
		resolution = readInt(1024);
		camera = NULL;
		mode = 1;
		break;	
	default:
		return -1;
	}

	printf("Select action\n"\
		"0) Rainbow\n"\
		"1) Selective rainbow\n");
	printf("[0]> ");

	ans = readInt(0, 1);

	switch (ans)
	{
	case 0: // Rainbow
		action = new BlueSky::Rainbow(selectRefraction(0));
		mgr.manage(action);
		break;
	case 1: // Selective rainbow
		{
			printf("Path [1100]: ");
			unsigned long long path = readBinary(12);
			action = new BlueSky::RainbowSelective(path, selectRefraction(0));
			mgr.manage(action);
		}
		break;
	default:
		return -1;
	}

	float radius = 0.0;
	int it = 0.0;
	if (camera)
	{
		printf("Sun radius (deg) [0.25]: ");
		radius = readFloat(0.25) * BlueSky::Math::PIf / 180.0f;
		printf("Iterations[1000000]: ");
		it = readInt(1000000);
	}

	printf("Select wavelength:\n"\
		"0) RGB\n"\
		"1) R (650nm)\n"\
		"2) G (510nm)\n"\
		"3) B (440nm)\n"\
		"4) Other monoichromatic\n");
	printf("[0]> ");

	BlueSky::Photon (*emiter)();
	switch (readInt(0, 4))
	{
	case 0:
		emiter = emitRGB;
		break;
	case 1:
		ph0 = BlueSky::Photon(
			BlueSky::Vector3f::UNIT_X, 
			BlueSky::Vector3f(1.0f, 0.0f, 0.0f),
			650.0f);
		emiter = emitPh0;
		break;
	case 2:
		ph0 = BlueSky::Photon(
			BlueSky::Vector3f::UNIT_X, 
			BlueSky::Vector3f(0.0f, 1.0f, 0.0f),
			510.0f);
		emiter = emitPh0;
		break;
	case 3:
		ph0 = BlueSky::Photon(
			BlueSky::Vector3f::UNIT_X, 
			BlueSky::Vector3f(0.0f, 0.0f, 1.0f),
			440.0f);
		emiter = emitPh0;
		break;
	case 4:
		printf("Wavelength [550.0]: ");
		ph0 = BlueSky::Photon(BlueSky::Vector3f::UNIT_X, readFloat(550.0f));
		emiter = emitPh0;
		break;
	}

	if (camera)
	{
		for (int i = 0; i < it; i++)
		{
			float x, y;
			BlueSky::Random::randomCircle(x, y);
			BlueSky::Photon ph = emiter();
			ph.dir.y = x * radius;
			ph.dir.z = y * radius;
			ph.dir.normalise();

			camera->collect(action->action(ph, BlueSky::Random::randomFloat()));
		}

		FILE *file = fopen("result.dat", "wt");
		fprintf(file, "#angle\tIR\tIG\tIB\n");
		for (int i = 0; i < camera->getResolution(); i++)
		{
			fprintf(file, "%f\t%f\t%f\t%f\n", 
				camera->getAngle(i) * 180.0 / BlueSky::Math::PIf, 
				camera->getData(i).x,
				camera->getData(i).y,
				camera->getData(i).z);
		}
		fclose(file);
	}
	else
	{
		FILE *file = fopen("result.dat", "wt");
		fprintf(file, "#param\tangle\n");
		for (int i = 0; i < resolution; i++)
		{
			float p = (float)i / (float)resolution;
			float p2 = (mode ? p*p : p);
			fprintf(file, "%f\t%f\n",
				p,
				acos(action->action(emiter(), p2).dir.x) * 180.0f / BlueSky::Math::PIf);
		}
	}

	return 0;
}
