/* ----------------------------------------------------------------------------
This file is part of BlueSky

Copyright (C) 2012 Piotr Polesiuk

Permission is hereby granted, free of charge, to any person obtaining a copy 
of this software and associated documentation files (the "Software"), to deal 
in the Software without restriction, including without limitation the rights 
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
copies of the Software, and to permit persons to whom the Software is 
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in 
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
THE SOFTWARE.
---------------------------------------------------------------------------- */

#include "actionDescr.h"

#include <QtCore/QStringList>
#include "errorOutput.h"

#include <BSActionAlternative.h>
#include <BSCrystalAction.h>
#include <BSCrystalSelectiveAction.h>
#include <BSIdentityAction.h>
#include <BSRainbow.h>
#include <BSRainbowSelective.h>

#ifndef NULL
#define NULL 0
#endif

ActionDescr::ActionDescr()
{
	type = IdentityAction;
	refraction = NULL;
	rsPath = 0LLU;
	crystal = NULL;
	rotation = NULL;
	defaultAction = NULL;
}

ActionDescr::ActionDescr(const ActionDescr &actionDescr)
{
	type = actionDescr.type;
	if (actionDescr.refraction)
		refraction = new RefractionDescr(*actionDescr.refraction);
	else
		refraction = NULL;
	rsPath = actionDescr.rsPath;
	pathSetString = actionDescr.pathSetString;
	if (actionDescr.crystal)
		crystal = new CrystalDescr(*actionDescr.crystal);
	else
		crystal = NULL;
	if (actionDescr.rotation)
		rotation = new RotationDescr(*actionDescr.rotation);
	else
		rotation = NULL;
	if (actionDescr.defaultAction)
		defaultAction = new ActionDescr(*actionDescr.defaultAction);
	else
		defaultAction = NULL;
	additionalActions = actionDescr.additionalActions;
	additionalProbabilities = actionDescr.additionalProbabilities;
}

ActionDescr::~ActionDescr()
{
	if (refraction)
		delete refraction;
	if (crystal)
		delete crystal;
	if (rotation)
		delete rotation;
	if (defaultAction)
		delete defaultAction;
}

const ActionDescr &ActionDescr::operator = (const ActionDescr &actionDescr)
{
	type = actionDescr.type;
	if (refraction)
		delete refraction;
	if (actionDescr.refraction)
		refraction = new RefractionDescr(*actionDescr.refraction);
	else
		refraction = NULL;
	rsPath = actionDescr.rsPath;
	pathSetString = actionDescr.pathSetString;
	if (crystal)
		delete crystal;
	if (actionDescr.crystal)
		crystal = new CrystalDescr(*actionDescr.crystal);
	else
		crystal = NULL;
	if (rotation)
		delete rotation;
	if (actionDescr.rotation)
		rotation = new RotationDescr(*actionDescr.rotation);
	else
		rotation = NULL;
	if (defaultAction)
		delete defaultAction;
	if (actionDescr.defaultAction)
		defaultAction = new ActionDescr(*actionDescr.defaultAction);
	else
		defaultAction = NULL;
	additionalActions = actionDescr.additionalActions;
	additionalProbabilities = actionDescr.additionalProbabilities;
	return *this;
}

ActionDescr::ActionType ActionDescr::getActionType() const
{
	return type;
}

void ActionDescr::setActionType(ActionType _type)
{
	type = _type;
}

RefractionDescr *ActionDescr::getRefractionDescr() const
{
	return refraction;
}

void ActionDescr::setRefractionDescr(RefractionDescr *refractionDescr)
{
	if (refraction)
		delete refraction;
	if (refractionDescr)
		refraction = new RefractionDescr(*refractionDescr);
	else
		refraction = NULL;
}

QString ActionDescr::getPathSetString() const
{
	return pathSetString;
}

void ActionDescr::setPathSetString(QString pathSet)
{
	pathSetString = pathSet;
}

CrystalDescr *ActionDescr::getCrystalDescr() const
{
	return crystal;
}

void ActionDescr::setCrystalDescr(CrystalDescr *crystalDescr)
{
	if (crystal)
		delete crystal;
	if (crystalDescr)
		crystal = new CrystalDescr(*crystalDescr);
	else
		crystal = NULL;
}
	
RotationDescr *ActionDescr::getRotationDescr() const
{
	return rotation;
}

void ActionDescr::setRotationDescr(RotationDescr *rotationDescr)
{
	if (rotation)
		delete rotation;
	if (rotationDescr)
		rotation = new RotationDescr(*rotationDescr);
	else
		rotation = NULL;
}

ActionDescr *ActionDescr::getDefaultActionDescr() const
{
	return defaultAction;
}

void ActionDescr::setDefaultActionDescr(ActionDescr *actionDescr)
{
	if (defaultAction)
		delete defaultAction;
	if (actionDescr)
		defaultAction = new ActionDescr(*actionDescr);
	else
		defaultAction = NULL;
}

int ActionDescr::getAdditionalActionCount() const
{
	return additionalActions.count();
}

const ActionDescr &ActionDescr::getAdditionalActionDescr(int index) const
{
	return additionalActions[index];
}

float ActionDescr::getAdditionalActionProbability(int index) const
{
	return additionalProbabilities[index];
}

void ActionDescr::appendAdditionalAction(const ActionDescr &actionDescr, float probability)
{
	additionalActions.append(actionDescr);
	additionalProbabilities.append(probability);
}

BlueSky::Action *ActionDescr::getAction() const
{
	switch (type)
	{
	case ActionAlternative:
		{
			BlueSky::ActionAlternative *result;
			if (defaultAction)
				result = new BlueSky::ActionAlternative(defaultAction->getAction());
			else
				result = new BlueSky::ActionAlternative();

			for (int i = 0; i < additionalActions.count(); i++)
				result->addAction(
					additionalActions[i].getAction(),
					additionalProbabilities[i]);

			return result;
		}
	case IdentityAction:
		return new BlueSky::IdentityAction();
	case Rainbow:
		if (refraction)
			return new BlueSky::Rainbow(refraction->getRefraction());
		else
			return new BlueSky::Rainbow();
	case RainbowSelective:
		{
			unsigned long long path = (rsPath != 0LLU ? rsPath : 0xFFFFFFFFFFFFFFFFLLU);
			if (refraction)
				return new BlueSky::RainbowSelective(path, refraction->getRefraction());
			else
				return new BlueSky::RainbowSelective(path);
		}
	case CrystalAction:
		{
			BlueSky::Crystal *bsCrystal = (
				crystal ? crystal->getCrystal() :
				BlueSky::Crystal::createPrism(6, 1.414f, 0.707f, 0.0f, 0.0f) );
			if (refraction)
			{
				if (rotation)
					return new BlueSky::CrystalAction(
						bsCrystal, 
						refraction->getRefraction(),
						rotation->getRotation()
					);
				else
					return new BlueSky::CrystalAction(bsCrystal, refraction->getRefraction());
			}
			else if (rotation)
				return new BlueSky::CrystalAction(bsCrystal, rotation->getRotation());
			else
				return new BlueSky::CrystalAction(bsCrystal);
		}
	case CrystalSelectiveAction:
		{
			BlueSky::Crystal *bsCrystal = (
				crystal ? crystal->getCrystal() :
				BlueSky::Crystal::createPrism(6, 1.414f, 0.707f, 0.0f, 0.0f) );
			BlueSky::CrystalPathSet *pathSet = parsePathSet(bsCrystal->getFaceCount());
			if (refraction)
			{
				if (rotation)
					return new BlueSky::CrystalSelectiveAction(
						bsCrystal,
						pathSet,
						refraction->getRefraction(),
						rotation->getRotation()
					);
				else
					return new BlueSky::CrystalSelectiveAction(
						bsCrystal,
						pathSet,
						refraction->getRefraction());
			}
			else if (rotation)
				return new BlueSky::CrystalSelectiveAction(
						bsCrystal, 
						pathSet, 
						rotation->getRotation());
			else
				return new BlueSky::CrystalSelectiveAction(bsCrystal, pathSet);
		}
	}
	return NULL;
}

QDomElement ActionDescr::getDomElement() const
{
	QDomDocument doc;
	switch (type)
	{
	case ActionAlternative:
		{
			QDomElement result = doc.createElement("ActionAlternative");
			if (defaultAction)
			{
				QDomElement defaultActNode = doc.createElement("DefaultAction");
				result.appendChild(defaultActNode);
				defaultActNode.appendChild(defaultAction->getDomElement());
			}

			for (int i = 0; i < additionalActions.count(); i++)
			{
				QDomElement additionalActNode = doc.createElement("AdditionalAction");
				result.appendChild(additionalActNode);
				additionalActNode.appendChild(additionalActions[i].getDomElement());
				additionalActNode.setAttribute("probability", additionalProbabilities[i]);
			}

			return result;
		}
	case IdentityAction:
		return doc.createElement("IdentityAction");
	case Rainbow:
		{
			QDomElement result = doc.createElement("Rainbow");
			if (refraction)
			{
				QDomElement refractionNode = doc.createElement("Refraction");
				result.appendChild(refractionNode);
				refractionNode.appendChild(refraction->getDomElement());
			}
			return result;
		}
	case RainbowSelective:
		{
			QDomElement result = doc.createElement("RainbowSelective");
			if (refraction)
			{
				QDomElement refractionNode = doc.createElement("Refraction");
				result.appendChild(refractionNode);
				refractionNode.appendChild(refraction->getDomElement());
			}
			QDomElement pathSet = doc.createElement("PathSet");
			result.appendChild(pathSet);
			QDomElement pathHexMask = doc.createElement("PathHexMask");
			pathSet.appendChild(pathHexMask);
			pathHexMask.setAttribute("mask", QString("%1").arg(rsPath, 0, 16));
			return result;
		}
	case CrystalAction:
		{
			QDomElement result = doc.createElement("CrystalAction");
			if (crystal)
			{
				QDomElement crystalNode = doc.createElement("Crystal");
				result.appendChild(crystalNode);
				crystalNode.appendChild(crystal->getDomElement());
			}
			if (refraction)
			{
				QDomElement refractionNode = doc.createElement("Refraction");
				result.appendChild(refractionNode);
				refractionNode.appendChild(refraction->getDomElement());
			}
			if (rotation)
			{
				QDomElement rotationNode = doc.createElement("Rotation");
				result.appendChild(rotationNode);
				rotationNode.appendChild(rotation->getDomElement());
			}
			return result;
		}
	case CrystalSelectiveAction:
		{
			QDomElement result = doc.createElement("CrystalSelectiveAction");
			if (crystal)
			{
				QDomElement crystalNode = doc.createElement("Crystal");
				result.appendChild(crystalNode);
				crystalNode.appendChild(crystal->getDomElement());
			}
			if (refraction)
			{
				QDomElement refractionNode = doc.createElement("Refraction");
				result.appendChild(refractionNode);
				refractionNode.appendChild(refraction->getDomElement());
			}
			if (rotation)
			{
				QDomElement rotationNode = doc.createElement("Rotation");
				result.appendChild(rotationNode);
				rotationNode.appendChild(rotation->getDomElement());
			}
			QDomElement pathSetNode = doc.createElement("PathSet");
			result.appendChild(pathSetNode);
			QDomElement pathSetStringNode = doc.createElement("PathSetString");
			pathSetNode.appendChild(pathSetStringNode);
			pathSetStringNode.setAttribute("string", pathSetString);
			return result;
		}
	default:
		return doc.createElement("Unknown");
	}
}

QString ActionDescr::getActionName() const
{
	switch (type)
	{
	case ActionAlternative:
		return QObject::tr("Alternative of actions");
	case IdentityAction:
		return QObject::tr("Identity action");
	case Rainbow:
		return QObject::tr("Rainbow");
	case RainbowSelective:
		return QObject::tr("Selected rainbows");
	case CrystalAction:
		return QObject::tr("Crystal");
	case CrystalSelectiveAction:
		return QObject::tr("Crystal (selected paths)");
	}
	return QObject::tr("Unknown");
}

QString ActionDescr::getRSPathString() const
{
	QString result = "";
	bool first = true;

	if ((rsPath & 0x1LLU) != 0LLU)
	{
		result.append("R");
		first = false;
	}

	int order = 0;
	unsigned long long mask = 0x2LLU;

	while (true)
	{
		if (mask == 0LLU)
			return result;

		if (rsPath & mask)
		{
			int minord = order;
			
			while (rsPath & mask)
			{
				order++;
				mask <<= 1;
			}

			if (!first)
				result.append(',');
			result.append(QString("%1").arg(minord));
			if (order > minord + 1)
				result.append(QString("-%1").arg(order-1));
		}

		order++;
		mask <<= 1;
	}
}

void ActionDescr::setRSPathString(QString path)
{
	rsPath = 0LL;

	QStringList elems = path.split(',', QString::SkipEmptyParts);

	for (int i = 0; i < elems.size(); i++)
	{
		if (elems[i].trimmed().toUpper() == "R")
			rsPath |= 1LLU;
		else if (elems[i].contains('-'))
		{
			QStringList elems2 = elems[i].split('-');
			int min = elems2[0].toInt();
			int max = elems2[1].toInt();
			if (max >= 62)
				rsPath |= (0xFFFFFFFFFFFFFFFFLLU << (min + 1));
			else
				rsPath |= ((0xFFFFFFFFFFFFFFFFLLU << (min + 1)) &
					~(0xFFFFFFFFFFFFFFFFLLU << (max + 2)));
		}
		else
		{
			int order = elems[i].toInt();
			rsPath |= (1LLU << (order + 1));
		}
	}
}

ActionDescr ActionDescr::fromDomElement(QDomElement node, QString nodeName, ErrorOutput *output)
{
	ActionDescr result;
	QDomElement actionNode = (nodeName.isEmpty() ? node : node.firstChildElement(nodeName));

	if (actionNode.isNull())
	{
		output->error(QObject::tr("No %1 node!").arg(nodeName));
		return result;
	}

	actionNode = actionNode.firstChildElement();
	if (actionNode.isNull())
	{
		output->error(QObject::tr("No action specified."));
		return result;
	}

	if (actionNode.tagName() == "ActionAlternative")
	{
		result.type = ActionAlternative;
		ActionDescr defaultActionDescr = fromDomElement(actionNode, "DefaultAction", output);
		result.setDefaultActionDescr(&defaultActionDescr);

		QDomElement addActNode = actionNode.firstChildElement("AdditionalAction");
		while (!addActNode.isNull())
		{
			ActionDescr additionalActionDescr = fromDomElement(addActNode, "", output);
			if (addActNode.hasAttribute("probability"))
			{
				bool ok;
				float prob = addActNode.attributeNode("probability").value().toFloat(&ok);

				if (!ok)
					output->error(QObject::tr(
						"ActionAlternative/AdditionalAction/probability : this is not valid number."));

				result.appendAdditionalAction(additionalActionDescr, prob);
			}
			else
				output->error(QObject::tr(
					"ActionAlternative/AdditionalAction : no probability attribute."));
				
			addActNode = addActNode.nextSiblingElement("AdditionalAction");
		}
		return result;
	}
	else if (actionNode.tagName() == "CrystalAction")
	{
		result.type = CrystalAction;
		CrystalDescr cd = CrystalDescr::fromDomElement(actionNode, "Crystal", output);
		result.setCrystalDescr(&cd);
		QDomElement refractionNode = actionNode.firstChildElement("Refraction");
		if (!refractionNode.isNull())
		{
			RefractionDescr rd = 
				RefractionDescr::fromDomElement(refractionNode, "", output);
			result.setRefractionDescr(&rd);
		}
		QDomElement rotationNode = actionNode.firstChildElement("Rotation");
		if (!rotationNode.isNull())
		{
			RotationDescr rd = 
				RotationDescr::fromDomElement(rotationNode, "", output);
			result.setRotationDescr(&rd);
		}
	}
	else if (actionNode.tagName() == "CrystalSelectiveAction")
	{
		result.type = CrystalSelectiveAction;
		CrystalDescr cd = CrystalDescr::fromDomElement(actionNode, "Crystal", output);
		result.setCrystalDescr(&cd);
		QDomElement refractionNode = actionNode.firstChildElement("Refraction");
		if (!refractionNode.isNull())
		{
			RefractionDescr rd = 
				RefractionDescr::fromDomElement(refractionNode, "", output);
			result.setRefractionDescr(&rd);
		}
		QDomElement rotationNode = actionNode.firstChildElement("Rotation");
		if (!rotationNode.isNull())
		{
			RotationDescr rd = 
				RotationDescr::fromDomElement(rotationNode, "", output);
			result.setRotationDescr(&rd);
		}
		result.pathSetString = pathSetStringFromDomElement(actionNode, output);
	}
	else if (actionNode.tagName() == "IdentityAction")
		result.type = IdentityAction;
	else if (actionNode.tagName() == "Rainbow")
	{
		result.type = Rainbow;
		QDomElement refractionNode = actionNode.firstChildElement("Refraction");
		if (!refractionNode.isNull())
		{
			RefractionDescr rd = 
				RefractionDescr::fromDomElement(refractionNode, "", output);
			result.setRefractionDescr(&rd);
		}
	}
	else if (actionNode.tagName() == "RainbowSelective")
	{
		result.type = RainbowSelective;
		QDomElement refractionNode = actionNode.firstChildElement("Refraction");
		if (!refractionNode.isNull())
		{
			RefractionDescr rd = 
				RefractionDescr::fromDomElement(refractionNode, "", output);
			result.setRefractionDescr(&rd);
		}
		result.rsPath = rainbowSelectivePathsFromDomElement(actionNode, output);
	}
	else
	{
		output->error(QObject::tr("Unknown action : %1").arg(actionNode.tagName()));
	}

	return result;
}
	
unsigned long long ActionDescr::rainbowSelectivePathsFromDomElement(QDomElement node, ErrorOutput *output)
{
	QDomElement pathSetNode = node.firstChildElement("PathSet");
	if (pathSetNode.isNull())
	{
		output->error(QObject::tr("No RainbowSelective/PathSet node."));
		return 0LLU;
	}
	
	unsigned long long path = 0LLU;

	QDomElement pathElement = pathSetNode.firstChildElement();
	while (!pathElement.isNull())
	{
		if (pathElement.tagName() == "PathBinMask")
		{
			if (pathElement.hasAttribute("mask"))
			{
				bool ok;
				path |= pathElement.attributeNode("mask").value().toULongLong(&ok, 2);
				if (!ok)
					output->error(QObject::tr(
						"RainbowSelective/PathSet/PathBinMask/mask : this is not valid binary mask."));
			}
			else
				output->error(QObject::tr(
					"RainbowSelective/PathSet/PathBinMask : No mask attribute."));
		}
		else if (pathElement.tagName() == "PathHexMask")
		{
			if (pathElement.hasAttribute("mask"))
			{
				bool ok;
				path |= pathElement.attributeNode("mask").value().toULongLong(&ok, 16);
				if (!ok)
					output->error(QObject::tr(
						"RainbowSelective/PathSet/PathHexMask/mask : this is not valid hex mask."));
			}
			else
				output->error(QObject::tr(
					"RainbowSelective/PathSet/PathHexMask : No mask attribute"));
		}
		else if (pathElement.tagName() == "Reflection")
			path |= 1LLU;
		else if (pathElement.tagName() == "Bow")
		{
			if (pathElement.hasAttribute("order"))
			{
				bool ok;
				int order = pathElement.attributeNode("order").value().toInt(&ok);
				if (!ok)
					output->error(QObject::tr(
						"RainbowSelective/PathSet/Bow/order : this is not valid number."));
				else if (order >= 0 && order <= 62)
					path |= (1LLU << (order + 1));
				else
					output->warning(QObject::tr(
						"RainbowSelective/PathSet/Bow/order : Order should be between 0 and 62."));
			}
			else
				output->error(QObject::tr(
					"RainbowSelective/PathSet/Bow : No order attribute."));
		}
		else if (pathElement.tagName() == "Range")
		{
			int min = ( 
				pathElement.hasAttribute("min") ?
				pathElement.attributeNode("min").value().toInt() :
				-1 );
			int max = (
				pathElement.hasAttribute("max") ?
				pathElement.attributeNode("max").value().toInt() :
				62 );
			if (min >= -1 && min <= max && max <= 62)
				path |= ((0xFFFFFFFFFFFFFFFFLLU << (min + 1)) &
					~(0xFFFFFFFFFFFFFFFFLLU << (max + 2)));
		}
		else
			output->warning(QObject::tr(
				"RainbowSelective/PathSet : Unknown path selector : %1.").arg(pathElement.tagName()));

		pathElement = pathElement.nextSiblingElement();
	}

	if (path == 0LLU)
		output->error(QObject::tr(
			"RainbowSelective/PathSet : no path selected."));

	return path;
}

QString ActionDescr::pathSetStringFromDomElement(QDomElement node, ErrorOutput *output)
{
	QDomElement pathSetNode = node.firstChildElement("PathSet");
	if (pathSetNode.isNull())
	{
		output->error(QObject::tr("No CrystalSelectiveAction/PathSet node."));
		return "";
	}

	QDomElement pathSetStringNode = pathSetNode.firstChildElement("PathSetString");
	if (pathSetStringNode.isNull())
	{
		output->error(QObject::tr("No CrystalSelectiveAction/PathSet/PathSetString node."));
		return "";
	}

	if (pathSetStringNode.hasAttribute("string"))
		return pathSetStringNode.attributeNode("string").value();
	else
	{
		output->error(QObject::tr("No CrystalSelectiveAction/PathSet/PathSetString/string attribute."));
		return "";
	}
}

BlueSky::CrystalPathSet *ActionDescr::parsePathSet(int faceCount) const
{
	BlueSky::CrystalPathSet *result = new BlueSky::CrystalPathSet(faceCount);

	QStringList pathList = pathSetString.split(';');
	for (int i = 0; i < pathList.count(); i++)
	{
		QStringList faceList = pathList[i].split(',');
		int *path = new int[faceList.count()];

		for (int j = 0; j < faceList.count(); j++)
		{
			if (faceList[j].trimmed() == "*")
				path[j] = -1;
			else
				path[j] = faceList[j].toInt();
		}

		result->addPath(path, faceList.count());
	}

	return result;
}
