/* ----------------------------------------------------------------------------
This file is part of BlueSky

Copyright (C) 2012 Piotr Polesiuk

Permission is hereby granted, free of charge, to any person obtaining a copy 
of this software and associated documentation files (the "Software"), to deal 
in the Software without restriction, including without limitation the rights 
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
copies of the Software, and to permit persons to whom the Software is 
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in 
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
THE SOFTWARE.
---------------------------------------------------------------------------- */

#include "blueSkyXmlScript.h"

#include <QtGui/QtGui>

#include "simulation.h"
#include "errorOutput.h"

BlueSkyXmlScript::BlueSkyXmlScript(QDomDocument document)
{
	domDocument = document;

	QString description = "";
}

BlueSkyXmlScript::~BlueSkyXmlScript()
{
}
	
bool BlueSkyXmlScript::parse(ErrorOutput *output)
{
	output->info("Interpreting BXS ...");

	QDomElement root = domDocument.documentElement();
	if (root.tagName() != "BXS")
	{
		output->error("This is not BXS script!");
		return false;
	}

	scriptName = root.attributeNode("name").value();

	sunDescr = SunDescr::fromDomElement(root, "Sun", output);
	actionDescr = ActionDescr::fromDomElement(root, "Action", output);
	cameraDescr = CameraDescr::fromDomElement(root, "Camera", output);

	QDomElement descriptionNode = root.firstChildElement("Description");
	if (!descriptionNode.isNull())
	{
		QDomElement htmlDescr = descriptionNode.firstChildElement("HTML");
		if (!htmlDescr.isNull())
		{
			QString descr;
			QTextStream stream(&descr);
			htmlDescr.save(stream, 0);

			description = descr;
		}

		QDomElement briefDescr = descriptionNode.firstChildElement("Brief");
		if (!briefDescr.isNull())
			briefDescription = briefDescr.text();
	}

	if (output->errorCount() == 0)
	{
		output->info("Success!");
		return true;
	}
	else
	{
		output->info(QString("%1 errors!").arg(output->errorCount()));
		return false;
	}
}

float BlueSkyXmlScript::getSunAzimuth()
{
	return sunDescr.getAzimuth();
}

float BlueSkyXmlScript::getSunAltitude()
{
	return sunDescr.getAltitude();
}

const ActionDescr &BlueSkyXmlScript::getActionDescr()
{
	return actionDescr;
}
	
CameraDescr::CameraType BlueSkyXmlScript::getCameraType()
{
	return cameraDescr.getCameraType();
}

int BlueSkyXmlScript::getCameraResolution()
{
	return cameraDescr.getResolution();
}

Simulation *BlueSkyXmlScript::getSimulation(int simThreads)
{
	BlueSky::Vector3f sunDirection = sunDescr.getSunDirection();
	BlueSky::Action *action = actionDescr.getAction();
	if (action == NULL)
		return NULL;
	BlueSky::Camera *camera = cameraDescr.getCamera(sunDirection);
	if (camera == NULL)
	{
		delete action;
		return NULL;
	}

	return new Simulation(sunDirection, action, camera, simThreads);
}
	
QString BlueSkyXmlScript::getDescription()
{
	return description;
}

QString BlueSkyXmlScript::getBriefDescription()
{
	return briefDescription;
}

QString BlueSkyXmlScript::getScriptName()
{
	return scriptName;
}

QString BlueSkyXmlScript::toText()
{
	return domDocument.toString();
}

void BlueSkyXmlScript::setSunDescr(const SunDescr &_sunDescr)
{
	sunDescr = _sunDescr;
	QDomElement rootNode = domDocument.documentElement();
	QDomElement oldSunNode = rootNode.firstChildElement("Sun");
	rootNode.replaceChild(sunDescr.getDomElement(), oldSunNode);
}
	
void BlueSkyXmlScript::setActionDescr(const ActionDescr &_actionDescr)
{
	actionDescr = _actionDescr;
	QDomElement actionNode = domDocument.documentElement().firstChildElement("Action");
	QDomElement oldActionNode = actionNode.firstChildElement();
	actionNode.replaceChild(actionDescr.getDomElement(), oldActionNode);
}
	
void BlueSkyXmlScript::setCameraDescr(const CameraDescr &_cameraDescr)
{
	cameraDescr = _cameraDescr;
	QDomElement cameraNode = domDocument.documentElement().firstChildElement("Camera");
	QDomElement oldCameraNode = cameraNode.firstChildElement();
	cameraNode.replaceChild(cameraDescr.getDomElement(), oldCameraNode);
}

BlueSkyXmlScript *BlueSkyXmlScript::fromText(QString text, ErrorOutput *output)
{
	output->reset();
	output->info("Parsing XML ...");
	QString error;
	int err_line, err_col;
	QDomDocument doc("BXS");

	if (!doc.setContent(text, &error, &err_line, &err_col))
	{
		output->error(QObject::tr("line %1, column %2 : %3")
			.arg(err_line).arg(err_col).arg(error));
		return NULL;
	}

	return new BlueSkyXmlScript(doc);
}

BlueSkyXmlScript *BlueSkyXmlScript::fromFile(QString path, ErrorOutput *output)
{
	output->reset();
	
	QFile file(path);
	if (!file.open(QFile::ReadOnly | QFile::Text))
	{
		output->error(QObject::tr("Can not read file %1:\n%2.")
			.arg(path).arg(file.errorString()));
		return NULL;
	}

	QTextStream in(&file);
	in.setCodec("UTF-8");
	QString text = in.readAll();

	return fromText(text, output);
}
