/* ----------------------------------------------------------------------------
This file is part of BlueSky

Copyright (C) 2012 Piotr Polesiuk

Permission is hereby granted, free of charge, to any person obtaining a copy 
of this software and associated documentation files (the "Software"), to deal 
in the Software without restriction, including without limitation the rights 
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
copies of the Software, and to permit persons to whom the Software is 
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in 
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
THE SOFTWARE.
---------------------------------------------------------------------------- */

#include "cameraDescr.h"

#include "errorOutput.h"
#include <QtCore/QObject>

#include <BSLinearCamera.h>
#include <BSSkyBoxCamera.h>

CameraDescr::CameraDescr()
{
	type       = LinearCamera;
	resolution = 1024;
}

CameraDescr::CameraDescr(const CameraDescr &cameraDescr)
{
	type       = cameraDescr.type;
	resolution = cameraDescr.resolution;
}

CameraDescr::~CameraDescr()
{
}

const CameraDescr &CameraDescr::operator = (const CameraDescr &cameraDescr)
{
	type       = cameraDescr.type;
	resolution = cameraDescr.resolution;
	return *this;
}

CameraDescr::CameraType CameraDescr::getCameraType() const
{
	return type;
}

void CameraDescr::setCameraType(CameraType _type)
{
	type = _type;
}

int CameraDescr::getResolution() const
{
	return resolution;
}

void CameraDescr::setResolution(int _resolution)
{
	resolution = _resolution;
}

BlueSky::Camera *CameraDescr::getCamera(BlueSky::Vector3f axis) const
{
	switch (type)
	{
	case LinearCamera:
		return new BlueSky::LinearCamera(resolution, axis);
	case SkyBoxCamera:
		return new BlueSky::SkyBoxCamera(resolution);
	}

	return NULL;
}

QDomElement CameraDescr::getDomElement() const
{
	QDomDocument doc;
	QDomElement result;

	switch (type)
	{
	case LinearCamera:
		result = doc.createElement("LinearCamera");
		break;
	case SkyBoxCamera:
		result = doc.createElement("SkyBoxCamera");
		break;
	default:
		result = doc.createElement("Unknown");
		break;
	}

	result.setAttribute("resolution", resolution);

	return result;
}

CameraDescr CameraDescr::fromDomElement(QDomElement node, QString nodeName, ErrorOutput *output)
{
	CameraDescr result;
	QDomElement cameraNode = (nodeName.isEmpty() ? node : node.firstChildElement(nodeName));

	if (cameraNode.isNull())
	{
		output->error(QObject::tr("No %1 node!").arg(nodeName));
		return result;
	}

	cameraNode = cameraNode.firstChildElement();
	if (cameraNode.isNull())
	{
		output->error(QObject::tr("No camera specified."));
		return result;
	}

	if (cameraNode.tagName() == "LinearCamera")
		result.type = LinearCamera;
	else if (cameraNode.tagName() == "SkyBoxCamera")
		result.type = SkyBoxCamera;
	else
	{
		output->error(QObject::tr("Unknown camera : %1").arg(cameraNode.tagName()));
	}

	if (cameraNode.hasAttribute("resolution"))
	{
		bool ok;
		result.resolution = cameraNode.attributeNode("resolution").value().toInt(&ok);
		if (!ok)
			output->error(QObject::tr("Camera/%1/resolution : this is not valid number.")
				.arg(cameraNode.tagName()));
		if (result.resolution < 1)
		{
			output->warning(QObject::tr("Camera resolution should be greater than 0."));
			result.resolution = 1024;
		}

		switch (result.type)
		{
		case LinearCamera:
			if (result.resolution > 1000000)
			{
				output->warning(QObject::tr("LinearCamera resolution should be less or equal 1000000."));
				result.resolution = 1000000;
			}
			break;
		case SkyBoxCamera:
			if (result.resolution > 10000)
			{
				output->warning(QObject::tr("SkyBoxCamera resolution should be less or equal 10000."));
				result.resolution = 10000;
			}
			break;
		}
	}

	return result;
}
