/* ----------------------------------------------------------------------------
This file is part of BlueSky

Copyright (C) 2012 Piotr Polesiuk

Permission is hereby granted, free of charge, to any person obtaining a copy 
of this software and associated documentation files (the "Software"), to deal 
in the Software without restriction, including without limitation the rights 
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
copies of the Software, and to permit persons to whom the Software is 
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in 
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
THE SOFTWARE.
---------------------------------------------------------------------------- */

#include "crystalDescr.h"

#include <QtCore/QObject>
#include "errorOutput.h"

CrystalDescr::CrystalDescr()
{
	type = Prism;
	joiningFacesCount = 6;
	elongation = 0.5f;
}

CrystalDescr::CrystalDescr(const CrystalDescr &crystalDescr)
{
	type = crystalDescr.type;
	joiningFacesCount = crystalDescr.joiningFacesCount;
	elongation = crystalDescr.elongation;
}

CrystalDescr::~CrystalDescr()
{
}

const CrystalDescr &CrystalDescr::operator = (const CrystalDescr &crystalDescr)
{
	type = crystalDescr.type;
	joiningFacesCount = crystalDescr.joiningFacesCount;
	elongation = crystalDescr.elongation;
	return *this;
}

CrystalDescr::CrystalType CrystalDescr::getCrystalType() const
{
	return type;
}

void CrystalDescr::setCrystalType(CrystalDescr::CrystalType _type)
{
	type = _type;
}

int CrystalDescr::getJoiningFacesCount() const
{
	return joiningFacesCount;
}

void CrystalDescr::setJoiningFacesCount(int _joiningFacesCount)
{
	joiningFacesCount = _joiningFacesCount;
}
	
float CrystalDescr::getElongation() const
{
	return elongation;
}

void CrystalDescr::setElongation(float _elongation)
{
	elongation = _elongation;
}

BlueSky::Crystal *CrystalDescr::getCrystal() const
{
	switch (type)
	{
	case Diamond:
		return BlueSky::Crystal::createDiamond(0.57735f, 0.57735f, 0.57735f, 0.57735f);
	case Prism:
		{
			float d = BlueSky::Math::sqrt(
				BlueSky::Math::sqr(elongation) + BlueSky::Math::sqr(1.0f - elongation));
			return BlueSky::Crystal::createPrism(
				joiningFacesCount,
				2.0f * elongation / d, 
				(1.0f - elongation) / d, 
				0.0f, 
				0.0f);
		}
	}

	return NULL;
}

QDomElement CrystalDescr::getDomElement() const
{
	QDomDocument doc;
	QDomElement result;

	switch (type)
	{
	case Diamond:
		result = doc.createElement("Diamond");
		break;
	case Prism:
		result = doc.createElement("Prism");
		result.setAttribute("sideCount", joiningFacesCount);
		result.setAttribute("elongation", elongation);
		break;
	}

	return result;
}

QString CrystalDescr::getCrystalName() const
{
	switch (type)
	{
	case Diamond:
		return QObject::tr("Octahedron");
	case Prism:
		return QObject::tr("Prism (%1-gonal)").arg(joiningFacesCount);
	}

	return QObject::tr("Unknown");
}

CrystalDescr CrystalDescr::fromDomElement(QDomElement node, QString nodeName, ErrorOutput *output)
{
	CrystalDescr result;
	QDomElement crystalNode = (nodeName.isEmpty() ? node : node.firstChildElement(nodeName));

	if (crystalNode.isNull())
	{
		output->error(QObject::tr("No %1 node!").arg(nodeName));
		return result;
	}

	crystalNode = crystalNode.firstChildElement();
	if (crystalNode.isNull())
	{
		output->error(QObject::tr("No crystal specified."));
		return result;
	}

	if (crystalNode.tagName() == "Diamond")
		result.type = Diamond;
/*	else if (crystalNode.tagName() == "PlaneList")
	{
		BlueSky::Crystal *crystal = new BlueSky::Crystal();

		QDomElement planeNode = crystalNode.firstChildElement("Plane");
		while (!planeNode.isNull())
		{
			float nx = planeNode.attributeNode("nx").value().toFloat();
			float ny = planeNode.attributeNode("ny").value().toFloat();
			float nz = planeNode.attributeNode("nz").value().toFloat();
			float distance = planeNode.attributeNode("distance").value().toFloat();

			if (distance <= 0.0f)
				distance = 0.2f;

			BlueSky::Vector3f normal = BlueSky::Vector3f(nx, ny, nz);
			if (normal == BlueSky::Vector3f::ZERO)
				normal = BlueSky::Vector3f::UNIT_X;

			crystal->addPlane(normal.normalised(), distance);

			planeNode = planeNode.nextSiblingElement("Plane");
		}

		return crystal;
	}*/
	else if (crystalNode.tagName() == "Prism")
	{
		result.type = Prism;
		if (crystalNode.hasAttribute("sideCount") && crystalNode.hasAttribute("elongation"))
		{
			bool ok;
			result.joiningFacesCount = crystalNode.attributeNode("sideCount").value().toInt(&ok);
			if (!ok)
				output->error(QObject::tr(
					"Prism/sideCount : this is not valid number."));
			else if (result.joiningFacesCount <= 2)
			{
				output->warning(QObject::tr(
					"Prism/sideCount should be greater than 2."));
				result.joiningFacesCount = 6;
			}

			result.elongation = crystalNode.attributeNode("elongation").value().toFloat(&ok);
			if (!ok)
				output->error(QObject::tr(
					"Prism/elongation : this is not valid number."));
			else if (result.elongation < 0.1f || result.elongation > 0.9f)
			{
				output->warning(QObject::tr(
					"Prism/elongation should be between 0.1 and 0.9"));
				result.elongation = 0.5f;
			}
		}
		else
			output->error(QObject::tr(
				"No Prism/sideCount or Prism/elongation attribute."));
	}
/*	else if (crystalNode.tagName() == "Rotated")
	{
		if (crystalNode.hasAttribute("angle"))
		{
			float angle = crystalNode.attributeNode("angle").value().toFloat();
			float axis_x = crystalNode.attributeNode("axis_x").value().toFloat();
			float axis_y = crystalNode.attributeNode("axis_y").value().toFloat();
			float axis_z = crystalNode.attributeNode("axis_z").value().toFloat();

			BlueSky::Vector3f axis = BlueSky::Vector3f(axis_x, axis_y, axis_z);
			if (axis == BlueSky::Vector3f::ZERO)
				axis = BlueSky::Vector3f::UNIT_Y;

			BlueSky::Quaternionf rot;
			rot.fromAngleAxis(angle, axis.normalised());

			BlueSky::Crystal *baseCrystal = getCrystal(crystalNode, "");
			if (baseCrystal)
			{
				baseCrystal->rotate(rot);
				return baseCrystal;
			}
			else
				return NULL;
		}
		else
		{
			float w = crystalNode.attributeNode("w").value().toFloat();
			float x = crystalNode.attributeNode("x").value().toFloat();
			float y = crystalNode.attributeNode("y").value().toFloat();
			float z = crystalNode.attributeNode("z").value().toFloat();

			BlueSky::Quaternionf rot = BlueSky::Quaternionf(w, x, y, z);
			if (rot == BlueSky::Quaternionf::ZERO)
				rot = BlueSky::Quaternionf::UNIT;

			BlueSky::Crystal *baseCrystal = getCrystal(crystalNode, "");
			if (baseCrystal)
			{
				baseCrystal->rotate(rot);
				return baseCrystal;
			}
			else
				return NULL;
		}
	}*/
	else
		output->error(QObject::tr("Unknown crystal type : %1").arg(crystalNode.tagName()));

	return result;
}
