/* ----------------------------------------------------------------------------
This file is part of BlueSky

Copyright (C) 2012 Piotr Polesiuk

Permission is hereby granted, free of charge, to any person obtaining a copy 
of this software and associated documentation files (the "Software"), to deal 
in the Software without restriction, including without limitation the rights 
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
copies of the Software, and to permit persons to whom the Software is 
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in 
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
THE SOFTWARE.
---------------------------------------------------------------------------- */

#include "displaySettingsWidget.h"

#include <QtGui/QtGui>
#include "skyView.h"

DisplaySettingsWidget::DisplaySettingsWidget(SkyView *_skyView)
{
	skyView = _skyView;

	dynBrightnessCheckBox = new QCheckBox(tr("Dynamic brightness"));
	connect(dynBrightnessCheckBox, SIGNAL(toggled(bool)), 
		this, SLOT(dynBrightnessToggled(bool)));
	dynBrightnessCheckBox->setChecked(false);

	useGammaCheckBox = new QCheckBox(tr("Gamma correction"));
	connect(useGammaCheckBox, SIGNAL(toggled(bool)),
		this, SLOT(useGammaToggled(bool)));
	useGammaCheckBox->setChecked(false);

	stereographicCheckBox = new QCheckBox(tr("Stereographic"));
	connect(stereographicCheckBox, SIGNAL(toggled(bool)),
		this, SLOT(stereographicToggled(bool)));
	stereographicCheckBox->setChecked(false);

	brightnessLabel = new QLabel(tr("Brightness"));
	gammaLabel = new QLabel(tr("Gamma"));
	gammaLabel->setEnabled(false);
	backgroundLabel = new QLabel(tr("Background brightness"));
	horizonLabel = new QLabel(tr("Horizon opacity"));

	brightnessSlider = new QSlider(Qt::Horizontal);
	connect(brightnessSlider, SIGNAL(valueChanged(int)),
		this, SLOT(brightnessValueChanged(int)));
	brightnessSlider->setRange(0, 1000);
	brightnessSlider->setValue(500);

	gammaSlider = new QSlider(Qt::Horizontal);
	connect(gammaSlider, SIGNAL(valueChanged(int)),
		this, SLOT(gammaValueChanged(int)));
	gammaSlider->setRange(0, 1000);
	gammaSlider->setValue(500);
	gammaSlider->setEnabled(false);

	backgroundSlider = new QSlider(Qt::Horizontal);
	connect(backgroundSlider, SIGNAL(valueChanged(int)),
		this, SLOT(backgroundValueChanged(int)));
	backgroundSlider->setRange(0, 1000);
	backgroundSlider->setValue(500);

	horizonSlider = new QSlider(Qt::Horizontal);
	connect(horizonSlider, SIGNAL(valueChanged(int)),
		this, SLOT(horizonValueChanged(int)));
	horizonSlider->setRange(0, 1000);
	horizonSlider->setValue(1000);

	QGridLayout *layout = new QGridLayout;
	layout->addWidget(dynBrightnessCheckBox, 0, 0);
	layout->addWidget(useGammaCheckBox, 1, 0);
	layout->addWidget(stereographicCheckBox, 2, 0);
	layout->addWidget(brightnessLabel, 0, 1);
	layout->addWidget(gammaLabel, 1, 1);
	layout->addWidget(backgroundLabel, 2, 1);
	layout->addWidget(horizonLabel, 3, 1);
	layout->addWidget(brightnessSlider, 0, 2);
	layout->addWidget(gammaSlider, 1, 2);
	layout->addWidget(backgroundSlider, 2, 2);
	layout->addWidget(horizonSlider, 3, 2);
	setLayout(layout);
}

void DisplaySettingsWidget::setOrientation(bool horizontal)
{
	QGridLayout *newLayout = new QGridLayout;

	if (horizontal)
	{
		brightnessSlider->setOrientation(Qt::Horizontal);
		gammaSlider->setOrientation(Qt::Horizontal);
		backgroundSlider->setOrientation(Qt::Horizontal);
		horizonSlider->setOrientation(Qt::Horizontal);
	
		brightnessLabel->setText(tr("Brightness"));
		gammaLabel->setText(tr("Gamma"));
		backgroundLabel->setText(tr("Background brightness"));
		horizonLabel->setText(tr("Horizon opacity"));

		newLayout->addWidget(dynBrightnessCheckBox, 0, 0);
		newLayout->addWidget(useGammaCheckBox, 1, 0);
		newLayout->addWidget(stereographicCheckBox, 2, 0);
		newLayout->addWidget(brightnessLabel, 0, 1);
		newLayout->addWidget(gammaLabel, 1, 1);
		newLayout->addWidget(backgroundLabel, 2, 1);
		newLayout->addWidget(horizonLabel, 3, 1);
		newLayout->addWidget(brightnessSlider, 0, 2);
		newLayout->addWidget(gammaSlider, 1, 2);
		newLayout->addWidget(backgroundSlider, 2, 2);
		newLayout->addWidget(horizonSlider, 3, 2);

		delete layout();
		setLayout(newLayout);
	}
	else
	{
		brightnessSlider->setOrientation(Qt::Vertical);
		gammaSlider->setOrientation(Qt::Vertical);
		backgroundSlider->setOrientation(Qt::Vertical);
		horizonSlider->setOrientation(Qt::Vertical);

		brightnessLabel->setText(tr("B"));
		gammaLabel->setText(tr("G"));
		backgroundLabel->setText(tr("Bk"));
		horizonLabel->setText(tr("H"));

		newLayout->addWidget(dynBrightnessCheckBox, 0, 0, 1, 4);
		newLayout->addWidget(useGammaCheckBox, 1, 0, 1, 4);
		newLayout->addWidget(stereographicCheckBox, 2, 0, 1, 4);
		newLayout->addWidget(brightnessLabel, 3, 0);
		newLayout->addWidget(gammaLabel, 3, 1);
		newLayout->addWidget(backgroundLabel, 3, 2);
		newLayout->addWidget(horizonLabel, 3, 3);
		newLayout->addWidget(brightnessSlider, 4, 0);
		newLayout->addWidget(gammaSlider, 4, 1);
		newLayout->addWidget(backgroundSlider, 4, 2);
		newLayout->addWidget(horizonSlider, 4, 3);

		delete layout();
		setLayout(newLayout);
	}
}

void DisplaySettingsWidget::dynBrightnessToggled(bool checked)
{
	skyView->setUseDynamicBrightness(checked);
}

void DisplaySettingsWidget::useGammaToggled(bool checked)
{
	gammaLabel->setEnabled(checked);
	gammaSlider->setEnabled(checked);
	skyView->setUseGamma(checked);
}

void DisplaySettingsWidget::stereographicToggled(bool checked)
{
	skyView->setStereographic(checked);
}

void DisplaySettingsWidget::brightnessValueChanged(int value)
{
	skyView->setBrightness(pow((float)value, 4.0f) / 6.25e10f);
}

void DisplaySettingsWidget::gammaValueChanged(int value)
{
	skyView->setGamma((float)value / 500.0f);
}

void DisplaySettingsWidget::backgroundValueChanged(int value)
{
	skyView->setBackgroundBrightness(pow((float)value, 4.0f) / 6.25e11f);
}

void DisplaySettingsWidget::horizonValueChanged(int value)
{
	skyView->setHorizonOpacity((float)value / 1000.0f);
}
