/* ----------------------------------------------------------------------------
This file is part of BlueSky

Copyright (C) 2012 Piotr Polesiuk

Permission is hereby granted, free of charge, to any person obtaining a copy 
of this software and associated documentation files (the "Software"), to deal 
in the Software without restriction, including without limitation the rights 
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
copies of the Software, and to permit persons to whom the Software is 
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in 
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
THE SOFTWARE.
---------------------------------------------------------------------------- */

#include "editActionDialog.h"

#include <QtGui/QtGui>
#include <typeinfo>

#include <BSIdentityAction.h>
#include <BSRainbow.h>

#include "editCrystalDialog.h"

EditActionDialog::EditActionDialog()
{
	createElements();
}

EditActionDialog::EditActionDialog(const ActionDescr &actionDescr)
{
	createElements();

	switch (actionDescr.getActionType())
	{
	case ActionDescr::IdentityAction:
		emit actionTypeComboBox->setCurrentIndex(0);
		break;
	case ActionDescr::Rainbow:
		emit actionTypeComboBox->setCurrentIndex(2);
		setRefractionDescr(actionDescr.getRefractionDescr());
		break;
	case ActionDescr::RainbowSelective:
		emit actionTypeComboBox->setCurrentIndex(3);
		setRefractionDescr(actionDescr.getRefractionDescr());
		pathLineEdit->setText(actionDescr.getRSPathString());
		break;
	case ActionDescr::CrystalAction:
		emit actionTypeComboBox->setCurrentIndex(5);
		setRefractionDescr(actionDescr.getRefractionDescr());
		setCrystalDescr(actionDescr.getCrystalDescr());
		setRotationDescr(actionDescr.getRotationDescr());
		break;
	case ActionDescr::CrystalSelectiveAction:
		emit actionTypeComboBox->setCurrentIndex(6);
		setRefractionDescr(actionDescr.getRefractionDescr());
		setCrystalDescr(actionDescr.getCrystalDescr());
		setRotationDescr(actionDescr.getRotationDescr());
		pathLineEdit->setText(actionDescr.getPathSetString());
		break;
	default:
		QMessageBox msgbox;
		msgbox.setText(tr("This action can not be edited."));
		msgbox.setIcon(QMessageBox::Warning);
		msgbox.exec();
	}
}

void EditActionDialog::okButtonPush()
{
	accept();
}

void EditActionDialog::editCrystalPush()
{
	EditCrystalDialog dialog(crystalDescr, rotationDescr);

	if (dialog.exec())
	{
		crystalLabel->setText(dialog.getCrystalName());
		crystalDescr = dialog.getCrystalDescr();
		rotationDescr = dialog.getRotationDescr();
	}
}

void EditActionDialog::actionTypeChanged(int index)
{
	switch (index)
	{
	case 0:
		descriptionLabel->setText(
			tr("Identity action returns an unchanged photon. "
			"It can be used to draw the sun."));
		refractionGroupBox->setEnabled(false);
		pathGroupBox->setEnabled(false);
		crystalGroupBox->setEnabled(false);
		break;
	case 2:
		descriptionLabel->setText(
			tr("Rainbow action represents interactions with the perfectly "
			"spherical drops of water or other liquid. It is used to simulate "
			"the rainbow."));
		refractionGroupBox->setEnabled(true);
		pathGroupBox->setEnabled(false);
		crystalGroupBox->setEnabled(false);
		materialComboBox->setCurrentIndex(2);
		predefinedMaterialRadioButton->setChecked(true);
		break;
	case 3:
		descriptionLabel->setText(
			tr("This action allows to simulate and draw rainbows of selected "
			"orders. Order of rainbow is a number of internal reflections of "
			"ray of light inside the drop of water. Commonly observed "
			"rainbows are first and second order rainbows.\n"
			"To specify which rainbows you want to draw, enter their orders "
			"into 'Selected paths' field (0 is also valid order) separated "
			"by comma character (','). Letter 'R' "
			"means that you want to simulate external reflexes. You can also "
			"specify range of orders by entering minimum and maximum value "
			"separated by '-'. e.g. \"R, 1, 4-6\" means that you want to "
			"simulate external reflexes and rainbows of 1st, 4th, 5th and "
			"6th order. Maximum order is a 62."));
		refractionGroupBox->setEnabled(true);
		pathGroupBox->setEnabled(true);
		crystalGroupBox->setEnabled(false);
		materialComboBox->setCurrentIndex(2);
		predefinedMaterialRadioButton->setChecked(true);
		break;
	case 5:
		descriptionLabel->setText(
			tr("This action allows to simulate complex phenomena arising in "
			"the crystals of ice or other substances. You can simulate halo, "
			"sun dogs and many others."));
		refractionGroupBox->setEnabled(true);
		pathGroupBox->setEnabled(false);
		crystalGroupBox->setEnabled(true);
		materialComboBox->setCurrentIndex(1);
		predefinedMaterialRadioButton->setChecked(true);
		break;
	case 6:
		descriptionLabel->setText(
			tr("This action allows to select one or more paths in crystal "
			"which are simulated. To specify path, enter numbers of faces "
			"in crystal which light ray visits, separated by comma (','). "
			"You can input many path, separating them by semicolon (';'). "
			"Asterisk character ('*') will be matched with all faces.\n"
			"In prism crystal first two faces (0 and 1) are bases of prism.\n"
			"E. g. if you want to select all paths with one internal "
			"reflection started in base of prism, just insert \"0,*,*;1,*,*\" "
			"into 'Selected paths' field."));
		refractionGroupBox->setEnabled(true);
		pathGroupBox->setEnabled(true);
		crystalGroupBox->setEnabled(true);
		materialComboBox->setCurrentIndex(1);
		predefinedMaterialRadioButton->setChecked(true);
		break;
	default:
		descriptionLabel->setText(tr(""));
		break;
	}
}

void EditActionDialog::constRefractionToggled(bool checked)
{
	constRefractionSpinBox->setEnabled(checked);
}

void EditActionDialog::powerRefractionToggled(bool checked)
{
	powerRefractionGroupBox->setEnabled(checked);
}

void EditActionDialog::predefinedMaterialToggled(bool checked)
{
	materialComboBox->setEnabled(checked);
}

void EditActionDialog::createElements()
{
	createRefractionGroupBox();
	createCrystalGroupBox();

	descriptionLabel = new QLabel();
	descriptionLabel->setWordWrap(true);
	descriptionLabel->setAlignment(Qt::AlignTop);

	actionTypeComboBox = new QComboBox();
	actionTypeComboBox->addItem(tr("Identity action"));
	actionTypeComboBox->addItem(tr("Rainbow"));
	actionTypeComboBox->addItem(tr("Selected rainbows"));
	actionTypeComboBox->addItem(tr("Crystal"));
	actionTypeComboBox->addItem(tr("Crystal (selected paths)"));
	actionTypeComboBox->insertSeparator(1);
	actionTypeComboBox->insertSeparator(4);
	connect(actionTypeComboBox, SIGNAL(currentIndexChanged(int)), 
		this, SLOT(actionTypeChanged(int)));

	pathGroupBox = new QGroupBox("Selected paths");
	pathLineEdit = new QLineEdit();
	pathGroupBox->setLayout(new QVBoxLayout);
	pathGroupBox->layout()->addWidget(pathLineEdit);

	okButton = new QPushButton(tr("Ok"));
	connect(okButton, SIGNAL(pressed()), this, SLOT(okButtonPush()));
	cancelButton = new QPushButton(tr("Cancel"));
	connect(cancelButton, SIGNAL(pressed()), this, SLOT(reject()));

	QWidget *buttonBar = new QWidget();
	QGridLayout *butLayout = new QGridLayout;
	butLayout->addWidget(okButton, 0, 1);
	butLayout->addWidget(cancelButton, 0, 2);
	butLayout->setColumnStretch(0, 1);
	buttonBar->setLayout(butLayout);
	
	QGridLayout *layout = new QGridLayout;
	layout->addWidget(actionTypeComboBox, 0, 0);
	layout->addWidget(refractionGroupBox, 1, 0);
	layout->addWidget(pathGroupBox, 2, 0);
	layout->addWidget(crystalGroupBox, 3, 0);
	layout->addWidget(descriptionLabel, 0, 1, 4, 1);
	layout->addWidget(buttonBar, 4, 0, 1, 2);
	setLayout(layout);

	actionTypeChanged(0);
}

void EditActionDialog::createRefractionGroupBox()
{
	refractionGroupBox = new QGroupBox(tr("Refractive Index"));

	constRefractionRadioButton    = new QRadioButton(tr("Constant refractive index."));
	powerRefractionRadioButton    = new QRadioButton(tr("Power function approximation."));
	predefinedMaterialRadioButton = new QRadioButton(tr("Predefined material."));

	connect(constRefractionRadioButton, SIGNAL(toggled(bool)), 
		this, SLOT(constRefractionToggled(bool)));
	connect(powerRefractionRadioButton, SIGNAL(toggled(bool)),
		this, SLOT(powerRefractionToggled(bool)));
	connect(predefinedMaterialRadioButton, SIGNAL(toggled(bool)),
		this, SLOT(predefinedMaterialToggled(bool)));

	constRefractionSpinBox = new QDoubleSpinBox();
	constRefractionSpinBox->setDecimals(5);
	constRefractionSpinBox->setRange(-1000.0, 1000.0);
	constRefractionSpinBox->setSingleStep(0.1);
	constRefractionSpinBox->setValue(1.333);
	constRefractionSpinBox->setEnabled(false);
	
	powerRefractionGroupBox = new QGroupBox();
	powerRefractionGroupBox->setFlat(true);

	redRefractionSpinBox = new QDoubleSpinBox();
	redRefractionSpinBox->setDecimals(5);
	redRefractionSpinBox->setRange(-1000.0, 1000.0);
	redRefractionSpinBox->setSingleStep(0.1);
	redRefractionSpinBox->setValue(1.331);
	
	greenRefractionSpinBox = new QDoubleSpinBox();
	greenRefractionSpinBox->setDecimals(5);
	greenRefractionSpinBox->setRange(-1000.0, 1000.0);
	greenRefractionSpinBox->setSingleStep(0.1);
	greenRefractionSpinBox->setValue(1.333);

	blueRefractionSpinBox = new QDoubleSpinBox();
	blueRefractionSpinBox->setDecimals(5);
	blueRefractionSpinBox->setRange(-1000.0, 1000.0);
	blueRefractionSpinBox->setSingleStep(0.1);
	blueRefractionSpinBox->setValue(1.339);

	QLabel *redLabel   = new QLabel(tr("red (700nm)"));
	QLabel *greenLabel = new QLabel(tr("green (550nm)"));
	QLabel *blueLabel  = new QLabel(tr("blue (400nm)"));

	QGridLayout *prLayout = new QGridLayout;
	prLayout->addWidget(redLabel, 0, 0);
	prLayout->addWidget(greenLabel, 1, 0);
	prLayout->addWidget(blueLabel, 2, 0);
	prLayout->addWidget(redRefractionSpinBox, 0, 1);
	prLayout->addWidget(greenRefractionSpinBox, 1, 1);
	prLayout->addWidget(blueRefractionSpinBox, 2, 1);
	powerRefractionGroupBox->setLayout(prLayout);
	powerRefractionGroupBox->setEnabled(false);

	materialComboBox = new QComboBox();
	materialComboBox->addItem(tr("Diamond"));
	materialComboBox->addItem(tr("Ice"));
	materialComboBox->addItem(tr("Water"));
	materialComboBox->setEnabled(false);

	QGridLayout *layout = new QGridLayout;
	layout->addWidget(constRefractionRadioButton,    0, 0);
	layout->addWidget(powerRefractionRadioButton,    1, 0);
	layout->addWidget(powerRefractionGroupBox,       2, 0, 1, 2);
	layout->addWidget(predefinedMaterialRadioButton, 3, 0);

	layout->addWidget(constRefractionSpinBox, 0, 1);
	layout->addWidget(materialComboBox, 3, 1);

	refractionGroupBox->setLayout(layout);
}

void EditActionDialog::createCrystalGroupBox()
{
	crystalGroupBox = new QGroupBox(tr("Crystal"));

	crystalLabel = new QLabel(tr("Prism (6-gonal)"));
	editCrystalButton = new QPushButton(tr("Edit crystal"));
	connect(editCrystalButton, SIGNAL(pressed()), this, SLOT(editCrystalPush()));

	QGridLayout *layout = new QGridLayout;
	layout->addWidget(crystalLabel, 0, 0);
	layout->addWidget(editCrystalButton, 0, 1);
	crystalGroupBox->setLayout(layout);
}

QString EditActionDialog::getActionName()
{
	switch (actionTypeComboBox->currentIndex())
	{
	case 0:
		return tr("Identity action");
	case 2:
		return tr("Rainbow");
	case 3:
		return tr("Selected rainbows");
	case 5:
		return tr("Crystal");
	case 6:
		return tr("Crystal (selected paths)");
	default:
		return "";
	}
}

ActionDescr EditActionDialog::getActionDescr()
{
	ActionDescr result;

	switch (actionTypeComboBox->currentIndex())
	{
	case 0:
		result.setActionType(ActionDescr::IdentityAction);
		break;
	case 2:
		{
			result.setActionType(ActionDescr::Rainbow);
			RefractionDescr refr = getRefractionDescr();
			result.setRefractionDescr(&refr);
			break;
		}
	case 3:
		{
			result.setActionType(ActionDescr::RainbowSelective);
			RefractionDescr refr = getRefractionDescr();
			result.setRefractionDescr(&refr);
			result.setRSPathString(pathLineEdit->text());
			break;
		}
	case 5:
		{
			result.setActionType(ActionDescr::CrystalAction);
			RefractionDescr refr = getRefractionDescr();
			result.setRefractionDescr(&refr);
			result.setCrystalDescr(&crystalDescr);
			result.setRotationDescr(&rotationDescr);
			break;
		}
	case 6:
		{
			result.setActionType(ActionDescr::CrystalSelectiveAction);
			RefractionDescr refr = getRefractionDescr();
			result.setRefractionDescr(&refr);
			result.setCrystalDescr(&crystalDescr);
			result.setRotationDescr(&rotationDescr);
			result.setPathSetString(pathLineEdit->text());
			break;
		}
	default:
		break;
	}

	return result;
}

RefractionDescr EditActionDialog::getRefractionDescr()
{
	RefractionDescr result;
	if (constRefractionRadioButton->isChecked())
	{
		result.setRefractionType(RefractionDescr::RefractionConst);
		result.setConstRefractiveIndex(constRefractionSpinBox->value());
	}
	else if (powerRefractionRadioButton->isChecked())
	{
		result.setRefractionType(RefractionDescr::RefractionPower);
		result.setPowerRIRgb(
			redRefractionSpinBox->value(),
			greenRefractionSpinBox->value(),
			blueRefractionSpinBox->value());
	}
	else if (predefinedMaterialRadioButton->isChecked())
	{
		result.setRefractionType(RefractionDescr::PredefinedMaterial);
		result.setMaterial((BlueSky::RefractionPower::Material)materialComboBox->currentIndex());
	}

	return result;
}

void EditActionDialog::setRefractionDescr(RefractionDescr *refractionDescr)
{
	if (refractionDescr == NULL)
		return;

	switch (refractionDescr->getRefractionType())
	{
	case RefractionDescr::RefractionConst:
		constRefractionRadioButton->setChecked(true);
		constRefractionSpinBox->setValue(refractionDescr->getConstRefractiveIndex());
		break;
	case RefractionDescr::RefractionPower:
		powerRefractionRadioButton->setChecked(true);
		redRefractionSpinBox->setValue(refractionDescr->getPowerRIRed());
		greenRefractionSpinBox->setValue(refractionDescr->getPowerRIGreen());
		blueRefractionSpinBox->setValue(refractionDescr->getPowerRIBlue());
		break;
	case RefractionDescr::PredefinedMaterial:
		predefinedMaterialRadioButton->setChecked(true);
		materialComboBox->setCurrentIndex((int)refractionDescr->getMaterial());
		break;
	}
}

void EditActionDialog::setCrystalDescr(CrystalDescr *_crystalDescr)
{
	if (_crystalDescr == NULL)
		return;

	crystalDescr = *_crystalDescr;
	crystalLabel->setText(crystalDescr.getCrystalName());
}

void EditActionDialog::setRotationDescr(RotationDescr *_rotationDescr)
{
	if (_rotationDescr == NULL)
		return;

	rotationDescr = *_rotationDescr;
}
