/* ----------------------------------------------------------------------------
This file is part of BlueSky

Copyright (C) 2012 Piotr Polesiuk

Permission is hereby granted, free of charge, to any person obtaining a copy 
of this software and associated documentation files (the "Software"), to deal 
in the Software without restriction, including without limitation the rights 
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
copies of the Software, and to permit persons to whom the Software is 
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in 
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
THE SOFTWARE.
---------------------------------------------------------------------------- */

#include "editCrystalDialog.h"

#include <QtGui/QtGui>

#include "editRotationDialog.h"
#include "probabilitySpinBoxDelegate.h"

EditCrystalDialog::EditCrystalDialog(
	const CrystalDescr &crystalDescr, 
	const RotationDescr &rotationDescr)
{
	createElements();

	switch (crystalDescr.getCrystalType())
	{
	case CrystalDescr::Diamond:
		crystalShapeComboBox->setCurrentIndex(0);
		break;
	case CrystalDescr::Prism:
		crystalShapeComboBox->setCurrentIndex(1);
		joiningFacesSpinBox->setValue(crystalDescr.getJoiningFacesCount());
		elongationSpinBox->setValue(crystalDescr.getElongation());
		break;
	default:
		QMessageBox::warning(this, tr("Edit crystal"), tr("This crystal can not be edited."));
	}

	if (rotationDescr.getRotationType() == RotationDescr::RotationAlternative)
	{
		if (rotationDescr.getDefaultRotationDescr())
		{
			defaultRotationDescr = *rotationDescr.getDefaultRotationDescr();
			defaultRotationLabel->setText(defaultRotationDescr.getRotationName());
		}

		for (int i = 0; i < rotationDescr.getAdditionalRotationCount(); i++)
			addRotation(
				rotationDescr.getAdditionalRotationDescr(i),
				rotationDescr.getAdditionalRotationProbability(i));
	}
	else
	{
		defaultRotationDescr = rotationDescr;
		defaultRotationLabel->setText(rotationDescr.getRotationName());
	}
}

void EditCrystalDialog::crystalShapeChanged(int shape)
{
	switch (shape)
	{
	case 0: // Octahedron
		joiningFacesLabel->setEnabled(false);
		joiningFacesSpinBox->setEnabled(false);
		elongationLabel->setEnabled(false);
		elongationSpinBox->setEnabled(false);
		break;
	case 1: // Prism
		joiningFacesLabel->setEnabled(true);
		joiningFacesSpinBox->setEnabled(true);
		elongationLabel->setEnabled(true);
		elongationSpinBox->setEnabled(true);
		break;
	}
}

void EditCrystalDialog::editDefaultRotation()
{
	EditRotationDialog dialog(defaultRotationDescr);
	if (dialog.exec())
	{
		defaultRotationLabel->setText(dialog.getRotationName());
		defaultRotationDescr = dialog.getRotationDescr();
	}
}

void EditCrystalDialog::addRotation()
{
	RotationDescr rd;
	EditRotationDialog dialog(rd);
	if (dialog.exec())
		addRotation(dialog.getRotationDescr(), 0.0f);
}

void EditCrystalDialog::editRotation()
{
	int row = rotationTableWidget->currentRow();
	if (row < 0)
		return;
	QTableWidgetItem *rotationItem = rotationTableWidget->item(row, 0);
	EditRotationDialog dialog(rotationItem->data(Qt::UserRole).value<RotationDescr>());
	if (dialog.exec())
	{
		rotationItem->setData(Qt::UserRole, QVariant::fromValue(dialog.getRotationDescr()));
		rotationItem->setText(dialog.getRotationName());
	}
}

void EditCrystalDialog::removeRotation()
{
	int row = rotationTableWidget->currentRow();
	if (row < 0)
		return;
	rotationTableWidget->removeRow(row);
}

void EditCrystalDialog::removeAllRotations()
{
	while (rotationTableWidget->rowCount() > 0)
		rotationTableWidget->removeRow(0);
}

void EditCrystalDialog::addRotation(const RotationDescr &rotationDescr, float probability)
{
	int row = rotationTableWidget->rowCount();
	rotationTableWidget->insertRow(row);
	QTableWidgetItem *rotationItem = 
		new QTableWidgetItem(rotationDescr.getRotationName());
	rotationItem->setFlags(rotationItem->flags() & ~Qt::ItemIsEditable);
	rotationItem->setData(Qt::UserRole, QVariant::fromValue(rotationDescr));
	rotationTableWidget->setItem(row, 0, rotationItem);
	rotationTableWidget->setItem(row, 1, 
		new QTableWidgetItem(QString("%1").arg(probability)));
}

void EditCrystalDialog::createElements()
{
	crystalShapeLabel = new QLabel(tr("Crystal shape"));
	crystalShapeComboBox = new QComboBox();
	crystalShapeComboBox->addItem(tr("Octahedron"));
	crystalShapeComboBox->addItem(tr("Prism"));
	connect(crystalShapeComboBox, SIGNAL(currentIndexChanged(int)),
		this, SLOT(crystalShapeChanged(int)));

	joiningFacesLabel = new QLabel(tr("Joining faces:"));
	joiningFacesSpinBox = new QSpinBox();
	joiningFacesSpinBox->setRange(3, 100);
	joiningFacesSpinBox->setValue(6);

	elongationLabel = new QLabel(tr("Elongation:"));
	elongationSpinBox = new QDoubleSpinBox();
	elongationSpinBox->setDecimals(2);
	elongationSpinBox->setRange(0.1, 0.9);
	elongationSpinBox->setSingleStep(0.1);
	elongationSpinBox->setValue(0.5);

	defaultRotationGroupBox = new QGroupBox(tr("Default rotation"));
	defaultRotationLabel = new QLabel(tr("Random rotation."));
	editDefaultRotationButton = new QPushButton(tr("Edit"));
	connect(editDefaultRotationButton, SIGNAL(pressed()), 
		this, SLOT(editDefaultRotation()));
	QGridLayout *drLayout = new QGridLayout;
	drLayout->addWidget(defaultRotationLabel, 0, 0);
	drLayout->addWidget(editDefaultRotationButton, 0, 1);
	defaultRotationGroupBox->setLayout(drLayout);

	additionalRotationsGroupBox = new QGroupBox(tr("Additional rotations"));
	QStringList labelList;
	labelList.append(tr("Rotation"));
	labelList.append(tr("Probability"));
	rotationTableWidget = new QTableWidget();
	rotationTableWidget->setColumnCount(2);
	rotationTableWidget->setHorizontalHeaderLabels(labelList);
	rotationTableWidget->setItemDelegateForColumn(1, 
		new ProbabilitySpinBoxDelegate());
	addRotationButton = new QPushButton(tr("Add"));
	editRotationButton = new QPushButton(tr("Edit"));
	removeRotationButton = new QPushButton(tr("Remove"));
	removeAllRotationsButton = new QPushButton(tr("Remove all"));
	connect(addRotationButton, SIGNAL(pressed()), this, SLOT(addRotation()));
	connect(editRotationButton, SIGNAL(pressed()), this, SLOT(editRotation()));
	connect(removeRotationButton, SIGNAL(pressed()), this, SLOT(removeRotation()));
	connect(removeAllRotationsButton, SIGNAL(pressed()), this, SLOT(removeAllRotations()));

	QGridLayout *arLayout = new QGridLayout;
	arLayout->addWidget(rotationTableWidget, 0, 0, 5, 1);
	arLayout->addWidget(addRotationButton, 0, 1);
	arLayout->addWidget(editRotationButton, 1, 1);
	arLayout->addWidget(removeRotationButton, 2, 1);
	arLayout->addWidget(removeAllRotationsButton, 3, 1);
	additionalRotationsGroupBox->setLayout(arLayout);

	okButton = new QPushButton(tr("Ok"));
	connect(okButton, SIGNAL(pressed()), this, SLOT(accept()));
	cancelButton = new QPushButton(tr("Cancel"));
	connect(cancelButton, SIGNAL(pressed()), this, SLOT(reject()));

	QWidget *buttonBar = new QWidget();
	QGridLayout *butLayout = new QGridLayout;
	butLayout->addWidget(okButton, 0, 1);
	butLayout->addWidget(cancelButton, 0, 2);
	butLayout->setColumnStretch(0, 1);
	buttonBar->setLayout(butLayout);

	QGridLayout *layout = new QGridLayout;
	layout->addWidget(crystalShapeLabel, 0, 0);
	layout->addWidget(crystalShapeComboBox, 0, 1);
	layout->addWidget(joiningFacesLabel, 1, 0);
	layout->addWidget(joiningFacesSpinBox, 1, 1);
	layout->addWidget(elongationLabel, 2, 0);
	layout->addWidget(elongationSpinBox, 2, 1);
	layout->addWidget(defaultRotationGroupBox, 3, 0, 1, 2);
	layout->addWidget(additionalRotationsGroupBox, 4, 0, 1, 2);
	layout->addWidget(buttonBar, 5, 0, 1, 2);
	setLayout(layout);

	crystalShapeComboBox->setCurrentIndex(1);
}

QString EditCrystalDialog::getCrystalName()
{
	switch (crystalShapeComboBox->currentIndex())
	{
	case 0: // Octahedron
		return tr("Octahedron");
	case 1: // Prism
		return tr("Prism (%1-gonal)").arg(joiningFacesSpinBox->value());
	}
	return tr("");
}

CrystalDescr EditCrystalDialog::getCrystalDescr()
{
	CrystalDescr result;
	switch (crystalShapeComboBox->currentIndex())
	{
	case 0: // Octahedron
		result.setCrystalType(CrystalDescr::Diamond);
		break;
	case 1: // Prism
		result.setCrystalType(CrystalDescr::Prism);
		result.setJoiningFacesCount(joiningFacesSpinBox->value());
		result.setElongation(elongationSpinBox->value());
		break;
	}

	return result;
}

RotationDescr EditCrystalDialog::getRotationDescr()
{
	if (rotationTableWidget->rowCount() == 0)
		return defaultRotationDescr;

	RotationDescr result;
	result.setRotationType(RotationDescr::RotationAlternative);
	result.setDefaultRotationDescr(&defaultRotationDescr);

	for (int row = 0; row < rotationTableWidget->rowCount(); row++)
	{
		result.appendAdditionalRotation(
			rotationTableWidget->item(row, 0)->data(Qt::UserRole).value<RotationDescr>(),
			rotationTableWidget->item(row, 1)->text().toFloat());
	}

	return result;
}
