/* ----------------------------------------------------------------------------
This file is part of BlueSky

Copyright (C) 2012 Piotr Polesiuk

Permission is hereby granted, free of charge, to any person obtaining a copy 
of this software and associated documentation files (the "Software"), to deal 
in the Software without restriction, including without limitation the rights 
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
copies of the Software, and to permit persons to whom the Software is 
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in 
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
THE SOFTWARE.
---------------------------------------------------------------------------- */

#include "editRotationDialog.h"

#include <QtGui/QtGui>

EditRotationDialog::EditRotationDialog(const RotationDescr &rotationDescr)
{
	createElements();

	switch (rotationDescr.getRotationType())
	{
	case RotationDescr::RandomUniformRotation:
		rotationTypeComboBox->setCurrentIndex(0);
		break;
	case RotationDescr::RotationY:
		rotationTypeComboBox->setCurrentIndex(1);
		break;
	case RotationDescr::RotationX90ZY:
		rotationTypeComboBox->setCurrentIndex(2);
		break;
	case RotationDescr::RotationX90Y:
		rotationTypeComboBox->setCurrentIndex(3);
		break;
	case RotationDescr::RotationXY:
		rotationTypeComboBox->setCurrentIndex(4);
		break;
	case RotationDescr::RotationZY:
		rotationTypeComboBox->setCurrentIndex(5);
		break;
	default:
		QMessageBox::warning(this, tr("Edit rotation"),
			tr("This rotation can not be edited"));
	}

	if (rotationDescr.isDeviated())
	{
		deviationGroupBox->setChecked(true);
		deviationSpinBox->setValue(rotationDescr.getDeviation());
	}
}

void EditRotationDialog::rotationTypeChanged(int index)
{
	switch (index)
	{
	case 0: // Random rotation
		rotationDescription->setText(
			tr("Completely random rotation with uniform distribution. "
			"It can be used to simulate halo."));
		break;
	case 1: // Plate crystals
		rotationDescription->setText(
			tr("Bases of crystals are oriented horizontally and axis is oriented "
			"vertically. This orientation is characteristic for plate crystals."));
		break;
	case 2: // Column crystals
		rotationDescription->setText(
			tr("Column crystals are oriented horizontally and they roll like "
			"a barrel."));
		break;
	case 3: // Parry crystals
		rotationDescription->setText(
			tr("One of joining face is oriented horizontally. It is rare orientation "
			"for column crystals."));
		break;
	case 4: // Lowitz crystals
		rotationDescription->setText(
			tr("Crystals are rotating around the axis parallel to one of the diagonals "
			"of the prism base. It is a very rare orientation for plate crystals"));
		break;
	case 5: // RotationZY
		rotationDescription->setText(
			tr("Crystals are rotating around the axis perpendicular to the joining "
			"face. This case is similar to Lowitz crystals, but is non-physical."));
		break;
	}
}

void EditRotationDialog::createElements()
{
	rotationDescription = new QLabel();
	rotationDescription->setWordWrap(true);
	rotationDescription->setAlignment(Qt::AlignTop);

	rotationTypeComboBox = new QComboBox();
	rotationTypeComboBox->addItem(tr("Random rotation"));
	rotationTypeComboBox->addItem(tr("Plate crystals"));
	rotationTypeComboBox->addItem(tr("Column crystals"));
	rotationTypeComboBox->addItem(tr("Parry crystals"));
	rotationTypeComboBox->addItem(tr("Lowitz crystals"));
	rotationTypeComboBox->addItem(tr("RotationZY crystals"));
	connect(rotationTypeComboBox, SIGNAL(currentIndexChanged(int)),
		this, SLOT(rotationTypeChanged(int)));

	deviationGroupBox = new QGroupBox(tr("Random deviation"));
	deviationGroupBox->setCheckable(true);
	deviationGroupBox->setChecked(false);
	deviationLabel = new QLabel(tr("Deviation variance (rad):"));
	deviationSpinBox = new QDoubleSpinBox();
	deviationSpinBox->setRange(0.0, 5.0);
	deviationSpinBox->setDecimals(3);
	deviationSpinBox->setSingleStep(0.01);

	QGridLayout *dLayout = new QGridLayout;
	dLayout->addWidget(deviationLabel, 0, 0);
	dLayout->addWidget(deviationSpinBox, 0, 1);
	deviationGroupBox->setLayout(dLayout);

	okButton = new QPushButton(tr("Ok"));
	connect(okButton, SIGNAL(pressed()), this, SLOT(accept()));
	cancelButton = new QPushButton(tr("Cancel"));
	connect(cancelButton, SIGNAL(pressed()), this, SLOT(reject()));

	QWidget *buttonBar = new QWidget();
	QGridLayout *butLayout = new QGridLayout;
	butLayout->addWidget(okButton, 0, 1);
	butLayout->addWidget(cancelButton, 0, 2);
	butLayout->setColumnStretch(0, 1);
	buttonBar->setLayout(butLayout);

	QGridLayout *layout = new QGridLayout;
	layout->addWidget(rotationTypeComboBox, 0, 0);
	layout->addWidget(deviationGroupBox, 1, 0);
	layout->addWidget(rotationDescription, 0, 1, 2, 1);
	layout->addWidget(buttonBar, 2, 0, 1, 2);
	setLayout(layout);

	rotationTypeChanged(0);
}

QString EditRotationDialog::getRotationName()
{
	switch (rotationTypeComboBox->currentIndex())
	{
	case 0: // RandomRotation
		return tr("Random rotation");
	case 1: // Plate crystals
		return tr("Plate crystals");
	case 2: // Column crystals
		return tr("Column crystals");
	case 3: // Parry crystals
		return tr("Parry crystals");
	case 4: // Lowitz crystals
		return tr("Lowitz crystals");
	case 5: // RotationZY
		return tr("RotationZY");
	}
	return "Unknown";
}

RotationDescr EditRotationDialog::getRotationDescr()
{
	RotationDescr result;

	switch (rotationTypeComboBox->currentIndex())
	{
	case 0: // RandomRotation
		result.setRotationType(RotationDescr::RandomUniformRotation);
		break;
	case 1: // Plate crystals
		result.setRotationType(RotationDescr::RotationY);
		break;
	case 2: // Column crystals
		result.setRotationType(RotationDescr::RotationX90ZY);
		break;
	case 3: // Parry crystals
		result.setRotationType(RotationDescr::RotationX90Y);
		break;
	case 4: // Lowitz crystals
		result.setRotationType(RotationDescr::RotationXY);
		break;
	case 5: // RotationZY
		result.setRotationType(RotationDescr::RotationZY);
		break;
	}

	if (deviationGroupBox->isChecked())
	{
		result.setDeviated(true);
		result.setDeviation(deviationSpinBox->value());
	}

	return result;
}
