/* ----------------------------------------------------------------------------
This file is part of BlueSky

Copyright (C) 2012 Piotr Polesiuk

Permission is hereby granted, free of charge, to any person obtaining a copy 
of this software and associated documentation files (the "Software"), to deal 
in the Software without restriction, including without limitation the rights 
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
copies of the Software, and to permit persons to whom the Software is 
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in 
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
THE SOFTWARE.
---------------------------------------------------------------------------- */

#include "mainWindow.h"

#include "skyView.h"
#include "simulation.h"
#include "newSimulationWizard.h"
#include "scriptEditor.h"
#include "blueSkyXmlScript.h"
#include "errorOutputWidget.h"
#include "displaySettingsWidget.h"
#include "probabilitySpinBoxDelegate.h"
#include "editActionDialog.h"
#include "settings.h"

#include <QtGui/QtGui>

MainWindow::MainWindow()
{
	simulation = NULL;
	newSimulationWizard = new NewSimulationWizard(this);

	createActions();
	createToolBars();
	createStatusBar();
	createTabs();
	createDockWindows();
	createMenus();

	createConnections();
}

MainWindow::~MainWindow()
{
	if (simulation)
		delete simulation;
}

void MainWindow::closeEvent(QCloseEvent *event)
{
	if (maybeSave())
	{
		Settings::instance()->writeSettings();
		event->accept();
	}
	else
		event->ignore();
}

void MainWindow::newEmptyScript()
{
	if (maybeSave())
	{
		scriptEditor->clear();
		scriptEditor->document()->setModified(false);
		currentScriptPath = "";
	}
}

void MainWindow::showNewSimWizard()
{
	if (maybeSave())
	{
		newSimulationWizard->restart();
		if (newSimulationWizard->exec())
		{
			scriptEditor->setPlainText(newSimulationWizard->getScriptCode());
			scriptEditor->document()->setModified(false);
			currentScriptPath = "";
			parseScript();
		}
	}
}

void MainWindow::open()
{
	if (maybeSave())
	{
		QString openedFileName = 
			QFileDialog::getOpenFileName(
				this,
				tr("Open script file"),
				QString(),
				tr("BXS file (*.bxs);; All files (*)"));
		if (!openedFileName.isEmpty())
		{
			if (loadScript(openedFileName))
				parseScript();
		}
	}
}

bool MainWindow::save()
{
	if (currentScriptPath.isEmpty())
		return saveAs();
	else
		return saveScript(currentScriptPath);
}

bool MainWindow::saveAs()
{
	QStringList filters;
	filters
		<< tr("BXS file (*.bxs)")
		<< tr("All files (*)");
	QFileDialog dialog(this, tr("Save As"));
	dialog.setFilters(filters);
	dialog.setAcceptMode(QFileDialog::AcceptSave);
	dialog.setFileMode(QFileDialog::AnyFile);

	if (dialog.exec())
	{
		if (dialog.selectedNameFilter() == filters[0])
			dialog.setDefaultSuffix("bxs");
		return saveScript(dialog.selectedFiles().first());
	}
	else
		return false;
}

void MainWindow::saveImage()
{
	QStringList filters;
	filters
		<< tr("Windows Bitmap (*.bmp)")
		<< tr("Joint Photographic Experts Group (*.jpg)")
		<< tr("Joint Photographic Experts Group (*.jpeg)")
		<< tr("Portable Network Graphics (*.png)")
		<< tr("Portable Pixmap (*.ppm)")
		<< tr("Tagged Image File Format (*.tiff)")
		<< tr("X11 Bitmap (*.xbm)")
		<< tr("X11 Pixmap (*.xpm)");
	QFileDialog dialog(this, tr("Save image"));
	dialog.setFilters(filters);
	dialog.setFileMode(QFileDialog::AnyFile);
	dialog.setAcceptMode(QFileDialog::AcceptSave);
	if (dialog.exec())
	{
		const char *format = NULL;
		if (dialog.selectedNameFilter() == filters[0]) format = "BMP";
		else if (dialog.selectedNameFilter() == filters[1]) format = "JPG";
		else if (dialog.selectedNameFilter() == filters[2]) format = "JPEG";
		else if (dialog.selectedNameFilter() == filters[3]) format = "PNG";
		else if (dialog.selectedNameFilter() == filters[4]) format = "PPM";
		else if (dialog.selectedNameFilter() == filters[5]) format = "TIFF";
		else if (dialog.selectedNameFilter() == filters[6]) format = "XBM";
		else if (dialog.selectedNameFilter() == filters[7]) format = "XPM";

		if (format)
			dialog.setDefaultSuffix(QString(format).toLower());

		skyView->getImage().save(dialog.selectedFiles().first(), format);
	}
}

void MainWindow::startSimulation()
{
	if (simulation)
	{
		stopSimulation();
		delete simulation;
		simulation = NULL;
	}

	BlueSkyXmlScript *script = 
		BlueSkyXmlScript::fromText(scriptEditor->toPlainText(), outputWidget);
	if (!script || !script->parse(outputWidget))
	{
		startSimAction->setEnabled(true);
		pauseSimAction->setEnabled(false);
		stopSimAction->setEnabled(false);
		setSimSettingsEnabled(false);
		QMessageBox::warning(this, tr("Start simulation"),
			tr("Invalid script!\nCheck output for details."));
		if (script)
			delete script;
		return;
	}

	if (simSettingsChanged)
	{
		ssUpdateScript(script);
		scriptEditor->setPlainText(script->toText());
	}

	ssUpdateFromScript(script);
	setSimSettingsEnabled(true);
	simSettingsChanged = false;

	simulation = script->getSimulation(Settings::instance()->getThreadCount());

	if (simulation)
	{
		simulation->start();
		description->setHtml(script->getDescription());
		skyView->updateSimulation(simulation);
		saveImageAction->setEnabled(true);

		startSimAction->setEnabled(true);
		pauseSimAction->setEnabled(true);
		stopSimAction->setEnabled(true);
	}
	else
	{
		startSimAction->setEnabled(true);
		pauseSimAction->setEnabled(false);
		stopSimAction->setEnabled(false);
	}

	delete script;
}

void MainWindow::pauseSimulation()
{
	if (simulation)
	{
		simulation->pause();
		startSimAction->setEnabled(simulation->getState() == Simulation::PAUSED);
	}
}

void MainWindow::stopSimulation()
{
	if (simulation)
		simulation->stop();
	startSimAction->setEnabled(true);
	pauseSimAction->setEnabled(false);
	stopSimAction->setEnabled(false);
}

void MainWindow::about()
{
	QMessageBox::about(this, tr("About QtBlueSky"), 
		tr("<H1>QtBlueSky</H1>"
		"<H3>version %1</H3>"
		"Main author: Piotr Polesiuk").arg(Settings::qtBlueSkyVersion()));
}

void MainWindow::scriptChanged()
{
	simSettingsChanged = false;
}

void MainWindow::parseScript()
{
	if (simulation)
	{
		stopSimulation();
		delete simulation;
		simulation = NULL;
	}

	BlueSkyXmlScript *script = 
		BlueSkyXmlScript::fromText(scriptEditor->toPlainText(), outputWidget);
	if (!script || !script->parse(outputWidget))
	{
		startSimAction->setEnabled(true);
		pauseSimAction->setEnabled(false);
		stopSimAction->setEnabled(false);
		setSimSettingsEnabled(false);
		if (script)
			delete script;
		return;
	}
	ssUpdateFromScript(script);
	setSimSettingsEnabled(true);
	simSettingsChanged = false;
	description->setHtml(script->getDescription());
	
	delete script;
}

void MainWindow::displaySettingsLocationChanged(Qt::DockWidgetArea area)
{
	if (area & (Qt::LeftDockWidgetArea | Qt::RightDockWidgetArea))
		displaySettingsWidget->setOrientation(false);
	else
		displaySettingsWidget->setOrientation(true);
}

void MainWindow::createActions()
{
	/* File menu */
	newEmptyScriptAction = new QAction(tr("New &Empty Script"), this);
	newEmptyScriptAction->setStatusTip(tr("Create new empty simulation script"));
	connect(newEmptyScriptAction, SIGNAL(triggered()), this, SLOT(newEmptyScript()));

	newAction = new QAction(tr("&New"), this);
	newAction->setShortcuts(QKeySequence::New);
	newAction->setStatusTip(tr("Create new simulation"));
	connect(newAction, SIGNAL(triggered()), this, SLOT(showNewSimWizard()));

	openAction = new QAction(tr("&Open"), this);
	openAction->setShortcuts(QKeySequence::Open);
	openAction->setStatusTip(tr("Open existing simulation settings"));
	connect(openAction, SIGNAL(triggered()), this, SLOT(open()));

	saveAction = new QAction(tr("&Save"), this);
	saveAction->setShortcuts(QKeySequence::Save);
	saveAction->setStatusTip(tr("Save simulation settings"));
	connect(saveAction, SIGNAL(triggered()), this, SLOT(save()));

	saveAsAction = new QAction(tr("Save &As"), this);
	saveAsAction->setShortcuts(QKeySequence::SaveAs);
	saveAsAction->setStatusTip(tr("Save simulation settings under new file"));
	connect(saveAsAction, SIGNAL(triggered()), this, SLOT(saveAs()));

	saveImageAction = new QAction(tr("Save &Image"), this);
	saveImageAction->setStatusTip(tr("Save current camera view as an image"));
	saveImageAction->setEnabled(false);
	connect(saveImageAction, SIGNAL(triggered()), this, SLOT(saveImage()));
	
	settingsAction = new QAction(tr("Se&ttings"), this);
	settingsAction->setStatusTip(tr("Configure the application"));
	connect(settingsAction, SIGNAL(triggered()), Settings::instance(), SLOT(exec()));

	exitAction = new QAction(tr("E&xit"), this);
	exitAction->setShortcuts(QKeySequence::Quit);
	exitAction->setStatusTip(tr("Exit the application"));
	connect(exitAction, SIGNAL(triggered()), this, SLOT(close()));

	/* Edit menu */
	undoAction = new QAction(tr("Undo"), this);
	undoAction->setShortcuts(QKeySequence::Undo);
	undoAction->setStatusTip(tr("Undo changes in script"));
	undoAction->setEnabled(false);

	redoAction = new QAction(tr("Redo"), this);
	redoAction->setShortcuts(QKeySequence::Redo);
	redoAction->setStatusTip(tr("Redo changes in script"));
	redoAction->setEnabled(false);
	
	cutAction = new QAction(tr("Cut"), this);
	cutAction->setShortcuts(QKeySequence::Cut);
	cutAction->setStatusTip(tr("Cut selected part of script"));
	cutAction->setEnabled(false);

	copyAction = new QAction(tr("Copy"), this);
	copyAction->setShortcuts(QKeySequence::Copy);
	copyAction->setStatusTip(tr("Copy selected part of script"));
	copyAction->setEnabled(false);

	pasteAction = new QAction(tr("Paste"), this);
	pasteAction->setShortcuts(QKeySequence::Paste);
	pasteAction->setStatusTip(tr("Paste copied/cut part of script"));

	/* Simulation menu */
	startSimAction = new QAction(tr("&Start"), this);
	startSimAction->setStatusTip(tr("Start the simulation"));
	startSimAction->setEnabled(true);
	connect(startSimAction, SIGNAL(triggered()), this, SLOT(startSimulation()));

	pauseSimAction = new QAction(tr("&Pause"), this);
	pauseSimAction->setStatusTip(tr("Pause the simulation"));
	pauseSimAction->setEnabled(false);
	connect(pauseSimAction, SIGNAL(triggered()), this, SLOT(pauseSimulation()));

	stopSimAction = new QAction(tr("&Stop"), this);
	stopSimAction->setStatusTip(tr("Stop the simulation"));
	stopSimAction->setEnabled(false);
	connect(stopSimAction, SIGNAL(triggered()), this, SLOT(stopSimulation()));

	/* Help menu */
	aboutAction = new QAction(tr("&About"), this);
	aboutAction->setStatusTip(tr("Show the QtBlueSky's About box."));
	connect(aboutAction, SIGNAL(triggered()), this, SLOT(about()));

	aboutQtAction = new QAction(tr("About &Qt"), this);
	aboutQtAction->setStatusTip(tr("Show the Qt library's About box."));
	connect(aboutQtAction, SIGNAL(triggered()), qApp, SLOT(aboutQt()));
}

void MainWindow::createMenus()
{
	fileMenu = menuBar()->addMenu(tr("&File"));
	fileMenu->addAction(newAction);
	fileMenu->addAction(newEmptyScriptAction);
	fileMenu->addAction(openAction);
	fileMenu->addAction(saveAction);
	fileMenu->addAction(saveAsAction);
	fileMenu->addAction(saveImageAction);
	fileMenu->addSeparator();
	fileMenu->addAction(settingsAction);
	fileMenu->addSeparator();
	fileMenu->addAction(exitAction);

	editMenu = menuBar()->addMenu(tr("&Edit"));
	editMenu->addAction(undoAction);
	editMenu->addAction(redoAction);
	editMenu->addSeparator();
	editMenu->addAction(cutAction);
	editMenu->addAction(copyAction);
	editMenu->addAction(pasteAction);

	viewMenu = menuBar()->addMenu(tr("&View"));
	viewMenu->addAction(simSettingsDockWidget->toggleViewAction());
	viewMenu->addAction(outputDockWidget->toggleViewAction());
	viewMenu->addAction(displaySettingsDockWidget->toggleViewAction());
	viewMenu->addSeparator();
	viewMenu->addAction(simulationToolBar->toggleViewAction());

	simulationMenu = menuBar()->addMenu(tr("&Simulation"));
	simulationMenu->addAction(startSimAction);
	simulationMenu->addAction(pauseSimAction);
	simulationMenu->addAction(stopSimAction);

	helpMenu = menuBar()->addMenu(tr("&Help"));
	helpMenu->addAction(aboutAction);
	helpMenu->addAction(aboutQtAction);
}

void MainWindow::createToolBars()
{
	simulationToolBar = addToolBar(tr("Simulation Toolbar"));
	simulationToolBar->addAction(startSimAction);
	simulationToolBar->addAction(pauseSimAction);
	simulationToolBar->addAction(stopSimAction);
}

void MainWindow::createStatusBar()
{
}

void MainWindow::createDockWindows()
{
	createSimulationSettingsWidget();

	simSettingsDockWidget = new QDockWidget(tr("Simulation settings"), this);
	simSettingsDockWidget->setMinimumWidth(200);
	simSettingsDockWidget->setAllowedAreas(Qt::LeftDockWidgetArea | Qt::RightDockWidgetArea);
	addDockWidget(Qt::LeftDockWidgetArea, simSettingsDockWidget);
	simSettingsDockWidget->setWidget(ssWidget);

	outputDockWidget = new QDockWidget(tr("Output"), this);
	outputDockWidget->setAllowedAreas(Qt::BottomDockWidgetArea);
	addDockWidget(Qt::BottomDockWidgetArea, outputDockWidget);
	outputWidget = new ErrorOutputWidget;
	outputDockWidget->setWidget(outputWidget);

	displaySettingsDockWidget = new QDockWidget(tr("Display settings"), this);
	displaySettingsDockWidget->setAllowedAreas(Qt::AllDockWidgetAreas);
	addDockWidget(Qt::BottomDockWidgetArea, displaySettingsDockWidget);
	displaySettingsWidget = new DisplaySettingsWidget(skyView);
	displaySettingsDockWidget->setWidget(displaySettingsWidget);
	connect(displaySettingsDockWidget, SIGNAL(dockLocationChanged(Qt::DockWidgetArea)),
		this, SLOT(displaySettingsLocationChanged(Qt::DockWidgetArea)));
}

void MainWindow::createTabs()
{
	tabWidget = new QTabWidget();
	setCentralWidget(tabWidget);
	
	description = new QTextEdit();
	description->setReadOnly(true);
	tabWidget->addTab(description, tr("Description"));

	skyView = new SkyView(simulation);
	tabWidget->addTab(skyView, tr("Image"));

	scriptEditor = new ScriptEditor();
	tabWidget->addTab(scriptEditor, tr("Script"));
	connect(scriptEditor, SIGNAL(textChanged()), this, SLOT(scriptChanged()));
}

void MainWindow::createConnections()
{
	connect(undoAction,  SIGNAL(triggered()), scriptEditor, SLOT(undo()));
	connect(redoAction,  SIGNAL(triggered()), scriptEditor, SLOT(redo()));
	connect(cutAction,   SIGNAL(triggered()), scriptEditor, SLOT(cut()));
	connect(copyAction,  SIGNAL(triggered()), scriptEditor, SLOT(copy()));
	connect(pasteAction, SIGNAL(triggered()), scriptEditor, SLOT(paste()));

	connect(scriptEditor, SIGNAL(copyAvailable(bool)), cutAction,  SLOT(setEnabled(bool)));
	connect(scriptEditor, SIGNAL(copyAvailable(bool)), copyAction, SLOT(setEnabled(bool)));
	connect(scriptEditor, SIGNAL(undoAvailable(bool)), undoAction, SLOT(setEnabled(bool)));
	connect(scriptEditor, SIGNAL(redoAvailable(bool)), redoAction, SLOT(setEnabled(bool)));
}

bool MainWindow::maybeSave()
{
	if (scriptEditor->document()->isModified())
	{
		QMessageBox::StandardButton ret;
		ret = QMessageBox::warning(this, tr("QtBlueSky"),
			tr("Current script has been modified.\n"
				"Do you want to save your changes?"),
			QMessageBox::Save | QMessageBox::Discard | QMessageBox::Cancel);
		if (ret == QMessageBox::Save)
			return save();
		else if (ret == QMessageBox::Cancel)
			return false;
	}
	return true;
}

bool MainWindow::saveScript(QString path)
{
	QFile file(path);
	if (!file.open(QFile::WriteOnly | QFile::Text))
	{
		QMessageBox::warning(this, tr("QtBlueSky"), 
			tr("Can not write file %1:\n%2.").arg(path).arg(file.errorString()));
		return false;
	}

	QTextStream out(&file);
	out.setCodec("UTF-8");
	QApplication::setOverrideCursor(Qt::WaitCursor);
	out << scriptEditor->toPlainText();
	QApplication::restoreOverrideCursor();

	currentScriptPath = path;
	scriptEditor->document()->setModified(false);

	return true;
}

bool MainWindow::loadScript(QString path)
{
	QFile file(path);
	if (!file.open(QFile::ReadOnly | QFile::Text))
	{
		QMessageBox::warning(this, tr("QtBlueSky"),
			tr("Can not read file %1:\n%2.").arg(path).arg(file.errorString()));
		return false;
	}

	QTextStream in(&file);
	in.setCodec("UTF-8");
	QApplication::setOverrideCursor(Qt::WaitCursor);
	scriptEditor->setPlainText(in.readAll());
	QApplication::restoreOverrideCursor();

	currentScriptPath = path;
	scriptEditor->document()->setModified(false);

	return true;
}

/* ============================================================================
                         SIMULATION SETTINGS WIDGET
============================================================================ */

void MainWindow::ssDoubleValueChanged(double /* value */)
{
	simSettingsChanged = true;
}

void MainWindow::ssIntValueChanged(int /* value */)
{
	simSettingsChanged = true;
}

void MainWindow::ssEditDefaultAction()
{
	EditActionDialog dialog(ssDefaultAction);

	if (dialog.exec())
	{
		ssDefaultAction = dialog.getActionDescr();
		ssDefaultActionLabel->setText(dialog.getActionName());
		simSettingsChanged = true;
	}
}

void MainWindow::ssAddAction()
{
	EditActionDialog dialog;
	if (dialog.exec())
	{
		ssAddAdditionalAction(dialog.getActionDescr(), 0.0f);
		simSettingsChanged = true;
	}
}

void MainWindow::ssEditAction()
{
	int row = ssActionsTableWidget->currentRow();
	if (row < 0)
		return;
	QTableWidgetItem *actionItem = ssActionsTableWidget->item(row, 0);
	EditActionDialog dialog(actionItem->data(Qt::UserRole).value<ActionDescr>());
	if (dialog.exec())
	{
		actionItem->setData(Qt::UserRole, QVariant::fromValue(dialog.getActionDescr()));
		actionItem->setText(dialog.getActionName());
	}
}

void MainWindow::ssRemoveAction()
{
	int row = ssActionsTableWidget->currentRow();
	if (row < 0)
		return;
	ssActionsTableWidget->removeRow(row);
}

void MainWindow::ssRemoveAllActions()
{
	while (ssActionsTableWidget->rowCount() > 0)
		ssActionsTableWidget->removeRow(0);
	simSettingsChanged = true;
}

void MainWindow::ssActionsTableChanged(int /* row */, int /* column */)
{
	simSettingsChanged = true;
}

void MainWindow::ssCameraTypeChanged(int index)
{
	switch (index)
	{
	case 0:
		ssCameraResolutionSpinBox->setMaximum(1000000);
		break;
	case 1:
		ssCameraResolutionSpinBox->setMaximum(10000);
		break;
	}
	simSettingsChanged = true;
}

void MainWindow::createSimulationSettingsWidget()
{
	simSettingsChanged = false;

	ssWidget = new QWidget();

	ssSunGroupBox = new QGroupBox(tr("Sun"));
	ssAzimuthLabel = new QLabel(tr("Azimuth:"));
	ssAltitudeLabel = new QLabel(tr("Altitude:"));
	
	ssAzimuthSpinBox = new QDoubleSpinBox();
	ssAzimuthSpinBox->setDecimals(1);
	ssAzimuthSpinBox->setRange(0.0, 360.0);
	ssAzimuthSpinBox->setSingleStep(1.0);
	ssAzimuthSpinBox->setWrapping(true);
	connect(ssAzimuthSpinBox, SIGNAL(valueChanged(double)),
		this, SLOT(ssDoubleValueChanged(double)));

	ssAltitudeSpinBox = new QDoubleSpinBox();
	ssAltitudeSpinBox->setDecimals(1);
	ssAltitudeSpinBox->setRange(-90.0, 90.0);
	ssAltitudeSpinBox->setSingleStep(1.0);
	connect(ssAltitudeSpinBox, SIGNAL(valueChanged(double)),
		this, SLOT(ssDoubleValueChanged(double)));

	QGridLayout *ssSunLayout = new QGridLayout();
	ssSunLayout->addWidget(ssAzimuthLabel, 0, 0);
	ssSunLayout->addWidget(ssAltitudeLabel, 1, 0);
	ssSunLayout->addWidget(ssAzimuthSpinBox, 0, 1);
	ssSunLayout->addWidget(ssAltitudeSpinBox, 1, 1);
	ssSunGroupBox->setLayout(ssSunLayout);
	
	ssDefaultActionGroupBox = new QGroupBox(tr("Default Action"));
	ssDefaultActionLabel = new QLabel(tr("Default action"));
	ssEditDefaultActionButton = new QPushButton(tr("Edit"));
	connect(ssEditDefaultActionButton, SIGNAL(pressed()),
		this, SLOT(ssEditDefaultAction()));

	QHBoxLayout *ssDefActionLayout = new QHBoxLayout;
	ssDefActionLayout->addWidget(ssDefaultActionLabel);
	ssDefActionLayout->addWidget(ssEditDefaultActionButton);
	ssDefaultActionGroupBox->setLayout(ssDefActionLayout);

	ssActionsGroupBox = new QGroupBox(tr("Additional actions"));
	QStringList labelList;
	labelList.append(tr("Action"));
	labelList.append(tr("Probability"));
	ssActionsTableWidget = new QTableWidget;
	ssActionsTableWidget->setColumnCount(2);
	ssActionsTableWidget->setHorizontalHeaderLabels(labelList);
	ssActionsTableWidget->setItemDelegateForColumn(1, 
		new ProbabilitySpinBoxDelegate());
	connect(ssActionsTableWidget, SIGNAL(cellChanged(int,int)),
		this, SLOT(ssActionsTableChanged(int,int)));

	ssAddActionButton = new QPushButton(tr("Add"));
	ssEditActionButton = new QPushButton(tr("Edit"));
	ssRemoveActionButton = new QPushButton(tr("Remove"));
	ssRemoveAllActionsButton = new QPushButton(tr("Remove All"));
	connect(ssAddActionButton, SIGNAL(pressed()), this, SLOT(ssAddAction()));
	connect(ssEditActionButton, SIGNAL(pressed()), this, SLOT(ssEditAction()));
	connect(ssRemoveActionButton, SIGNAL(pressed()), this, SLOT(ssRemoveAction()));
	connect(ssRemoveAllActionsButton, SIGNAL(pressed()), this, SLOT(ssRemoveAllActions()));

	QGridLayout *ssActionsLayout = new QGridLayout();
	ssActionsLayout->addWidget(ssActionsTableWidget, 0, 0, 1, 2);
	ssActionsLayout->addWidget(ssAddActionButton, 1, 0);
	ssActionsLayout->addWidget(ssEditActionButton, 1, 1);
	ssActionsLayout->addWidget(ssRemoveActionButton, 2, 0);
	ssActionsLayout->addWidget(ssRemoveAllActionsButton, 2, 1);
	ssActionsGroupBox->setLayout(ssActionsLayout);

	ssCameraGroupBox = new QGroupBox(tr("Camera"));
	ssCameraTypeLabel = new QLabel(tr("Type"));
	ssCameraResolutionLabel = new QLabel(tr("Resolution"));

	ssCameraTypeComboBox = new QComboBox();
	ssCameraTypeComboBox->addItem(tr("Linear Camera"));
	ssCameraTypeComboBox->addItem(tr("Sky Box Camera"));
	connect(ssCameraTypeComboBox, SIGNAL(currentIndexChanged(int)),
		this, SLOT(ssCameraTypeChanged(int)));

	ssCameraResolutionSpinBox = new QSpinBox();
	ssCameraResolutionSpinBox->setRange(1, 1000000);
	ssCameraResolutionSpinBox->setValue(1024);
	connect(ssCameraResolutionSpinBox, SIGNAL(valueChanged(int)),
		this, SLOT(ssIntValueChanged(int)));

	QGridLayout *ssCameraLayout = new QGridLayout();
	ssCameraLayout->addWidget(ssCameraTypeLabel, 0, 0);
	ssCameraLayout->addWidget(ssCameraResolutionLabel, 1, 0);
	ssCameraLayout->addWidget(ssCameraTypeComboBox, 0, 1);
	ssCameraLayout->addWidget(ssCameraResolutionSpinBox, 1, 1);
	ssCameraGroupBox->setLayout(ssCameraLayout);

	QVBoxLayout *layout = new QVBoxLayout();
	layout->addWidget(ssSunGroupBox);
	layout->addWidget(ssDefaultActionGroupBox);
	layout->addWidget(ssActionsGroupBox);
	layout->addWidget(ssCameraGroupBox);
	layout->addWidget(new QWidget());
	layout->setStretch(4, 10);
	ssWidget->setLayout(layout);

	setSimSettingsEnabled(false);
}

void MainWindow::setSimSettingsEnabled(bool enabled)
{
	ssSunGroupBox->setEnabled(enabled);
	ssDefaultActionGroupBox->setEnabled(enabled);
	ssActionsGroupBox->setEnabled(enabled);
	ssCameraGroupBox->setEnabled(enabled);
}

void MainWindow::ssUpdateScript(BlueSkyXmlScript *script)
{
	script->setSunDescr(ssGetSunDescr());
	script->setActionDescr(ssGetActionDescr());
	script->setCameraDescr(ssGetCameraDescr());
}

void MainWindow::ssUpdateFromScript(BlueSkyXmlScript *script)
{
	ssAzimuthSpinBox->setValue(script->getSunAzimuth());
	ssAltitudeSpinBox->setValue(script->getSunAltitude());

	ssRemoveAllActions();
	if (script->getActionDescr().getActionType() == ActionDescr::ActionAlternative)
	{
		if (script->getActionDescr().getDefaultActionDescr())
		{
			ssDefaultAction = *script->getActionDescr().getDefaultActionDescr();
			ssDefaultActionLabel->setText(ssDefaultAction.getActionName());
		}

		for (int i = 0; i < script->getActionDescr().getAdditionalActionCount(); i++)
		{
			ssAddAdditionalAction(
				script->getActionDescr().getAdditionalActionDescr(i),
				script->getActionDescr().getAdditionalActionProbability(i));
		}
	}
	else
	{
		ssDefaultAction = script->getActionDescr();
		ssDefaultActionLabel->setText(ssDefaultAction.getActionName());
	}

	switch (script->getCameraType())
	{
	case CameraDescr::LinearCamera:
		ssCameraTypeComboBox->setCurrentIndex(0);
		break;
	case CameraDescr::SkyBoxCamera:
		ssCameraTypeComboBox->setCurrentIndex(1);
		break;
	}
	ssCameraResolutionSpinBox->setValue(script->getCameraResolution());
}

void MainWindow::ssAddAdditionalAction(const ActionDescr &actionDescr, float probability)
{
	int row = ssActionsTableWidget->rowCount();
	ssActionsTableWidget->insertRow(row);
	QTableWidgetItem *actionItem = 
		new QTableWidgetItem(actionDescr.getActionName());
	actionItem->setFlags(actionItem->flags() & ~Qt::ItemIsEditable);
	actionItem->setData(Qt::UserRole, QVariant::fromValue(actionDescr));
	ssActionsTableWidget->setItem(row, 0, actionItem);
	ssActionsTableWidget->setItem(row, 1, new QTableWidgetItem(QString("%1").arg(probability)));
}

SunDescr MainWindow::ssGetSunDescr()
{
	SunDescr result;
	result.setAzimuth(ssAzimuthSpinBox->value());
	result.setAltitude(ssAltitudeSpinBox->value());

	return result;
}

ActionDescr MainWindow::ssGetActionDescr()
{
	if (ssActionsTableWidget->rowCount() == 0)
		return ssDefaultAction;

	ActionDescr result;
	result.setActionType(ActionDescr::ActionAlternative);
	result.setDefaultActionDescr(&ssDefaultAction);
	
	for (int row = 0; row < ssActionsTableWidget->rowCount(); row++)
	{
		result.appendAdditionalAction(
			ssActionsTableWidget->item(row, 0)->data(Qt::UserRole).value<ActionDescr>(),
			ssActionsTableWidget->item(row, 1)->text().toFloat());
	}

	return result;
}

CameraDescr MainWindow::ssGetCameraDescr()
{
	CameraDescr result;

	switch (ssCameraTypeComboBox->currentIndex())
	{
	case 0: // Linear camera
		result.setCameraType(CameraDescr::LinearCamera);
		break;
	case 1: // Sky Box camera
		result.setCameraType(CameraDescr::SkyBoxCamera);
		break;
	}

	result.setResolution(ssCameraResolutionSpinBox->value());

	return result;
}
