/* ----------------------------------------------------------------------------
This file is part of BlueSky

Copyright (C) 2012 Piotr Polesiuk

Permission is hereby granted, free of charge, to any person obtaining a copy 
of this software and associated documentation files (the "Software"), to deal 
in the Software without restriction, including without limitation the rights 
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
copies of the Software, and to permit persons to whom the Software is 
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in 
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
THE SOFTWARE.
---------------------------------------------------------------------------- */

#include "newSimulationWizard.h"

#include <QtXml/QtXml>

#include "editActionDialog.h"
#include "simulation.h"
#include "probabilitySpinBoxDelegate.h"
#include "nullErrorOutput.h"
#include "blueSkyXmlScript.h"

NewSimulationWizard::NewSimulationWizard(QWidget *parent)
	: QWizard(parent)
{
	introPage = new NewSimulationWizardPages::IntroPage;
	actionsPage = new NewSimulationWizardPages::ActionsPage;
	cameraSettingsPage = new NewSimulationWizardPages::CameraSettingsPage;
	sunSettingsPage = new NewSimulationWizardPages::SunSettingsPage;
	exampleSimPage = new NewSimulationWizardPages::ExampleSimPage;
	conclusionPage = new NewSimulationWizardPages::ConclusionPage;

	setPage(Page_Intro, introPage);
	setPage(Page_Actions, actionsPage);
	setPage(Page_CameraSettings, cameraSettingsPage);
	setPage(Page_SunSettings, sunSettingsPage);
	setPage(Page_ExampleSim, exampleSimPage);
	setPage(Page_Conclusion, conclusionPage);

	setStartId(Page_Intro);

	setWindowTitle(tr("Simulation Wizard"));
}

QString NewSimulationWizard::getScriptCode()
{
	switch (introPage->nextId())
	{
	case Page_Conclusion:
		return
			"<!DOCTYPE BXS>\n"
			"<BXS>\n"
			" <Sun>\n"
			"  <Position azimuth=\"0\" altitude=\"0\"/>\n"
			" </Sun>\n"
			" <Action>\n"
			"  <IdentityAction/>\n"
			" </Action>\n"
			" <Camera>\n"
			"  <LinearCamera resolution=\"1024\"/>\n"
			" </Camera>\n"
			" <Description>\n"
			"  <HTML>\n"
			"  </HTML>\n"
			" </Description>\n"
			"</BXS>";
	case Page_Actions:
		{
			QDomDocument doc("BXS");
			QDomElement bxsNode = doc.createElement("BXS");
			doc.appendChild(bxsNode);

			bxsNode.appendChild(sunSettingsPage->getSunDescr().getDomElement());

			QDomElement actionNode = doc.createElement("Action");
			bxsNode.appendChild(actionNode);
			actionNode.appendChild(actionsPage->getActionDescr().getDomElement());

			QDomElement cameraNode = doc.createElement("Camera");
			bxsNode.appendChild(cameraNode);
			cameraNode.appendChild(cameraSettingsPage->getCameraDescr().getDomElement());

			QDomElement descriptionNode = doc.createElement("Description");
			bxsNode.appendChild(descriptionNode);

			QDomElement descriptionHtmlNode = doc.createElement("HTML");
			descriptionNode.appendChild(descriptionHtmlNode);

			return doc.toString();
		}
	case Page_ExampleSim:
		return exampleSimPage->getScriptCode();
	default:
		return "";
	}
}

NewSimulationWizardPages::IntroPage::IntroPage(QWidget *parent)
	: QWizardPage(parent)
{
	setTitle(tr("Introduction"));
	
	topLabel = new QLabel(tr("This wizard will help you create new simulation. "
		"Select which simulation you want to create."));
	topLabel->setWordWrap(true);

	minimalScriptRadioButton = new QRadioButton(
		tr("Create minimal simulation script."));
	simWizardRadioButton = new QRadioButton(
		tr("Create simulation using this wizard"));
	exampleSimRadioButton = new QRadioButton(
		tr("Open example simulation."));

	exampleSimRadioButton->setChecked(true);

	QVBoxLayout *layout = new QVBoxLayout;
	layout->addWidget(topLabel);
	layout->addWidget(minimalScriptRadioButton);
	layout->addWidget(simWizardRadioButton);
	layout->addWidget(exampleSimRadioButton);
	setLayout(layout);
}

int NewSimulationWizardPages::IntroPage::nextId() const
{
	if (minimalScriptRadioButton->isChecked())
		return NewSimulationWizard::Page_Conclusion;
	else if (simWizardRadioButton->isChecked())
		return NewSimulationWizard::Page_Actions;
	else if (exampleSimRadioButton->isChecked())
		return NewSimulationWizard::Page_ExampleSim;
	else
		return NewSimulationWizard::Page_Conclusion;
}

NewSimulationWizardPages::ActionsPage::ActionsPage(QWidget *parent)
	: QWizardPage(parent)
{
	defaultAction = ActionDescr();

	setTitle(tr("Select Action"));

	topLabel = new QLabel(tr("Select action preformed as default and "
		"optionally select additional action and their probabilities of "
		"occurence."));
	topLabel->setWordWrap(true);

	defaultActionGroupBox = new QGroupBox(tr("Default Action"));
	defaultActionLabel = new QLabel(tr("Identity action"));
	editDefaultActionButton = new QPushButton(tr("Edit"));
	connect(editDefaultActionButton, SIGNAL(pressed()), this, SLOT(editDefaultAction()));

	QGridLayout *dalayout = new QGridLayout;
	dalayout->addWidget(defaultActionLabel, 0, 0);
	dalayout->addWidget(editDefaultActionButton, 0, 1);
	dalayout->setColumnStretch(0, 1);
	defaultActionGroupBox->setLayout(dalayout);

	additionalActionsGroupBox = new QGroupBox(tr("Additional Actions"));
	QStringList labelList;
	labelList.append(tr("Action"));
	labelList.append(tr("Probability"));
	actionTableWidget = new QTableWidget;
	actionTableWidget->setColumnCount(2);
	actionTableWidget->setHorizontalHeaderLabels(labelList);
	actionTableWidget->setItemDelegateForColumn(1, 
		new ProbabilitySpinBoxDelegate());

	addActionButton = new QPushButton(tr("Add"));
	editActionButton = new QPushButton(tr("Edit"));
	removeActionButton = new QPushButton(tr("Remove"));
	removeAllActionsButton = new QPushButton(tr("Remove all"));
	connect(addActionButton, SIGNAL(pressed()), this, SLOT(addAction()));
	connect(editActionButton, SIGNAL(pressed()), this, SLOT(editAction()));
	connect(removeActionButton, SIGNAL(pressed()), this, SLOT(removeAction()));
	connect(removeAllActionsButton, SIGNAL(pressed()), this, SLOT(removeAllActions()));

	QGridLayout *aalayout = new QGridLayout;
	aalayout->addWidget(actionTableWidget, 0, 0, 5, 1);
	aalayout->addWidget(addActionButton, 0, 1);
	aalayout->addWidget(editActionButton, 1, 1);
	aalayout->addWidget(removeActionButton, 2, 1);
	aalayout->addWidget(removeAllActionsButton, 3, 1);
	additionalActionsGroupBox->setLayout(aalayout);

	QVBoxLayout *layout = new QVBoxLayout;
	layout->addWidget(topLabel);
	layout->addWidget(defaultActionGroupBox);
	layout->addWidget(additionalActionsGroupBox);
	setLayout(layout);
}
		
ActionDescr NewSimulationWizardPages::ActionsPage::getActionDescr()
{
	if (actionTableWidget->rowCount() == 0)
		return defaultAction;

	ActionDescr result;
	result.setActionType(ActionDescr::ActionAlternative);
	result.setDefaultActionDescr(&defaultAction);

	for (int row = 0; row < actionTableWidget->rowCount(); row++)
	{
		result.appendAdditionalAction(
			actionTableWidget->item(row, 0)->data(Qt::UserRole).value<ActionDescr>(),
			actionTableWidget->item(row, 1)->text().toFloat());
	}

	return result;
}

int NewSimulationWizardPages::ActionsPage::nextId() const
{
	return NewSimulationWizard::Page_CameraSettings;
}

void NewSimulationWizardPages::ActionsPage::editDefaultAction()
{
	EditActionDialog dialog(defaultAction);

	if (dialog.exec())
	{
		defaultActionLabel->setText(dialog.getActionName());
		defaultAction = dialog.getActionDescr();
	}
}

void NewSimulationWizardPages::ActionsPage::addAction()
{
	EditActionDialog dialog;
	if (dialog.exec())
	{
		int row = actionTableWidget->rowCount();
		actionTableWidget->insertRow(row);
		QTableWidgetItem *actionItem = 
			new QTableWidgetItem(dialog.getActionName());
		actionItem->setFlags(actionItem->flags() & ~Qt::ItemIsEditable);
		actionItem->setData(Qt::UserRole, QVariant::fromValue(dialog.getActionDescr()));
		actionTableWidget->setItem(row, 0, actionItem);
		actionTableWidget->setItem(row, 1, new QTableWidgetItem("0"));
	}
}

void NewSimulationWizardPages::ActionsPage::editAction()
{
	int row = actionTableWidget->currentRow();
	if (row < 0)
		return;
	QTableWidgetItem *actionItem = actionTableWidget->item(row, 0);
	EditActionDialog dialog(actionItem->data(Qt::UserRole).value<ActionDescr>());
	if (dialog.exec())
	{
		actionItem->setData(Qt::UserRole, QVariant::fromValue(dialog.getActionDescr()));
		actionItem->setText(dialog.getActionName());
	}
}

void NewSimulationWizardPages::ActionsPage::removeAction()
{
	int row = actionTableWidget->currentRow();
	if (row < 0)
		return;
	actionTableWidget->removeRow(row);
}

void NewSimulationWizardPages::ActionsPage::removeAllActions()
{
	while (actionTableWidget->rowCount() > 0)
		actionTableWidget->removeRow(0);
}

NewSimulationWizardPages::CameraSettingsPage::CameraSettingsPage(QWidget *parent)
	: QWizardPage(parent)
{
	setTitle(tr("Camera Settings"));

	topLabel = new QLabel(tr("Choose one type of camera and its resolution."));
	topLabel->setWordWrap(true);

	cameraDescription = new QLabel("");
	cameraDescription->setWordWrap(true);

	cameraTypeComboBox = new QComboBox();
	cameraTypeComboBox->addItem(tr("Linear Camera"));
	cameraTypeComboBox->addItem(tr("Sky Box Camera"));
	connect(cameraTypeComboBox, SIGNAL(currentIndexChanged(int)), this, SLOT(cameraSelected(int)));

	cameraResolutionSpinBox = new QSpinBox();
	cameraResolutionSpinBox->setRange(1, 1000000);
	cameraResolutionSpinBox->setSingleStep(1);
	cameraResolutionSpinBox->setValue(1024);

	QGridLayout *layout = new QGridLayout();
	layout->addWidget(topLabel, 0, 0, 1, 2);
	layout->addWidget(new QLabel("Camera type:"), 1, 0);
	layout->addWidget(cameraTypeComboBox, 2, 0);
	layout->addWidget(new QLabel("Camera resolution:"), 3, 0);
	layout->addWidget(cameraResolutionSpinBox, 4, 0);
	layout->addWidget(cameraDescription, 1, 1, 5, 1);
	setLayout(layout);

	cameraSelected(0);
}

CameraDescr NewSimulationWizardPages::CameraSettingsPage::getCameraDescr()
{
	CameraDescr result;
	switch (cameraTypeComboBox->currentIndex())
	{
	case 0: // LinearCamera
		result.setCameraType(CameraDescr::LinearCamera);
		break;
	case 1: // SkyBoxCamera
		result.setCameraType(CameraDescr::SkyBoxCamera);
		break;
	default:
		break;
	}
	result.setResolution(cameraResolutionSpinBox->value());
	return result; 
}

int NewSimulationWizardPages::CameraSettingsPage::nextId() const
{
	return NewSimulationWizard::Page_SunSettings;
}

void NewSimulationWizardPages::CameraSettingsPage::cameraSelected(int index)
{
	switch (index)
	{
	case 0: // Linear camera
		cameraDescription->setText(tr("A linear camera is a 1D camera which "
			"allows to generate high quality images in relatively short time. "
			"It's disadventage is that the camera can be used only to capture "
			"view from an axial-symmetry phonomena, e.g. rainbow, halo."));
		cameraResolutionSpinBox->setRange(1, 1000000);
		break;
	case 1: // Sky box camera
		cameraDescription->setText(tr("A sky box camera is a 2D camera whose "
			"field of view covers the whole sky. High quality images are "
			"usually burdened with noise. That's why if you want to generate "
			"an axial-symmetry phenomena, use a linear camera."));
		cameraResolutionSpinBox->setRange(1, 10000);
		break;
	default:
		break;
	}
}

NewSimulationWizardPages::SunSettingsPage::SunSettingsPage(QWidget *parent)
	: QWizardPage(parent)
{
	setTitle(tr("Sun settings"));

	topLabel = new QLabel(
		tr("Set azimuth and altitude of the sun. Different sun altitudes "
		"can cause different nature of some phenomena. e.g. circumzenithal "
		"arc is invisible when the sun is too high."));
	topLabel->setWordWrap(true);
	azimuthLabel = new QLabel(tr("Azimuth [deg]"));
	altitudeLabel = new QLabel(tr("Altitude [deg]"));

	azimuthSpinBox = new QDoubleSpinBox();
	azimuthSpinBox->setDecimals(1);
	azimuthSpinBox->setRange(0.0, 360.0);
	azimuthSpinBox->setSingleStep(1.0);
	azimuthSpinBox->setWrapping(true);

	altitudeSpinBox = new QDoubleSpinBox();
	altitudeSpinBox->setDecimals(1);
	altitudeSpinBox->setRange(-90.0, 90.0);
	altitudeSpinBox->setSingleStep(1.0);

	QGridLayout *layout = new QGridLayout;
	layout->addWidget(topLabel, 0, 0, 1, 2);
	layout->addWidget(azimuthLabel, 1, 0);
	layout->addWidget(altitudeLabel, 2, 0);
	layout->addWidget(azimuthSpinBox, 1, 1);
	layout->addWidget(altitudeSpinBox, 2, 1);
	setLayout(layout);
}

int NewSimulationWizardPages::SunSettingsPage::nextId() const
{
	return NewSimulationWizard::Page_Conclusion;
}

SunDescr NewSimulationWizardPages::SunSettingsPage::getSunDescr()
{
	SunDescr result;
	result.setAzimuth(azimuthSpinBox->value());
	result.setAltitude(altitudeSpinBox->value());
	return result;
}

NewSimulationWizardPages::ExampleSimPage::ExampleSimPage(QWidget *parent)
	: QWizardPage(parent)
{
	setTitle(tr("Example simulation"));
	
	topLabel = new QLabel(tr("Select example simulation which you want to open."));

	examplesListWidget = new QListWidget();
	examplesListWidget->setSelectionMode(QAbstractItemView::SingleSelection);
	connect(examplesListWidget, SIGNAL(currentRowChanged(int)),
		this, SLOT(selectedScriptChanged(int)));

	exampleDescription = new QLabel(tr("No examples."));
	exampleDescription->setWordWrap(true);
	exampleDescription->setAlignment(Qt::AlignTop);

	QGridLayout *layout = new QGridLayout;
	layout->addWidget(topLabel, 0, 0, 1, 2);
	layout->addWidget(examplesListWidget, 1, 0);
	layout->addWidget(exampleDescription, 1, 1);
	layout->setColumnStretch(0, 10);
	layout->setColumnStretch(1, 10);
	setLayout(layout);

	registerField("example*", examplesListWidget);

	findExamples();
}

int NewSimulationWizardPages::ExampleSimPage::nextId() const
{
	return NewSimulationWizard::Page_Conclusion;
}

QString NewSimulationWizardPages::ExampleSimPage::getScriptCode() const
{
	if (examplesListWidget->currentRow() < 0)
		return "";

	QFile file(examplesPaths[examplesListWidget->currentRow()]);
	if (!file.open(QFile::ReadOnly | QFile::Text))
	{
		return "";
	}

	QTextStream in(&file);
	in.setCodec("UTF-8");
	return in.readAll();
}

void NewSimulationWizardPages::ExampleSimPage::selectedScriptChanged(int index)
{
	if (index < 0)
		return;

	exampleDescription->setText(examplesDescriptions[index]);
}

void NewSimulationWizardPages::ExampleSimPage::findExamples()
{
	QDir examplesDir("examples");
	if (!examplesDir.exists())
		return;

	QStringList filters;
	filters.append("*.bxs");
	QStringList exampleFiles = examplesDir.entryList(filters, QDir::NoFilter, QDir::Name);

	NullErrorOutput output;

	for (int i = 0; i < exampleFiles.count(); i++)
	{
		QString path = examplesDir.absoluteFilePath(exampleFiles[i]);
		BlueSkyXmlScript *script = BlueSkyXmlScript::fromFile(path, &output);

		if (script == NULL)
			continue;

		script->parse(&output);
		
		examplesDescriptions.append(script->getBriefDescription());
		examplesPaths.append(path);
		QString name = script->getScriptName();
		if (name.isEmpty())
			name = tr("<User script>");
		examplesListWidget->addItem(name);

		delete script;
	}
}

NewSimulationWizardPages::ConclusionPage::ConclusionPage(QWidget *parent)
	: QWizardPage(parent)
{
	setTitle(tr("Conclusion"));

	topLabel = new QLabel(
		tr("Simulation script will be created. Click 'Finish' to close this "
		"wizard, and next click 'Start' to see simulation results, or edit "
		"the simulation."));
	topLabel->setWordWrap(true);

	QVBoxLayout *layout = new QVBoxLayout;
	layout->addWidget(topLabel);
	setLayout(layout);
}

int NewSimulationWizardPages::ConclusionPage::nextId() const
{
	return -1;
}
