/* ----------------------------------------------------------------------------
This file is part of BlueSky

Copyright (C) 2012 Piotr Polesiuk

Permission is hereby granted, free of charge, to any person obtaining a copy 
of this software and associated documentation files (the "Software"), to deal 
in the Software without restriction, including without limitation the rights 
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
copies of the Software, and to permit persons to whom the Software is 
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in 
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
THE SOFTWARE.
---------------------------------------------------------------------------- */

#include "refractionDescr.h"

#include "errorOutput.h"
#include <QtCore/QObject>
#include <BSRefractionConst.h>

RefractionDescr::RefractionDescr()
{
	type = RefractionConst;
	constRefractiveIndex = 1.0f;
}

RefractionDescr::RefractionDescr(const RefractionDescr &refractionDescr)
{
	type                 = refractionDescr.type;
	constRefractiveIndex = refractionDescr.constRefractiveIndex;
	powerRIShift         = refractionDescr.powerRIShift;
	powerRIScale         = refractionDescr.powerRIScale;
	powerRIPower         = refractionDescr.powerRIPower;
	material             = refractionDescr.material;
}

RefractionDescr::~RefractionDescr()
{
}

const RefractionDescr &RefractionDescr::operator = (const RefractionDescr &refractionDescr)
{
	type                 = refractionDescr.type;
	constRefractiveIndex = refractionDescr.constRefractiveIndex;
	powerRIShift         = refractionDescr.powerRIShift;
	powerRIScale         = refractionDescr.powerRIScale;
	powerRIPower         = refractionDescr.powerRIPower;
	material             = refractionDescr.material;

	return *this;
}

RefractionDescr::RefractionType RefractionDescr::getRefractionType() const
{
	return type;
}

void RefractionDescr::setRefractionType(RefractionDescr::RefractionType _type)
{
	type = _type;
}

float RefractionDescr::getConstRefractiveIndex() const
{
	return constRefractiveIndex;
}

void RefractionDescr::setConstRefractiveIndex(float refractiveIndex)
{
	constRefractiveIndex = refractiveIndex;
}

float RefractionDescr::getPowerRIShift() const
{
	return powerRIShift;
}

float RefractionDescr::getPowerRIScale() const
{
	return powerRIScale;
}

float RefractionDescr::getPowerRIPower() const
{
	return powerRIPower;
}

void RefractionDescr::setPowerRI(float shift, float scale, float power)
{
	powerRIShift = shift;
	powerRIScale = scale;
	powerRIPower = power;
}

float RefractionDescr::getPowerRIRed() const
{
	return powerRIShift + powerRIScale * BlueSky::Math::pow(700.0f, powerRIPower);
}

float RefractionDescr::getPowerRIGreen() const
{
	return powerRIShift + powerRIScale * BlueSky::Math::pow(550.0f, powerRIPower);
}

float RefractionDescr::getPowerRIBlue() const
{
	return powerRIShift + powerRIScale * BlueSky::Math::pow(400.0f, powerRIPower);
}

void RefractionDescr::setPowerRIRgb(float rn, float gn, float bn)
{
	fitRgb2powerRI(rn, gn, bn);
}

BlueSky::RefractionPower::Material RefractionDescr::getMaterial() const
{
	return material;
}

void RefractionDescr::setMaterial(BlueSky::RefractionPower::Material _material)
{
	material = _material;
}

BlueSky::Refraction *RefractionDescr::getRefraction() const
{
	switch (type)
	{
	case RefractionConst:
		return new BlueSky::RefractionConst(constRefractiveIndex);
	case RefractionPower:
		return new BlueSky::RefractionPower(powerRIShift, powerRIScale, powerRIPower);
	case PredefinedMaterial:
		return new BlueSky::RefractionPower(material);
	default:
		return NULL;
	}
}

QDomElement RefractionDescr::getDomElement() const
{
	QDomDocument doc;
	QDomElement result;

	switch (type)
	{
	case RefractionConst:
		result = doc.createElement("RefractionConst");
		result.setAttribute("refractiveIndex", constRefractiveIndex);
		break;
	case RefractionPower:
		result = doc.createElement("RefractionPower");
		result.setAttribute("shift", powerRIShift);
		result.setAttribute("scale", powerRIScale);
		result.setAttribute("power", powerRIPower);
		break;
	case PredefinedMaterial:
		result = doc.createElement("RefractionPower");
		switch (material)
		{
		case BlueSky::RefractionPower::DIAMOND:
			result.setAttribute("material", "DIAMOND");
			break;
		case BlueSky::RefractionPower::ICE:
			result.setAttribute("material", "ICE");
			break;
		case BlueSky::RefractionPower::WATER:
			result.setAttribute("material", "WATER");
			break;
		default:
			break;
		}
		break;
	default:
		result = doc.createElement("Unknown");
		break;
	}

	return result;
}

RefractionDescr RefractionDescr::fromDomElement(QDomElement node, QString nodeName, ErrorOutput *output)
{
	RefractionDescr result;
	QDomElement refrNode = (nodeName.isEmpty() ? node : node.firstChildElement(nodeName));

	if (refrNode.isNull())
	{
		output->error(QObject::tr("No %1 node!").arg(nodeName));
		return result;
	}

	refrNode = refrNode.firstChildElement();
	if (refrNode.isNull())
	{
		output->error(QObject::tr("No refraction specified."));
		return result;
	}

	if (refrNode.tagName() == "RefractionConst")
	{
		result.type = RefractionConst;
		if (refrNode.hasAttribute("refractiveIndex"))
		{
			bool ok;
			result.constRefractiveIndex = 
				refrNode.attributeNode("refractiveIndex").value().toFloat(&ok);
			if (!ok)
				output->error(QObject::tr(
					"RefractionConst/refractiveIndex : this is not valid number."));
			else if (result.constRefractiveIndex == 0.0f)
			{
				output->warning(QObject::tr(
					"Refractive index should be different than 0.0."));
				result.constRefractiveIndex = 1.0f;
			}
		}
		else
			output->error(QObject::tr("RefractionConst : no refractiveIndex attribute."));
	}
	else if (refrNode.tagName() == "RefractionPower")
	{
		if (refrNode.hasAttribute("material"))
		{
			result.type = PredefinedMaterial;
			QString mat = refrNode.attributeNode("material").value();
			if (mat == "DIAMOND")
				result.material = BlueSky::RefractionPower::DIAMOND;
			else if (mat == "ICE")
				result.material = BlueSky::RefractionPower::ICE;
			else if (mat == "WATER")
				result.material = BlueSky::RefractionPower::WATER;
			else
				output->error(QObject::tr("Unknown material : %1").arg(mat));
		}
		else if (refrNode.hasAttribute("shift") 
			&& refrNode.hasAttribute("scale") 
			&& refrNode.hasAttribute("power"))
		{
			result.type = RefractionPower;
			bool ok;
			result.powerRIShift = refrNode.attributeNode("shift").value().toFloat(&ok);
			if (!ok)
				output->error(QObject::tr(
					"RefractionPower/shift : this is not valid number."));
			result.powerRIScale = refrNode.attributeNode("scale").value().toFloat(&ok);
			if (!ok)
				output->error(QObject::tr(
					"RefractionPower/scale : this is not valid number."));
			result.powerRIPower = refrNode.attributeNode("power").value().toFloat(&ok);
			if (!ok)
				output->error(QObject::tr(
					"RefractionPower/power : this is not valid number."));
			if (result.powerRIShift == 0.0f && result.powerRIScale == 0.0f)
				output->warning("shift and scale should not be both set to 0.");
		}
		else
			output->error(QObject::tr(
				"RefractionPower : no material or shift,scale,power attributes."));
	}
	else
		output->error(QObject::tr(
			"Unknown refraction model : %1.").arg(refrNode.tagName()));

	return result;
}

bool RefractionDescr::fitRgb2powerRI(float rn, float gn, float bn)
{
	if (rn > bn)
	{
		if (fitRgb2powerRI(-rn, -gn, -bn))
		{
			powerRIShift = -powerRIShift;
			powerRIScale = -powerRIScale;
			return true;
		}
		else
			return false;
	}

	if (rn == bn)
	{
		if (gn == rn)
		{
			powerRIShift = rn;
			powerRIScale = 0.0f;
			powerRIPower = 0.0f;
			return true;
		}
		else
			return false;
	}

	if (gn <= rn || gn >= bn)
		return false;

	if (2.0f * gn >= rn + bn) // positive power
	{
		float ppower = 1.0f;
		float power = 2.0f;
		float step = 1.0f;

		float a, b;
		int iter = 0;
		do
		{
			iter ++;
			float x1 = BlueSky::Math::pow(550.0f, power);
			float x2 = BlueSky::Math::pow(400.0f, power);
			a = (x2*gn - x1*bn) / (x2 - x1);
			b = (bn - gn) / (x2 - x1);

			if (a + b*BlueSky::Math::pow(700.0f, power) < rn)
			{
				step *= 0.5f;
				power = ppower + step;
			}
			else
			{
				ppower = power;
				power  = ppower + step;
			}
		} while (step > 0.0f && power != ppower && iter < 1000);

		powerRIShift = a;
		powerRIScale = b;
		powerRIPower = power;

		return true;
	}
	else // negative power
	{
		float power = -1.0f;

		float a, b;
		int iter = 0;
		while (true)
		{
			iter++;
			float x1 = BlueSky::Math::pow(550.0f, power);
			float x2 = BlueSky::Math::pow(400.0f, power);
			a = (x2*gn - x1*bn) / (x2 - x1);
			b = (bn - gn) / (x2 - x1);

			if (a + b*BlueSky::Math::pow(700.0f, power) < rn || iter > 1000)
				break;
			power *= 0.5;
		}

		float ppower = power;
		float step = 1.0f;
		power = ppower - step;

		iter = 0;
		do
		{
			iter++;
			float x1 = BlueSky::Math::pow(550.0f, power);
			float x2 = BlueSky::Math::pow(400.0f, power);
			a = (x2*gn - x1*bn) / (x2 - x1);
			b = (bn - gn) / (x2 - x1);

			if (a + b*BlueSky::Math::pow(700.0f, power) > rn)
			{
				step *= 0.5f;
				power = ppower - step;
			}
			else
			{
				ppower = power;
				power  = ppower - step;
			}
		} while (step > 0.0f && power != ppower && iter < 1000);

		powerRIShift = a;
		powerRIScale = b;
		powerRIPower = power;

		return true;
	}
}
