/* ----------------------------------------------------------------------------
This file is part of BlueSky

Copyright (C) 2012 Piotr Polesiuk

Permission is hereby granted, free of charge, to any person obtaining a copy 
of this software and associated documentation files (the "Software"), to deal 
in the Software without restriction, including without limitation the rights 
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
copies of the Software, and to permit persons to whom the Software is 
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in 
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
THE SOFTWARE.
---------------------------------------------------------------------------- */

#include "rotationDescr.h"

#include <QtCore/QObject>
#include "errorOutput.h"
#include <BSGaussianDeviation.h>
#include <BSRandomUniformRotation.h>
#include <BSRotationAlternative.h>
#include <BSRotationX90Y.h>
#include <BSRotationX90ZY.h>
#include <BSRotationXY.h>
#include <BSRotationY.h>
#include <BSRotationZY.h>

RotationDescr::RotationDescr()
{
	type = RandomUniformRotation;
	deviated = false;
	deviation = 0.0f;
	defaultRotation = NULL;
}

RotationDescr::RotationDescr(const RotationDescr &rotationDescr)
{
	type = rotationDescr.type;
	deviated = rotationDescr.deviated;
	deviation = rotationDescr.deviation;
	if (rotationDescr.defaultRotation)
		defaultRotation = new RotationDescr(*rotationDescr.defaultRotation);
	else
		defaultRotation = NULL;
	additionalRotations = rotationDescr.additionalRotations;
	additionalProbabilities = rotationDescr.additionalProbabilities;
}

RotationDescr::~RotationDescr()
{
	if (defaultRotation)
		delete defaultRotation;
}

const RotationDescr &RotationDescr::operator = (const RotationDescr &rotationDescr)
{
	type = rotationDescr.type;
	deviated = rotationDescr.deviated;
	deviation = rotationDescr.deviation;
	if (defaultRotation)
		delete defaultRotation;
	if (rotationDescr.defaultRotation)
		defaultRotation = new RotationDescr(*rotationDescr.defaultRotation);
	else
		defaultRotation = NULL;
	additionalRotations = rotationDescr.additionalRotations;
	additionalProbabilities = rotationDescr.additionalProbabilities;
	return *this;
}

RotationDescr::RotationType RotationDescr::getRotationType() const
{
	return type;
}

void RotationDescr::setRotationType(RotationDescr::RotationType _type)
{
	type = _type;
}

bool RotationDescr::isDeviated() const
{
	return deviated;
}

void RotationDescr::setDeviated(bool _deviated)
{
	deviated = _deviated;
}

float RotationDescr::getDeviation() const
{
	return deviation;
}

void RotationDescr::setDeviation(float _deviation)
{
	deviation = _deviation;
}

RotationDescr *RotationDescr::getDefaultRotationDescr() const
{
	return defaultRotation;
}

void RotationDescr::setDefaultRotationDescr(RotationDescr *rotationDescr)
{
	if (defaultRotation)
		delete defaultRotation;
	if (rotationDescr)
		defaultRotation = new RotationDescr(*rotationDescr);
	else
		defaultRotation = NULL;
}

int RotationDescr::getAdditionalRotationCount() const
{
	return additionalRotations.count();
}

const RotationDescr &RotationDescr::getAdditionalRotationDescr(int index) const
{
	return additionalRotations[index];
}

float RotationDescr::getAdditionalRotationProbability(int index) const
{
	return additionalProbabilities[index];
}

void RotationDescr::appendAdditionalRotation(const RotationDescr &rotationDescr, float probability)
{
	additionalRotations.append(rotationDescr);
	additionalProbabilities.append(probability);
}

BlueSky::Rotation *RotationDescr::getRotation() const
{
	BlueSky::Rotation *result;

	switch (type)
	{
	case RotationAlternative:
		{
			BlueSky::RotationAlternative *res;
			if (defaultRotation)
				res = new BlueSky::RotationAlternative(defaultRotation->getRotation());
			else
				res = new BlueSky::RotationAlternative();
			for (int i = 0; i < additionalRotations.count(); i++)
				res->addRotation(additionalRotations[i].getRotation(), additionalProbabilities[i]);
			result = res;
			break;
		}
	case RandomUniformRotation:
		result = new BlueSky::RandomUniformRotation();
		break;
	case RotationX90Y:
		result = new BlueSky::RotationX90Y();
		break;
	case RotationX90ZY:
		result = new BlueSky::RotationX90ZY();
		break;
	case RotationXY:
		result = new BlueSky::RotationXY();
		break;
	case RotationY:
		result = new BlueSky::RotationY();
		break;
	case RotationZY:
		result = new BlueSky::RotationZY();
		break;
	default:
		result = NULL;
	}

	if (result && deviated)
		result = new BlueSky::GaussianDeviation(deviation, result);

	return result;
}

QDomElement RotationDescr::getDomElement() const
{
	QDomDocument doc;
	QDomElement result;

	switch (type)
	{
	case RotationAlternative:
		{
			result = doc.createElement("RotationAlternative");
			if (defaultRotation)
			{
				QDomElement defRotNode = doc.createElement("DefaultRotation");
				result.appendChild(defRotNode);
				defRotNode.appendChild(defaultRotation->getDomElement());
			}
			for (int i = 0; i < additionalRotations.count(); i++)
			{
				QDomElement addRotNode = doc.createElement("AdditionalRotation");
				result.appendChild(addRotNode);
				addRotNode.setAttribute("probability", additionalProbabilities[i]);
				addRotNode.appendChild(additionalRotations[i].getDomElement());
			}
			break;
		}
	case RandomUniformRotation:
		result = doc.createElement("RandomUniformRotation");
		break;
	case RotationX90Y:
		result = doc.createElement("RotationX90Y");
		break;
	case RotationX90ZY:
		result = doc.createElement("RotationX90ZY");
		break;
	case RotationXY:
		result = doc.createElement("RotationXY");
		break;
	case RotationY:
		result = doc.createElement("RotationY");
		break;
	case RotationZY:
		result = doc.createElement("RotationZY");
		break;
	}

	if (deviated)
	{
		result.setAttribute("deviation", deviation);
	}

	return result;
}

QString RotationDescr::getRotationName() const
{
	switch (type)
	{
	case RotationAlternative:
		return QObject::tr("Alternative of rotations");
	case RandomUniformRotation:
		return QObject::tr("Random rotation");
	case RotationX90Y:
		return QObject::tr("Parry crystals");
	case RotationX90ZY:
		return QObject::tr("Column crystals");
	case RotationXY:
		return QObject::tr("Liwitz crystals");
	case RotationY:
		return QObject::tr("Plate crystals");
	case RotationZY:
		return QObject::tr("RotationZY");
	}
	return "Unknown";
}

RotationDescr RotationDescr::fromDomElement(QDomElement node, QString nodeName, ErrorOutput *output)
{
	RotationDescr result;
	QDomElement rotationNode = (nodeName.isEmpty() ? node : node.firstChildElement(nodeName));

	if (rotationNode.isNull())
	{
		output->error(QObject::tr("No %1 node!").arg(nodeName));
		return result;
	}

	rotationNode = rotationNode.firstChildElement();
	if (rotationNode.isNull())
	{
		output->error(QObject::tr("No action specified."));
		return result;
	}

/*	if (rotationNode.tagName() == "ConstRotation")
	{
		if (rotationNode.hasAttribute("angle"))
		{
			float angle = rotationNode.attributeNode("angle").value().toFloat();
			float axis_x = rotationNode.attributeNode("axis_x").value().toFloat();
			float axis_y = rotationNode.attributeNode("axis_y").value().toFloat();
			float axis_z = rotationNode.attributeNode("axis_z").value().toFloat();

			BlueSky::Vector3f axis = BlueSky::Vector3f(axis_x, axis_y, axis_z);
			if (axis == BlueSky::Vector3f::ZERO)
				axis = BlueSky::Vector3f::UNIT_Y;

			return new BlueSky::ConstRotation(angle, axis.normalised());
		}
		else if (rotationNode.hasAttribute("w") &&
			rotationNode.hasAttribute("x") &&
			rotationNode.hasAttribute("y") &&
			rotationNode.hasAttribute("z"))
		{
			float w = rotationNode.attributeNode("w").value().toFloat();
			float x = rotationNode.attributeNode("x").value().toFloat();
			float y = rotationNode.attributeNode("y").value().toFloat();
			float z = rotationNode.attributeNode("z").value().toFloat();

			BlueSky::Quaternionf q = BlueSky::Quaternionf(w, x, y, z);
			if (q == BlueSky::Quaternionf::ZERO)
				q = BlueSky::Quaternionf::UNIT;

			return new BlueSky::ConstRotation(q);
		}
		else
			return new BlueSky::ConstRotation();
	}
	else if (rotationNode.tagName() == "GaussianDeviation")
	{
		float angle = rotationNode.attributeNode("angle").value().toFloat();
		BlueSky::Rotation *baseRotation = getRotation(rotationNode, "BaseRotation");
		if (baseRotation)
			return new BlueSky::GaussianDeviation(angle, baseRotation);
		else
			return NULL;
	}*/
	if (rotationNode.tagName() == "RandomUniformRotation")
		result.type = RandomUniformRotation;
	else if (rotationNode.tagName() == "RotationAlternative")
	{
		result.type = RotationAlternative;
		RotationDescr defaultRot = 
			RotationDescr::fromDomElement(rotationNode, "DefaultRotation", output);
		result.setDefaultRotationDescr(&defaultRot);

		QDomElement addRotNode = rotationNode.firstChildElement("AdditionalRotation");
		while (!addRotNode.isNull())
		{
			RotationDescr additionalRot = 
				RotationDescr::fromDomElement(addRotNode, "", output);
			if (addRotNode.hasAttribute("probability"))
			{
				bool ok;
				float prob = addRotNode.attributeNode("probability").value().toFloat(&ok);
				if (!ok)
					output->error(QObject::tr(
						"RotationAlternative/AdditionalRotation/probability : this is not valid number."));

				result.appendAdditionalRotation(additionalRot, prob);
			}
			else
				output->error(QObject::tr(
					"RotationAlternative/AdditionalRotation : no probability attribute."));

			addRotNode = addRotNode.nextSiblingElement("AdditionalRotation");
		}
	}
	else if (rotationNode.tagName() == "RotationX90Y")
		result.type = RotationX90Y;
	else if (rotationNode.tagName() == "RotationX90ZY")
		result.type = RotationX90ZY;
	else if (rotationNode.tagName() == "RotationXY")
		result.type = RotationXY;
	else if (rotationNode.tagName() == "RotationY")
		result.type = RotationY;
	else if (rotationNode.tagName() == "RotationZY")
		result.type = RotationZY;
	else
		output->error(QObject::tr(
			"Unknown rotation model : %1.").arg(rotationNode.tagName()));

	if (rotationNode.hasAttribute("deviation"))
	{
		bool ok;
		result.deviated = true;
		result.deviation = rotationNode.attributeNode("deviation").value().toFloat(&ok);
		if (!ok)
			output->error(QObject::tr(
				"%1/deviation : this is not valid number.").arg(rotationNode.tagName()));
	}

	return result;
}
