/* ----------------------------------------------------------------------------
This file is part of BlueSky

Copyright (C) 2012 Piotr Polesiuk

Permission is hereby granted, free of charge, to any person obtaining a copy 
of this software and associated documentation files (the "Software"), to deal 
in the Software without restriction, including without limitation the rights 
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
copies of the Software, and to permit persons to whom the Software is 
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in 
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
THE SOFTWARE.
---------------------------------------------------------------------------- */

#include "skyViewDrawingThread.h"

#include "skyView.h"

#include <BSQuaternionf.h>

SkyViewDrawingThread::SkyViewDrawingThread(SkyView *parent)
{
	skyViewParent = parent;

	flag_abort = 0;
	flag_redraw = 0;

	dynBrightness = 1.0f;	

	connect(this, SIGNAL(redrawingComplete()), parent, SLOT(setPixmap()));

	start();
}

SkyViewDrawingThread::~SkyViewDrawingThread()
{
	flag_abort = 1;
	waitCondition.wakeOne();
	wait();
}

void SkyViewDrawingThread::redraw()
{
	flag_redraw = 1;
	waitCondition.wakeOne();
}

void SkyViewDrawingThread::run()
{
	int waitTime = 100;
	while (!flag_abort)
	{
		do
		{
			if (flag_redraw)
			{
				flag_redraw = 0;
				waitTime = 100;
			}
			else
				waitTime += waitTime >> 1;
			redrawBody();
		} while (flag_redraw);
		mutex.lock();
		waitCondition.wait(&mutex, waitTime);
		mutex.unlock();
	}
}

void SkyViewDrawingThread::redrawBody()
{
	int sizex = skyViewParent->size().width() - 10;
	int sizey = skyViewParent->size().height() - 10;

	bool stereographic = skyViewParent->stereographic;
	bool useGamma = skyViewParent->useGamma;
	float gamma = skyViewParent->gamma;
	float backgroundBrightness = skyViewParent->backgroundBrightness;
	float horizonOpacity = skyViewParent->horizonOpacity;

	if (sizex < 0 || sizey < 0)
		return;

	QImage img(sizex, sizey, QImage::Format_RGB32);
	BlueSky::Quaternionf rot =
		BlueSky::Quaternionf::yRotation(skyViewParent->yRot) *
		BlueSky::Quaternionf::xRotation(skyViewParent->xRot);

	BlueSky::Vector3f dir = rot.rotate(0.7f * BlueSky::Vector3f::UNIT_Z);
	BlueSky::Vector3f dirx = rot.rotate(0.5f * BlueSky::Vector3f::UNIT_X);
	BlueSky::Vector3f diry = rot.rotate(0.5f * BlueSky::Vector3f::UNIT_Y);

	float scale = 2.0 / ((sizex + sizey) * skyViewParent->zoomFactor);

	float newBrightness = 0.00001f;
	if (!skyViewParent->useDynamicBrightness)
		dynBrightness = skyViewParent->brightness;

	for (int y = 0; y < sizey; y++)
	{
		for (int x = 0; x < sizex; x++)
		{
			float tx = (float)(x - sizex/2)*scale;
			float ty = (float)(sizey/2 - y)*scale;

			BlueSky::Vector3f tdir;
			if (stereographic)
			{
				tx *= 0.357f;
				ty *= 0.357f;
				float d = 1.0f / (1.0f + tx*tx + ty*ty);
				tdir = rot.rotate(
					BlueSky::Vector3f(
						2.0f * tx * d, 
						2.0f * ty * d, 
						-(tx*tx + ty*ty - 1.0f) * d));
			}
			else
				tdir = 
					(dir + tx*dirx + ty*diry).normalised();
			BlueSky::Vector3f color =
				skyViewParent->camera->getDataByDirection(-tdir);

			color += BlueSky::Vector3f(0.04f, 0.2f, 1.0f) * backgroundBrightness;
			newBrightness += color.x + color.y + color.z;
			color *= dynBrightness;

			int r, g, b;
			if (useGamma)
			{
				r = (int)(255.0f * BlueSky::Math::pow(color.x, gamma));
				g = (int)(255.0f * BlueSky::Math::pow(color.y, gamma));
				b = (int)(255.0f * BlueSky::Math::pow(color.z, gamma));
			}
			else
			{
				r = (int)(255.0f * color.x);
				g = (int)(255.0f * color.y);
				b = (int)(255.0f * color.z);
			}
			if (r > 255) r = 255;
			else if (r < 0) r = 0;
			if (g > 255) g = 255;
			else if (g < 0) g = 0;
			if (b > 255) b = 255;
			else if (b < 0) b = 0;

			if (tdir.y < 0.0)
			{
				r = (int)((float)r * (1.0f - horizonOpacity));
				g = (int)((float)g * (1.0f - horizonOpacity) + horizonOpacity * 100.0f);
				b = (int)((float)b * (1.0f - horizonOpacity));
			}

			img.setPixel(x, y, qRgb(r, g, b));
		}
	}

	dynBrightness = skyViewParent->brightness * sizex * sizey / newBrightness;
	if (dynBrightness != dynBrightness)
		dynBrightness = 1.0f;

	skyViewParent->newPixmap = QPixmap::fromImage(img);
	redrawingComplete();
}
