/* ----------------------------------------------------------------------------
This file is part of BlueSky

Copyright (C) 2012 Piotr Polesiuk

Permission is hereby granted, free of charge, to any person obtaining a copy 
of this software and associated documentation files (the "Software"), to deal 
in the Software without restriction, including without limitation the rights 
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
copies of the Software, and to permit persons to whom the Software is 
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in 
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
THE SOFTWARE.
---------------------------------------------------------------------------- */

#include "sunDescr.h"

#include "errorOutput.h"
#include <QtCore/QObject>

SunDescr::SunDescr()
{
	azimuth = 0.0f;
	altitude = 30.0f;
}

SunDescr::SunDescr(const SunDescr &sunDescr)
{
	azimuth = sunDescr.azimuth;
	altitude = sunDescr.altitude;
}

SunDescr::~SunDescr()
{
}

const SunDescr &SunDescr::operator = (const SunDescr &sunDescr)
{
	azimuth = sunDescr.azimuth;
	altitude = sunDescr.altitude;
	return *this;
}

float SunDescr::getAzimuth() const
{
	return azimuth;
}

void SunDescr::setAzimuth(float _azimuth)
{
	azimuth = _azimuth;
}

float SunDescr::getAltitude() const
{
	return altitude;
}
	
void SunDescr::setAltitude(float _altitude)
{
	altitude = _altitude;
}

BlueSky::Vector3f SunDescr::getSunDirection() const
{
	float radAzimuth  = azimuth  * BlueSky::Math::PIf / 180.0f;
	float radAltitude = altitude * BlueSky::Math::PIf / 180.0f;
				
	return BlueSky::Vector3f(
		BlueSky::Math::sin(radAzimuth) * BlueSky::Math::cos(radAltitude),
		-BlueSky::Math::sin(radAltitude),
		BlueSky::Math::cos(radAzimuth) * BlueSky::Math::cos(radAltitude));
}

QDomElement SunDescr::getDomElement() const
{
	QDomDocument doc;
	QDomElement sunNode = doc.createElement("Sun");
	QDomElement directionNode = doc.createElement("Position");
	sunNode.appendChild(directionNode);
	directionNode.setAttribute("azimuth",  azimuth);
	directionNode.setAttribute("altitude", altitude);

	return sunNode;
}

SunDescr SunDescr::fromDomElement(QDomElement node, QString nodeName, ErrorOutput *output)
{
	SunDescr result;
	QDomElement sunNode = (nodeName.isEmpty() ? node : node.firstChildElement(nodeName));

	if (sunNode.isNull())
	{
		output->error(QObject::tr("No %1 node!").arg(nodeName));
		return result;
	}

	QDomElement positionNode = sunNode.firstChildElement("Position");
	if (positionNode.isNull())
	{
		output->error(QObject::tr("No Position node in Sun node!"));
		return result;
	}

	if (positionNode.hasAttribute("azimuth"))
	{
		bool ok;
		result.azimuth = positionNode.attributeNode("azimuth").value().toFloat(&ok);
		if (!ok)
			output->error(QObject::tr("Sun/Position/azimuth : this is not valid number"));
	}
	else
		output->error(QObject::tr("No azimuth attribute in Sun/Position node!"));

	if (positionNode.hasAttribute("altitude"))
	{
		bool ok;
		result.altitude = positionNode.attributeNode("altitude").value().toFloat(&ok);
		if (!ok)
			output->error(QObject::tr("Sun/Position/altitude : this is not valid number"));
	}
	else
		output->error(QObject::tr("No altitude attribute in Sun/Position node!"));
	
	return result;
}
