/* ----------------------------------------------------------------------------
This file is part of BlueSky

Copyright (C) 2012 Piotr Polesiuk

Permission is hereby granted, free of charge, to any person obtaining a copy 
of this software and associated documentation files (the "Software"), to deal 
in the Software without restriction, including without limitation the rights 
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
copies of the Software, and to permit persons to whom the Software is 
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in 
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
THE SOFTWARE.
---------------------------------------------------------------------------- */

#ifndef BS_ACTIONALTERNATIVE_H_INCLUDED
#define BS_ACTIONALTERNATIVE_H_INCLUDED

#include "BSAction.h"

namespace BlueSky
{

//! Action which performs orher action from list with given probability.
/*! When object of ActionAlternative contains list of actions
 * A1, A2, ... , An and list of probabilities P1, P2, ... , Pn, it performs 
 * action Ak with probability Pk or default action with probability 
 * 1 - P1 - P2 - ... - Pn. Sum P1 + P2 + ... + Pn shouldn't be 
 * greater then 1.
 *
 * \remarks This is a dynamic Object, so it can be created only by new operator.
 * If you want to use local objects of ActionAlternative, use ObjectManager
 * class.
 */
class ActionAlternative : public Action
{
private:

	Action *defaultAction;
	int maxActionCount;
	int actionCount;
	Action **actions;
	float *probabilities;

public:

	//! Creates new ActionAlternative object.
	/*! Created action list is empty and default action is an IdentityAction.
	 */
	ActionAlternative();

	//! Creates new ActionAlternative object with given default action.
	/*! Created action list is empty.
	 *
	 * \param[in] _defaultAction Default action.
	 */
	ActionAlternative(Action *_defaultAction);

	//! Creates new ActionAlternative object with given action list.
	/*! Default action is set to IdentityAction.
	 *
	 * \param[in] _actionCount Number of actions in action list.
	 * \param[in] _actions Action list with length equal to _actionCount.
	 * \param[in] _probabilities List of probabilities for actions stored
	 * _actions.
	 */
	ActionAlternative(int _actionCount, Action **_actions, float *_probabilities);

	//! Creates new ActionAlternative object with given action list and default action.
	/*!
	 * \param[in] _actionCount Number of actions in action list.
	 * \param[in] _actions Action list with length equal to _actionCount.
	 * \param[in] _probabilities List of probabilities for actions stored
	 * _actions.
	 * \param[in] _defaultAction Default action.
	 */
	ActionAlternative(int _actionCount, Action **_actions, float *_probabilities, Action *_defaultAction);

	//! Destructor of ActionAlternative.
	/*! Removes references to all contained actions.
	 */
	~ActionAlternative();

	//! Adds action to action list.
	/*! New action is added to the end of list.
	 *
	 * \param[in] action New action.
	 * \param[in] probability A probability associated with added action.
	 */
	void addAction(Action *action, float probability);

	//! Sets default action.
	/*! This method removes reference to old default action.
	 * \param[in] action New default action.
	 */
	void setDefaultAction(Action *action);

	//! Gets default action.
	/*!
	 * \return Action performed as default.
	 */
	inline Action *getDefaultAction()
	{
		return defaultAction;
	}

	//! Action on Photon which can change direction, color of wavelength.
	/*! Performed action is selected from action list with associated
	 * probabilites. When no action is selected, default action is performed.
	 *
	 * \param[in] ph Photon before interaction.
	 *
	 * \return A new Photon after interaction.
	 */
	Photon action(Photon ph);

	//! Action on Photon with additional parameter.
	/*! Performed action is selected from action list with associated
	 * probabilites. When no action is selected, default action is performed.
	 *
	 * \param[in] ph Photon before interaction.
	 * \param[in] param Additional parameter.
	 *
	 * \return A new Photon after interaction.
	 */
	Photon action(Photon ph, float param);
};

}

#endif
