/* ----------------------------------------------------------------------------
This file is part of BlueSky

Copyright (C) 2012 Piotr Polesiuk

Permission is hereby granted, free of charge, to any person obtaining a copy 
of this software and associated documentation files (the "Software"), to deal 
in the Software without restriction, including without limitation the rights 
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
copies of the Software, and to permit persons to whom the Software is 
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in 
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
THE SOFTWARE.
---------------------------------------------------------------------------- */

#include "BSCrystal.h"

#include <cstdlib>

namespace BlueSky
{

Crystal::Crystal()
{
	planeCount = 0;
	maxPlaneCount = 1;

	planeNormals   = (Vector3f*)malloc(sizeof(Vector3f));
	planeDistances = (float*)malloc(sizeof(float));
}

Crystal::Crystal(int _planeCount, const Vector3f *_planeNormals, const float *_planeDistances)
{
	planeCount = _planeCount;
	if (planeCount > 0)
		maxPlaneCount = _planeCount;
	else
		maxPlaneCount = 1;

	planeNormals   = (Vector3f*)malloc(sizeof(Vector3f) * maxPlaneCount);
	planeDistances = (float*)malloc(sizeof(float) * maxPlaneCount);

	for (int i = 0; i < _planeCount; i++)
	{
		planeNormals[i]   = _planeNormals[i].normalised();
		planeDistances[i] = _planeDistances[i];
	}
}

Crystal::~Crystal()
{
	free(planeNormals);
	free(planeDistances);
}
	
void Crystal::addPlane(Vector3f normal, float distance)
{
	if (planeCount >= maxPlaneCount)
	{
		maxPlaneCount *= 2;
		planeNormals = (Vector3f*)realloc(planeNormals, sizeof(Vector3f) * maxPlaneCount);
		planeDistances = (float*)realloc(planeDistances, sizeof(float) * maxPlaneCount);
	}

	planeNormals[planeCount] = normal.normalised();
	planeDistances[planeCount] = distance;
	planeCount++;
}

int Crystal::outsideCollision(const Vector3f &direction, Vector3f &position, Vector3f &normal) const
{
	int selected = -1;
	float distance = 0.0f;
	float distance2 = 1.0e+10f;

	for (int i = 0; i < planeCount; i++)
	{
		float dot = direction.dot(planeNormals[i]);
		if (dot == 0.0f)
			continue;

		float dist = (-position.dot(planeNormals[i]) + planeDistances[i]) / dot;
		if (dot < 0.0f)
		{
			if (dist > distance)
			{
				distance = dist;
				selected = i;
			}
			else
				continue;
		}
		else if (dist < distance2)
			distance2 = dist;
		else
			continue;

		if (distance2 < distance)
			return -1;
	}

	if (selected >= 0)
	{
		position += distance * direction;
		normal = planeNormals[selected];
	}
	return selected;
}

int Crystal::insideCollision(const Vector3f &direction, Vector3f &position, Vector3f &normal) const
{
	int selected = -1;
	float distance = 0.0f;
	float distance2 = 1.0e+10f;

	for (int i = 0; i < planeCount; i++)
	{
		float dot = direction.dot(planeNormals[i]);
		if (dot == 0.0f)
			continue;

		float dist = (-position.dot(planeNormals[i]) + planeDistances[i]) / dot;

		if (dot < 0.0)
		{
			if (dist > distance)
				distance = dist;
			else
				continue;
		}
		else if (dist < distance2)
		{
			distance2 = dist;
			selected = i;
		}
		else
			continue;

		if (distance2 < distance)
			return -1;
	}

	if (selected >= 0)
	{
		position += distance2 * direction;
		normal = planeNormals[selected];
	}
	return selected;
}

void Crystal::rotate(const Quaternionf &rot)
{
	for (int i = 0; i < planeCount; i++)
		planeNormals[i] = rot.rotate(planeNormals[i]);
}

Crystal *Crystal::createPrism(int sideCount, float height, float radius, float phase, float incline)
{
	float alpha = Math::PI2f / sideCount;
	phase += 0.5f * alpha;
	float r = radius * Math::cos(0.5f * alpha);

	Vector3f *normals = new Vector3f[sideCount+2];
	float *dists = new float[sideCount+2];

	normals[0] = Vector3f::UNIT_Y;
	dists[0] = height * 0.5f;
	normals[1] = -Vector3f::UNIT_Y;
	dists[1] = height * 0.5f;

	for (int i = 0; i < sideCount; i++)
	{
		Vector3f pos = r * Vector3f(
			Math::cos(alpha * i + phase),
			0.0f,
			Math::sin(alpha * i + phase));
		normals[i+2] = (pos - pos.x * incline * Vector3f::UNIT_Y).normalised();
		dists[i+2] = pos.dot(normals[i+2]);
	}

	Crystal *result = new Crystal(sideCount + 2, normals, dists);
	delete [] normals;
	delete [] dists;

	return result;
}

Crystal *Crystal::createDiamond(float d1, float d2, float d3, float d4)
{
	Vector3f normals[8];
	float dists[8];

	normals[0] = Vector3f( 1.0f,  1.0f,  1.0f).normalised();
	normals[1] = Vector3f( 1.0f,  1.0f, -1.0f).normalised();
	normals[2] = Vector3f( 1.0f, -1.0f,  1.0f).normalised();
	normals[3] = Vector3f( 1.0f, -1.0f, -1.0f).normalised();
	normals[4] = Vector3f(-1.0f,  1.0f,  1.0f).normalised();
	normals[5] = Vector3f(-1.0f,  1.0f, -1.0f).normalised();
	normals[6] = Vector3f(-1.0f, -1.0f,  1.0f).normalised();
	normals[7] = Vector3f(-1.0f, -1.0f, -1.0f).normalised();
	dists[0] = d1;
	dists[1] = d2;
	dists[2] = d3;
	dists[3] = d4;
	dists[4] = d4;
	dists[5] = d3;
	dists[6] = d2;
	dists[7] = d1;

	return new Crystal(8, normals, dists);
}

}
