/* ----------------------------------------------------------------------------
This file is part of BlueSky

Copyright (C) 2012 Piotr Polesiuk

Permission is hereby granted, free of charge, to any person obtaining a copy 
of this software and associated documentation files (the "Software"), to deal 
in the Software without restriction, including without limitation the rights 
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
copies of the Software, and to permit persons to whom the Software is 
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in 
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
THE SOFTWARE.
---------------------------------------------------------------------------- */

#ifndef BS_CRYSTAL_H_INCLUDED
#define BS_CRYSTAL_H_INCLUDED

#include "BSObject.h"
#include "BSVector3f.h"
#include "BSQuaternionf.h"

namespace BlueSky
{

//! Convex polyhedron representing crystals.
/*!
 * \remark This is a dynamic Object, so it can be created only by new operator.
 * If you want to create local objects of Crystal, use a ObjectManager class.
 */
class Crystal : public Object
{
public:

	//! Creates new empty crystal.
	/*! An empty crystal does not have any faces. No ray collide with it.
	 */
	Crystal();

	//! Creates a new crystal using given planes.
	/*! Plane is represented by a normal vector directed out of the crystal and
	 * the distance from (0,0,0) point, measured along the normal vector.
	 *
	 * \param[in] _planeCount The number of crystal planes.
	 * \param[in] _planeNormals A table of normal vectors that determines 
	 * crystal planes.
	 * \param[in] _planeDistances A table of distances from (0,0,0) point of 
	 * given planes.
	 */
	Crystal(int _planeCount, const Vector3f *_planeNormals, const float *_planeDistances);

	//! Destructor of Crystal class.
	~Crystal();

	//! Adds new plane to the crystal.
	/*! Plane is represented by a normal vector directed out of the crystal and
	 * the distance form (0,0,0) point, measured along the normal vector.
	 *
	 * \param[in] normal The normal vector of new plane.
	 * \param[in] distance The distance from (0,0,0) point of new plane.
	 */
	void addPlane(Vector3f normal, float distance);

	//! Check for collision with crystal surface from outside.
	/*!
	 * \param[in] direction The direction of tested ray (must be normalised).
	 * \param[in,out] position Start position of tested ray. On function return
	 * this paramteter is set to collision point if any is found or rest 
	 * unchanged otherwise
	 * \param[out] normal The normal vector of face which collide with given ray.
	 * When collision doesn't occur, this value is undefined.
	 *
	 * \return Id of colliding face or -1 when no collision found.
	 */
	int outsideCollision(const Vector3f &direction, Vector3f &position, Vector3f &normal) const;

	//! Check for collision with crystal surface from inside.
	/*!
	 * \param[in] direction The direction of tested ray (must be normalised).
	 * \param[in,out] position Start position of tested ray. On function return
	 * this paramteter is set to collision point if any is found or rest 
	 * unchanged otherwise
	 * \param[out] normal The normal vector of face which collide with given ray.
	 * When collision doesn't occur, this value is undefined.
	 *
	 * \return Id of colliding face or -1 when no collision found.
	 */
	int insideCollision(const Vector3f &direction, Vector3f &position, Vector3f &normal) const;

	//! Rotates crystal using given Quaternionf.
	/*!
	 * \remarks This function doesn't create copy of crystal, so when one 
	 * crystal is used by many objects, all of them will see performed rotation.
	 *
	 * \param[in] rot Quaternionf representing performed rotation.
	 */
	void rotate(const Quaternionf &rot);

	//! Creates new crystal of prism shape using given parameters.
	/*! First to faces of created crystal are bases and next sideCount faces
	 * are side faces. Position of vertices of created prism are given by
	 * formula:
	 * \f[
	 * x_{bj} = r\sin\left(\frac{2\pi{}j}{N} + f\right) \quad
	 * y_{bj} = \frac{1}{2}bh \quad
	 * z_{bj} = r\cos\left(\frac{2\pi{}j}{N} + f\right) + \frac{1}{2}bhi
	 * \f]
	 * where N is a sideCount, h is a height, r is a radius, f is a phase, i is a incline,
	 * b is a -1 for lower base and 1 for upper base and j = 0, ... , N-1.
	 *
	 * Result crystal has no references to him
	 *
	 * \param[in] sideCount Number of side faces.
	 * \param[in] height Height of created crystal (distacne between bases)
	 * \param[in] radius Radius of circumcirle of base
	 * \param[in] phase Phase of base.
	 * \param[in] incline Incline of created prism crystal.
	 *
	 * \return Prism crystal with given parameters.
	 */
	static Crystal *createPrism(int sideCount, float height, float radius, float phase, float incline);

	//! Creates octahedral crystal.
	/*! Parameters describe the distances from (0,0,0) point of faces in four 
	 * direction of normal vectors of faces (created crystal is centrally 
	 * symmetric).
	 *
	 * \param[in] d1 Distance from (0,0,0) point in first direction 
	 * ((1,1,1) or (-1,-1,-1)).
	 * \param[in] d2 Distance from (0,0,0) point in second direction 
	 * ((1,1,-1) or (-1,-1,1)).
	 * \param[in] d3 Distance from (0,0,0) point in third direction 
	 * ((1,-1,1) or (-1,1,-1)).
	 * \param[in] d4 Distance from (0,0,0) point in fourth direction 
	 * ((1,-1,-1) or (-1,1,1)).
	 *
	 * \return Created octahedral crystal.
	 */
	static Crystal *createDiamond(float d1, float d2, float d3, float d4);

	//! Gets number of faces in the crystal.
	/*!
	 * \return number of faces in the crystal
	 */
	inline int getFaceCount() const
	{
		return planeCount;
	}

private:

	int planeCount;
	int maxPlaneCount;

	Vector3f *planeNormals;
	float *planeDistances;
};

}

#endif
