/* ----------------------------------------------------------------------------
This file is part of BlueSky

Copyright (C) 2012 Piotr Polesiuk

Permission is hereby granted, free of charge, to any person obtaining a copy 
of this software and associated documentation files (the "Software"), to deal 
in the Software without restriction, including without limitation the rights 
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
copies of the Software, and to permit persons to whom the Software is 
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in 
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
THE SOFTWARE.
---------------------------------------------------------------------------- */

#ifndef BS_CRYSTALPATHSET_H_INCLUDED
#define BS_CRYSTALPATHSET_H_INCLUDED

#include "BSObject.h"

namespace BlueSky
{

//! Representation of light path in crystal
/*! A single path is coded by sequence of numbers which are ids of visited 
 * crystal faces. This class allows to create acyclic graph of selected paths.
 *
 * \remark This is a dynamic Object, so it can be created only by new operator.
 * If you want to use local object of CrystalPathSet, use a ObjectManager 
 * class.
*/
class CrystalPathSet : public Object
{
private:

	bool accepting;
	bool existsSuccessor;
	int facesCount;

	CrystalPathSet **successors;

public:

	//! Create an empty path set.
	/*!
	 * \param[in] _facesCount Number of faces in crystal.
	 * \param[in] _accepting A flag indicating if paths can end at created 
	 * object
	 */
	CrystalPathSet(int _facesCount, bool _accepting = false);

	//! Descructor of CrystalPathSet
	~CrystalPathSet();

	//! Check if this path is accepting
	/*!
	 * \return true when path is accepting, false otherwise.
	 */
	inline bool isAccepting() const
	{
		return accepting;
	}

	//! Check if exists path longer than 0 in current path set.
	/*!
	 * \return true when exists path longer than 0 in set, false otherwise.
	 */
	inline bool hasSuccessor() const
	{
		return existsSuccessor;
	} 

	//! Move forward in path tree.
	/*!
	 * \param[in] faceId An id of selected face.
	 *
	 * \return Less set of path which starts on faceId (without first element)
	 * or NULL when no path starts on faceId.
	 */
	inline CrystalPathSet *moveNext(int faceId) const
	{
		return successors[faceId];
	}

	//! Add path to set.
	/*!
	 * \param[in] path seqence of ids of faces on path. If n-th element of
	 * this sequence is set to -1 then any face can appear on path as n-th 
	 * face. 
	 * \param[in] length Length of added path. Last face of path is accepting.
	 */
	void addPath(int *path, int length);
};

}

#endif
