/* ----------------------------------------------------------------------------
This file is part of BlueSky

Copyright (C) 2012 Piotr Polesiuk

Permission is hereby granted, free of charge, to any person obtaining a copy 
of this software and associated documentation files (the "Software"), to deal 
in the Software without restriction, including without limitation the rights 
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
copies of the Software, and to permit persons to whom the Software is 
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in 
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
THE SOFTWARE.
---------------------------------------------------------------------------- */

#ifndef BS_CRYSTALSELECTIVEACTION_H_INCLUDED
#define BS_CRYSTALSELECTIVEACTION_H_INCLUDED

#include "BSAction.h"
#include "BSCrystal.h"
#include "BSCrystalPathSet.h"
#include "BSRefraction.h"
#include "BSRotation.h"

namespace BlueSky
{

//! Selected actions on given crystal.
/*! This Action represents refraction and reflexes on given crystal, but
 * you can specify which paths will keep photons alive and which will kill 
 * them.
 *
 * \remark This is a dynamic Object, so it can be created only by new operator.
 * If you want to create local object of CrystalSelectiveAction, use an
 * ObjectManager class.
 */
class CrystalSelectiveAction : public Action
{
private:

	Crystal *crystal;
	CrystalPathSet *pathSet;
	Rotation *rotation;
	Refraction *refraction;

public:

	//! Creates new CrystalSelectiveAction object.
	/*! Default rotation model is RandomUniformRotation, and default refraction
	 * model is RefractionPower(RefractionPower::ICE)
	 *
	 * \param[in] _crystal Crystal.
	 * \param[in] _pathSet A path set in given crystal.
	 */
	CrystalSelectiveAction(Crystal *_crystal, CrystalPathSet *_pathSet);

	//! Creates new CrystalSelectiveAction object.
	/*! Default refraction model is RefractionPower(RefractionPower::ICE)
	 *
	 * \param[in] _crystal Crystal.
	 * \param[in] _pathSet A path set in given crystal.
	 * \param[in] _rotation Rotation model.
	 */
	CrystalSelectiveAction(
		Crystal *_crystal, 
		CrystalPathSet *_pathSet,
		Rotation *_rotation);

	//! Creates new CrystalSelectiveAction object.
	/*! Default rotation model is RandomUniformRotation.
	 *
	 * \param[in] _crystal Crystal.
	 * \param[in] _pathSet A path set in given crystal.
	 * \param[in] _refraction Refraction model.
	 */
	CrystalSelectiveAction(
		Crystal *_crystal, 
		CrystalPathSet *_pathSet,
		Refraction *_refraction);

	//! Creates new CrystalSelectiveAction object.
	/*!
	 * \param[in] _crystal Crystal.
	 * \param[in] _pathSet A path set in given crystal.
	 * \param[in] _refraction Refraction model.
	 * \param[in] _rotation Rotation model.
	 */
	CrystalSelectiveAction(
		Crystal *_crystal, 
		CrystalPathSet *_pathSet,
		Refraction *_refraction,
		Rotation *_rotation);

	//! Destructor of CrystalSelectiveAction.
	~CrystalSelectiveAction();

	//! Change directon and brightness of given Photon.
	/*!
	 * \param[in] ph Photon before interaction.
	 * \return A new Photon after interaction.
	 */
	Photon action(Photon ph);
};

}

#endif
