/* ----------------------------------------------------------------------------
This file is part of BlueSky

Copyright (C) 2012 Piotr Polesiuk

Permission is hereby granted, free of charge, to any person obtaining a copy 
of this software and associated documentation files (the "Software"), to deal 
in the Software without restriction, including without limitation the rights 
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
copies of the Software, and to permit persons to whom the Software is 
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in 
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
THE SOFTWARE.
---------------------------------------------------------------------------- */

#include "BSLinearCamera.h"
#include "BSMath.h"

#include <string.h>

namespace BlueSky
{

LinearCamera::LinearCamera(const int _resolution, Vector3f _axis)
{
	resolution = _resolution;
	axis = _axis;
	data = new Vector3f[resolution];
	for (int i = 0; i < resolution; i++)
		data[i] = Vector3f::ZERO;
	photon_count = 0LL;
}

LinearCamera::~LinearCamera()
{
	delete [] data;
}

void LinearCamera::collect(const Photon& ph)
{
	collect(ph.dir.dot(axis), ph.color);
}
	
void LinearCamera::collect(const float x, const Vector3f& color)
{
	int data_x = (int)((x + 1.0f) * 0.5f * (float)resolution);

	if (data_x < 0 || data_x >= resolution)
		return;

	data[data_x] += color;
	photon_count++;
}

Vector3f LinearCamera::getDataByAngle(const float angle) const
{
	int data_x = (int)((Math::cos(angle) + 1.0f) * 0.5f * (float)resolution);

	if (data_x < 0 || data_x >= resolution)
		return Vector3f::ZERO;

	return data[data_x] * ((float)resolution / (float)photon_count);
}
	
Vector3f LinearCamera::getDataByDirection(const Vector3f& direction) const
{
	float data_xf = (float)resolution * (direction.dot(axis) + 1.0f) * 0.5f;
	int data_x = (int)data_xf;
	data_xf = Math::fractionalPart(data_xf);

	if (data_x < 0 || data_x >= resolution)
		return Vector3f::ZERO;
	else if (data_x == resolution - 1)
		return data[data_x] * ((float)resolution / (float)photon_count);
	else
		return (data[data_x] * (1.0f - data_xf) + data[data_x + 1] * data_xf) *
			((float)resolution / (float)photon_count);
}

Vector3f LinearCamera::getData(int i) const
{
	if (i < 0 || i >= resolution)
		return Vector3f::ZERO;

	return data[i] * ((float)resolution / (float)photon_count);
}
	
float LinearCamera::getAngle(int i) const
{
	if (i < 0 || i >= resolution)
		return 0.0f;
	return Math::acos(((float)i + 0.5f) * 2.0f / (float)resolution - 1.0f);
}

void LinearCamera::clean()
{
	photon_count = 0LL;
	memset(data, 0, resolution * sizeof(Vector3f));
}

}
