/* ----------------------------------------------------------------------------
This file is part of BlueSky

Copyright (C) 2012 Piotr Polesiuk

Permission is hereby granted, free of charge, to any person obtaining a copy 
of this software and associated documentation files (the "Software"), to deal 
in the Software without restriction, including without limitation the rights 
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
copies of the Software, and to permit persons to whom the Software is 
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in 
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
THE SOFTWARE.
---------------------------------------------------------------------------- */

#ifndef BS_LINEAR_CAMERA_H_INCLUDED
#define BS_LINEAR_CAMERA_H_INCLUDED

#include "BSCamera.h"

namespace BlueSky
{

//! Linear camera for phenomena with axial symmetry.
/*! This camera register only angle between own axis and photon direction.
 * Generates high quality images in short time, but only for phenomena with
 * axial symmetry such as rainbow or halo. 
 * For other phenomena use SkyBoxCamera.
 *
 * \remarks This is a dynamic object so can be created only by new constructor
 * If you want to ues local objects of LinearCamera use ObjectManager class.
 *
 * \sa Object, ObjectManager, SkyBoxCamera
 */
class LinearCamera : public Camera
{
private:

	int resolution;
	Vector3f axis;
	Vector3f *data;

	long long photon_count;

public:

	//! Constructor of LinearCamera.
	/*! Creates new object of LinearCamera with given resolution and axis.
	 *
	 * \remarks This object can be created only by new operator.
	 *
	 * \param[in] _resolution Resolution of camera.
	 * \param[in] _axis Axis of symmetry of camera.
	 */
	LinearCamera(const int _resolution, Vector3f _axis = Vector3f::UNIT_X);

	//! Destructor of LinearCamera.
	/*! 
	 * \remarks Use delete operator only when you are absolutely sure that 
	 * there is no references to removed object. Better solution is using remref()
	 * method or ObjectManager class.
	 */
	~LinearCamera();

	//! Collects photon.
	/*!
	 * \param[in] ph A photon which is collected.
	 */
	void collect(const Photon& ph);

	//! Collects photon with given parameters.
	/*! Collects photon with given color and cosine of the angle beetwen the
	 * direction of photon and axis of the camera.
	 *
	 * \param[in] x Cosine of the angle beetwen the direction of photon and
	 * axis of the camera.
	 * \param[in] color Color of the photon coded as Vector3f
	 */
	void collect(const float x, const Vector3f& color);

	//! Gets color registered on given angle.
	/*!
	 * \param[in] angle Angle beetwen requested direction and axis of the 
	 * camera.
	 *
	 * \return Color registered on given angle.
	 */
	Vector3f getDataByAngle(const float angle) const;

	//! Gets color registred in given direction
	/*!
	 * \remarks A direction means direction of photons which activated reqested
	 * cell on camera. This direction is opposite to direction in which camera
	 * looks.
	 *
	 * \param[in] direction Requested direction.
	 *
	 * \return Color registered on given direction.
	 */
	Vector3f getDataByDirection(const Vector3f& direction) const;

	//! Gets color registered on cell with given id.
	/*! If id is out of range [0; resolution), then returs Vector3f::ZERO.
	 *
	 * \remarks The angle corresponding to given cell can be checked using
	 * then method getAngle().
	 *
	 * \param[in] i Id of requested cell.
	 *
	 * \return Color registered on cell with given id.
	 */
	Vector3f getData(int i) const;

	//! Gets angle corresponding to cell with given id.
	/*! If id is out of range [0; resolution), then returs 0.
	 *
	 * \param[in] i Id of cell.
	 *
	 * \return Angle corresponding to cell with given id.
	 */
	float getAngle(int i) const;

	//! Gets resolution of the camera.
	/*!
	 * \return Resolution of the camera
	 */
	inline int getResolution() const
	{
		return resolution;
	}

	//! Gets axis of the camera
	/*!
	 * \return axis of the camera
	 */
	inline Vector3f getAxis() const
	{
		return axis;
	}

	//! Cleanup camera data.
	/*! Resets all camera data.
	 */
	void clean();
};

}

#endif
