/* ----------------------------------------------------------------------------
This file is part of BlueSky

Copyright (C) 2012 Piotr Polesiuk

Permission is hereby granted, free of charge, to any person obtaining a copy 
of this software and associated documentation files (the "Software"), to deal 
in the Software without restriction, including without limitation the rights 
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
copies of the Software, and to permit persons to whom the Software is 
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in 
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
THE SOFTWARE.
---------------------------------------------------------------------------- */

#ifndef BS_MATH_H_INCLUDED
#define BS_MATH_H_INCLUDED

#include <cmath>

namespace BlueSky
{

//! Mathematical functions and constants.
class Math
{
public:

	//! Absolute value of x.
	/*!
	 * \param[in] x Argument of abs function.
	 *
	 * \return Absolute value of x.
	 */
	inline static float abs(float x)
	{
		return x > 0.0f ? x : -x;
	}

	//! Arc cosine of x.
	/*!
	 * \param[in] x Argument of arc cosine function.
	 *
	 * \return Arc cosine of x.
	 */
	inline static float acos(float x)
	{
		return ::acos(x);
	}

	//! Cosine of x.
	/*!
	 * \param[in] x Argument of cosine function.
	 *
	 * \return Cosine of x.
	 */
	inline static float cos(float x)
	{
		return ::cos(x);
	}

	//! Cuts values larger than 1.
	/*! This function is given by formula:
	 * \f[ cut_1(x) = \left\{\begin{array}{l l}
	 *       1 & x \geq 1 \\
	 *       x & x \leq 1 \end{array}\right.
	 * \f]
	 *
	 * \param[in] x Argument of cut1 function.
	 *
	 * \return x when x is less then 1 and 1 otherwise
	 */
	inline static float cut1(float x)
	{
		return (x > 1.0f) ? 1.0f : x;
	}

	//! Fractional part of x
	/*!
	 * \param[in] x Argument of fractional part function.
	 *
	 * \return Fractional part of x.
	 */
	inline static float fractionalPart(float x)
	{
		return x - ::floor(x);
	}

	//! Signum of x.
	/*!
	 * \param[in] x Argument of fsgn function
	 *
	 * \return -1.0f for negative x, 1.0f for positive x, 0.0f when x == 0.0f
	 */
	inline static float fsgn(float x)
	{
		return (x == 0.0f ? 0.0f : (x > 0.0f ? 1.0f : -1.0f));
	}

	//! Natural logarithm of x.
	/*!
	 * \param[in] x Argument of natural logarithm function.
	 *
	 * \return natural logarithm of x.
	 */
	inline static float log(float x)
	{
		return ::log(x);
	}

	//! Raise to power.
	/*!
	 * \param[in] base Base of the power.
	 * \param[in] exponent Exponenet of the power.
	 *
	 * \return base raised to the power exponent.
	 */
	inline static float pow(float base, float exponent)
	{
		return ::pow(base, exponent);
	}

	//! Sine of x.
	/*!
	 * \param[in] x Argument of sine function.
	 *
	 * \return sine of x.
	 */
	inline static float sin(float x)
	{
		return ::sin(x);
	}

	//! Square of x.
	/*!
	 * \param[in] x Argument of square function.
	 *
	 * \return Square of x.
	 */
	inline static float sqr(float x)
	{
		return x*x;
	}

	//! Square root of x.
	/*!
	 * \param[in] x Argument of square root function.
	 *
	 * \return Square root of x.
	 */
	inline static float sqrt(float x)
	{
		return ::sqrt(x);
	}

	//! pi constatnt.
	static const float PIf;

	//! 2*pi constant.
	static const float PI2f;
};

}

#endif
