/* ----------------------------------------------------------------------------
This file is part of BlueSky

Copyright (C) 2012 Piotr Polesiuk

Permission is hereby granted, free of charge, to any person obtaining a copy 
of this software and associated documentation files (the "Software"), to deal 
in the Software without restriction, including without limitation the rights 
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
copies of the Software, and to permit persons to whom the Software is 
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in 
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
THE SOFTWARE.
---------------------------------------------------------------------------- */

#ifndef BS_OBJECT_H_INCLUDED
#define BS_OBJECT_H_INCLUDED

namespace BlueSky
{

//! Dynamic object with garbage collection by reference counting.
/*! Dynamic objects has field which contains number of objects that refer
 * to it. This number can be increase or decrease by addRef() and remRef() 
 * methods, and when this number is decreased to zero, object is removed.
 *
 * This is a common pattern of using Object class:
 * \code
 * Object *obj = ... // create dynamic object
 * obj->addRef()
 * // do something
 * ...
 * obj->remRef(); // obj will be removed unless it is used by another object.
 * \endcode
 * But we suggest to use ObjectManager class insted of.
 *
 * \remark Dynamic object should be created only by new operator, and cyclic
 * references between them are forbidden.
 * When dynamic object is created, reference count is set to zero.
 *
 * \sa ObjectManager
 */
class Object
{
private:

	int referenceCount;

public:

	//! Creates new object with no references to him.
	inline Object() :
		referenceCount(0)
	{
	}

	//! Destructor of Object.
	/*! \remarks Use delete operator only when you are absolutely sure that
	 * there is no references to removed object. Better solution is using
	 * remref() method and ObjectManager
	 */
	inline virtual ~Object()
	{
	}

	//! Add new reference to object. 
	/*! After using the object, you can remove reference by call the remRef
	 * method.
	 */
	inline void addRef()
	{
		referenceCount++;
	}

	//! Remove reference to object.
	/*! After removing the last reference, object is automatically removed 
	 * from memory.
	 * \return The bool value that indicating whether object has been deleted.
	 */
	bool remRef();
};

}

#endif
