/* ----------------------------------------------------------------------------
This file is part of BlueSky

Copyright (C) 2012 Piotr Polesiuk

Permission is hereby granted, free of charge, to any person obtaining a copy 
of this software and associated documentation files (the "Software"), to deal 
in the Software without restriction, including without limitation the rights 
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
copies of the Software, and to permit persons to whom the Software is 
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in 
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
THE SOFTWARE.
---------------------------------------------------------------------------- */

#ifndef BS_OBJECTMANAGER_H_INCLUDED
#define BS_OBJECTMANAGER_H_INCLUDED

#include "BSObject.h"

namespace BlueSky
{

//! Dynamic local Object manager.
/*! You can add dynamic objects to ObjectManager using method manage(). Managed
 * objects has additional reference from ObjectManager, so they can not be 
 * removed until manager is alive. If you are using ObjectManager, you don't
 * need to create references and remove them by addRef() and remRef().
 *
 * \code
 * void foo(int bar)
 * {
 *     ObjectManager mgr; // create local object manager.
 *     Object *obj = ...  // create some object
 *     mgr.manage(obj);   // create reference from mgr to obj.
 *
 *     // do something
 *     ...
 *
 *     Object *obj2 = ... // create another object
 *     mgr.manage(obj2);
 *
 *     // do something again
 *     ...
 *
 * } // Here obj and obj2 are removed unless another object refers to them.
 * \endcode
 *
 * \remark ObjectManager is not a dynamic object, so you can create local 
 * objects of ObjectManager.
 *
 * \sa Object
 */
class ObjectManager
{
private:

	int objectCount;
	int arraySize;
	Object **objects;

public:

	//! Constructor of ObjectManager.
	/*! Creates new object manager with array for dynamic object of given size.
	 * This array expands automatically when needed.
	 *
	 * \param[in] _arraySize Size of array for dynamic objects.
	 */
	explicit ObjectManager(int _arraySize = 16);

	//! Destructor of ObjectManager.
	/*! When object manager is removed, it removes references to managed 
	 * objects, so unused objects will be removed. Use manage method to mark
	 * objects as managed.
	 */
	~ObjectManager();

	//! Manage dynamic object.
	/*! This method adds creates reference to given dynamic object, and
	 * adds it to array of dynamic objects. Use it to manage local objects.
	 *
	 * \param[in] obj Dynamic object which will be marked as managed.
	 */
	void manage(Object *obj);
};

}

#endif
