/* ----------------------------------------------------------------------------
This file is part of BlueSky

Copyright (C) 2012 Piotr Polesiuk

Permission is hereby granted, free of charge, to any person obtaining a copy 
of this software and associated documentation files (the "Software"), to deal 
in the Software without restriction, including without limitation the rights 
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
copies of the Software, and to permit persons to whom the Software is 
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in 
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
THE SOFTWARE.
---------------------------------------------------------------------------- */

#ifndef BS_QUATERNIONF_H_INCLUDED
#define BS_QUATERNIONF_H_INCLUDED

#include "BSVector3f.h"
#include "BSMath.h"

namespace BlueSky
{

//! Standart quaternion of four floats.
/*! Quaternions are used to represent rotations in 3D space.
 */
class Quaternionf
{
public:
	//! Real coordinate of the quaternion.
	float w;
	//! First imaginary coordinate of the quaternion.
	float x;
	//! Second imaginary coordinate of the quaternion.
	float y;
	//! Third imaginary coordinate of the quaternion.
	float z;

	//! Creates quaternion.
	inline Quaternionf()
	{
	}

	//! Creates quaternion from real number.
	/*!
	 * \param[in] _w Real coordinate of created quaternion.
	 */
	inline Quaternionf(float _w) :
		w(_w), x(0.0f), y(0.0f), z(0.0f)
	{
	}

	//! Creates quaterion from vector.
	/*! Real coordinate has value 0; vector coordinates are mapped to 
	 * imaginary coorfinates.
	 *
	 * \param[in] v Vector with imaginary coordinates of created quaternion.
	 */
	inline Quaternionf(Vector3f v) :
		w(0.0f), x(v.x), y(v.y), z(v.z)
	{
	}

	//! Creates quaternion with given coordinates
	/*!
	 * \param[in] _w Real coordinate of created quaternion.
	 * \param[in] _x First imaginary coordinate of created quaternion.
	 * \param[in] _y Second imaginary coordinate of created quaternion.
	 * \param[in] _z Third imaginary coordinate of created quaternion.
	 */
	inline Quaternionf(float _w, float _x, float _y, float _z) :
		w(_w), x(_x), y(_y), z(_z)
	{
	}

	//! Copy quaternions.
	inline Quaternionf& operator = (const Quaternionf& q)
	{
		w = q.w;
		x = q.x;
		y = q.y;
		z = q.z;
		return *this;
	}

	//! Check for equality of quaternions.
	inline bool operator == (const Quaternionf& q) const
	{
		return (w == q.w && x == q.x && y == q.y && z == q.z);
	}

	//! Check for inequality of quaternions.
	inline bool operator != (const Quaternionf& q) const
	{
		return (w != q.w || x != q.x || y != q.y || z != q.z);
	}

	//! Compute sum of two quaternions.
	inline Quaternionf operator + (const Quaternionf& q) const
	{
		return Quaternionf(
			w + q.w,
			x + q.x,
			y + q.y,
			z + q.z);
	}

	//! Compute difference of two quaternions.
	inline Quaternionf operator - (const Quaternionf& q) const
	{
		return Quaternionf(
			w - q.w,
			x - q.x,
			y - q.y,
			z - q.z);
	}

	//! Compute product of two quaternions.
	Quaternionf operator * (const Quaternionf& q) const;
	//! Compute quotient of two quaternions.
	Quaternionf operator / (const Quaternionf& q) const;

	//! Compute product of quaternion and real number.
	inline Quaternionf operator * (const float c) const
	{
		return Quaternionf(
			w * c,
			x * c,
			y * c,
			z * c);
	}

	//! Compute quotient of quaternion and real number.
	inline Quaternionf operator / (const float c) const
	{
		float cinv = 1.0f / c;
		return Quaternionf(
			w * cinv,
			x * cinv,
			y * cinv,
			z * cinv);
	}

	//! Unary plus.
	inline const Quaternionf& operator + () const
	{
		return *this;
	}

	//! Compute negation of the quaternion.
	inline Quaternionf operator - () const
	{
		return Quaternionf(-w, -x, -y, -z);
	}

	//! Compute conjugation of the quaternion.
	inline Quaternionf operator ~ () const
	{
		return Quaternionf(w, -x, -y, -z);
	}

	//! Compute product of real number and quaternion.
	inline friend Quaternionf operator * (const float c, const Quaternionf& q)
	{
		return Quaternionf(
			c * q.w,
			c * q.x,
			c * q.y,
			c * q.z);
	}

	//! Compute quotient of real number and quaternion.
	inline friend Quaternionf operator / (const float c, const Quaternionf& q)
	{
		float cpls = c / (q.w*q.w + q.x*q.x + q.y*q.y + q.z*q.z);
		return Quaternionf(
			 q.w * cpls,
			-q.x * cpls,
			-q.y * cpls,
			-q.z * cpls);
	}

	//! Add quaternion to another.
	inline Quaternionf& operator += (const Quaternionf& q)
	{
		w += q.w;
		x += q.x;
		y += q.y;
		z += q.z;

		return *this;
	}

	//! Subtract quaternion from another.
	inline Quaternionf& operator -= (const Quaternionf& q)
	{
		w -= q.w;
		x -= q.x;
		y -= q.y;
		z -= q.z;
	
		return *this;
	}

	//! Multiply quaternion by another.
	inline Quaternionf& operator *= (const Quaternionf& q)
	{
		return (*this = *this * q);
	}

	//! Divide quaternion by another.
	inline Quaternionf& operator /= (const Quaternionf& q)
	{
		return (*this = *this / q);
	}

	//! Multiply quaternion by real number.
	inline Quaternionf& operator *= (const float c)
	{
		w *= c;
		x *= c;
		y *= c;
		z *= c;

		return *this;
	}

	//! Divide quaternion by real number.
	inline Quaternionf& operator /= (const float c)
	{
		float cinv = 1.0f / c;
		w *= cinv;
		x *= cinv;
		y *= cinv;
		z *= cinv;

		return *this;
	}

	//! Conjugate the quaternion.
	inline Quaternionf& conjugate()
	{
		x = -x;
		y = -y;
		z = -z;
		return *this;
	}

	//! Gets conjugated copy of the quaternion.
	inline Quaternionf conjugated() const
	{
		return Quaternionf(w, -x, -y, -z);
	}

	//! Inverse the quaternion.
	inline Quaternionf& inverse()
	{
		float lsi = 1.0f / (w*w + x*x + y*y + z*z);
		w *= lsi;
		x *= -lsi;
		y *= -lsi;
		z *= -lsi;
		return *this;
	}

	//! Get inversed copy of quaternion.
	inline Quaternionf inversed() const
	{
		float lsi = 1.0f / (w*w + x*x + y*y + z*z);
		return Quaternionf(
			 w * lsi,
			-x * lsi,
			-y * lsi,
			-z * lsi);
	}

	//! Get length of the quaternion.
	inline float length() const
	{
		return Math::sqrt(w*w + x*x + y*y + z*z);
	}

	//! Gets square of quaternion lenght.
	inline float lengthSQ() const
	{
		return w*w + x*x + y*y + z*z;
	}

	//! Normalise the quaternion.
	/*!
	 * \return length of the quaternion before normalisation.
	 */
	inline float normalise()
	{
		float l = Math::sqrt(w*w + x*x + y*y + z*z);
		float linv = 1.0f / l;

		w *= linv;
		x *= linv;
		y *= linv;
		z *= linv;

		return l;
	}

	//! Gets normalised copy of the quaternion.
	inline Quaternionf normalised() const
	{
		Quaternionf res = *this;
		res.normalise();
		return res;
	}

	//! Convert quaternion to vector.
	/* Maps imaginary coordinates to vector coordinate.
	 * Real coordinate of quaternion is ignored.
	 */
	inline Vector3f vec() const
	{
		return Vector3f(x, y, z);
	}

	//! Create quaternion representing rotation around given axis of specified angle.
	void fromAngleAxis(const float angle, const Vector3f& axis);

	//! Rotate vector.
	Vector3f rotate(const Vector3f& v) const;

	//! Create quaternion representing rotation around x axis of specified angle.
	inline static Quaternionf xRotation(const float angle)
	{
		Quaternionf result;
		result.fromAngleAxis(angle, Vector3f::UNIT_X);
		return result;
	}

	//! Create quaternion representing rotation around y axis of specified angle.
	inline static Quaternionf yRotation(const float angle)
	{
		Quaternionf result;
		result.fromAngleAxis(angle, Vector3f::UNIT_Y);
		return result;
	}

	//! Create quaternion representing rotation around z axis of specified angle.
	inline static Quaternionf zRotation(const float angle)
	{
		Quaternionf result;
		result.fromAngleAxis(angle, Vector3f::UNIT_Z);
		return result;
	}

	//! Zero quaterion.
	static const Quaternionf ZERO;
	//! Real unit quaterion.
	static const Quaternionf UNIT;
	//! First imaginary unit quaternion.
	static const Quaternionf UNIT_X;
	//! Second imaginary unit quaternion.
	static const Quaternionf UNIT_Y;
	//! Third imaginary unit quaternion.
	static const Quaternionf UNIT_Z;

	//! Rotation around X axis by 90 degrees.
	static const Quaternionf ROT_X90;
	//! Rotation around Y axis by 90 degrees.
	static const Quaternionf ROT_Y90;
	//! Rotation around Z axis by 90 degrees.
	static const Quaternionf ROT_Z90;
};

}

#endif
