/* ----------------------------------------------------------------------------
This file is part of BlueSky

Copyright (C) 2012 Piotr Polesiuk

Permission is hereby granted, free of charge, to any person obtaining a copy 
of this software and associated documentation files (the "Software"), to deal 
in the Software without restriction, including without limitation the rights 
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
copies of the Software, and to permit persons to whom the Software is 
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in 
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
THE SOFTWARE.
---------------------------------------------------------------------------- */

#ifndef BS_RAINBOW_H_INCLUDED
#define BS_RAINBOW_H_INCLUDED

#include "BSPhoton.h"
#include "BSAction.h"
#include "BSRefraction.h"
#include "BSObject.h"

namespace BlueSky
{

//! Action of photons to simulate simple rainbow.
/*! This action simulates interacition with perfectly spherical drop of water
 * or other liquids.
 *
 * \remark This is a dynamic Object, so can be created only by new operator.
 * If you want to create local objects of Rainbow, use ObjectManager class.
 *
 * \sa RainbowSelective, Object, ObjectManager
 */
class Rainbow : public Action
{
private:
	
	Refraction *refraction;
	
public:

	//! Creates new Rainbow Action.
	/*! Default refraction model is RefractionPower with parameters for water
	 * in visible spectrum.
	 *
	 * \remarks This object can be created only by new operator.
	 */
	Rainbow();

	//! Creates new Rainbow Action with given refractive index.
	/*!
	 * \param[in] _refractiveIndex Refractive index.
	 */
	explicit Rainbow(float _refractiveIndex);

	//! Creates new Rainbow Action with given refraction model.
	/*!
	 * \param[in] _refraction Refraction model used by rainbow. This constructor
	 * creates new reference to _refraction.
	 */
	explicit Rainbow(Refraction *_refraction);

	//! Destructor of Rainbow.
	/*!
	 * \remarks Use delete opeator only when you are absolutely sure that
	 * there is no references to removed object. Better solution is using
	 * remref() method or ObjectManager class.
	 */
	~Rainbow();

	//! Change direction of given photon.
	/*!
	 * \param[in] ph Photon before interaction.
	 *
	 * \return New Photon after interaction.
	 */
	Photon action(Photon ph);

	//! Change direction of given photon using given square excentricity.
	/*! Excentricity is a sine of angle of incidence.
	 *
	 * \param[in] ph Photon begore interaction.
	 * \param[in] param Excentricity (sine of angle of incidence).
	 *
	 * \return New Photon after interaction.
	 */
	Photon action(Photon ph, float param);
};

}

#endif
