/* ----------------------------------------------------------------------------
This file is part of BlueSky

Copyright (C) 2012 Piotr Polesiuk

Permission is hereby granted, free of charge, to any person obtaining a copy 
of this software and associated documentation files (the "Software"), to deal 
in the Software without restriction, including without limitation the rights 
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
copies of the Software, and to permit persons to whom the Software is 
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in 
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
THE SOFTWARE.
---------------------------------------------------------------------------- */

#include "BSRainbowSelective.h"

#include "BSRandom.h"
#include "BSRefractionPower.h"
#include "BSRefractionConst.h"

namespace BlueSky
{

RainbowSelective::RainbowSelective(unsigned long long _path)
{
	path = _path;
	refraction = new RefractionPower(RefractionPower::WATER);
	refraction->addRef();
}

RainbowSelective::RainbowSelective(unsigned long long _path, float _refractiveIndex)
{
	path = _path;
	refraction = new RefractionConst(_refractiveIndex);
	refraction->addRef();
}

RainbowSelective::RainbowSelective(unsigned long long _path, Refraction *_refraction)
{
	path = _path;
	refraction = _refraction;
	refraction->addRef();
}

RainbowSelective::~RainbowSelective()
{
	refraction->remRef();
}

Photon RainbowSelective::action(Photon ph)
{
	return action(ph, Random::randomFloat());
}

Photon RainbowSelective::action(Photon ph, float param)
{
	float refractiveIndex = refraction->getRefractiveIndex(ph.wavelength);
	float power = 1.0f;
	float st1 = Math::sqrt(param);
	float ct1 = Math::sqrt(1.0f - param);
	float st2 = st1 / refractiveIndex;
	float ct2, pb;
	if (st2 <= 1.0f)
	{
		ct2 = Math::sqrt(1.0f - Math::sqr(st2));
		pb  = Math::cut1(Math::sqr(
			(refractiveIndex*ct1 - ct2) / (refractiveIndex*ct1 + ct2)));
	}
	else
	{
		ct2 = 0.0;
		pb = 1.0f;
	}
	// 0 - never; 1 - random; 2 - always
	int reflexState =
		((path & 1) ? ((path & 0xFFFFFFFFFFFFFFFELLU) ? 1 : 2) : 0);
	if (reflexState > 0 && (reflexState == 2 || Random::randomFloat() < pb)) // Reflex
	{
		if (reflexState == 2)
			power *= pb;
		Vector3f v1, v2;
		ph.dir.orthoBase(v1, v2);
		float t = Random::randomFloat(Math::PI2f);
		Vector3f norm = 
			(st1 * Math::sin(t)) * v1 + 
			(st1 * Math::cos(t)) * v2 +
			-ct1 * ph.dir;
		Photon result = ph;
		result.dir += (2.0f * ct1) * norm;
		result.color *= power;
		return result;
	}
	else // Refract
	{
		if (reflexState == 0)
			power *= (1.0f - pb);
		unsigned long long path2 = path >> 1;

		float sg = 2.0f * st2 * ct2; // sin (2t1)
		float cg = 1.0f - 2.0f * Math::sqr(ct2);
		float vx = ct1*ct2 + st1*st2;
		float vy = ct1*st2 - st1*ct2;
		reflexState = 
			((path2 & 1) ? ((path2 & 0xFFFFFFFFFFFFFFFELLU) ? 1 : 0) : 2);
		while (reflexState > 0 && (reflexState == 2 || Random::randomFloat() < pb))
		{
			if (reflexState == 2)
				power *= pb;
			path2 >>= 1;
			float tvx = vx*cg + vy*sg;
			vy = vy*cg - vx*sg;
			vx = tvx;
			reflexState = 
				((path2 & 1) ? ((path2 & 0xFFFFFFFFFFFFFFFELLU) ? 1 : 0) : 2);
		}
		if (reflexState == 0)
			power *= (1.0f - pb);
		float tvx = vx*ct2 - vy*st2;
		vy = vy*ct2 + vx*st2;
		vx = tvx;
		tvx = vx*ct1 + vy*st1;
		vy  = vy*ct1 - vx*st1;
		vx  = tvx;
		Vector3f v1, v2;
		ph.dir.orthoBase(v1, v2);
		float t = Random::randomFloat(Math::PI2f);
		Photon result = ph;
		result.dir =
			(vy * Math::sin(t)) * v1 +
			(vy * Math::cos(t)) * v2 +
			vx * ph.dir;
		result.color *= power;
		return result;
	}
}

}
