/* ----------------------------------------------------------------------------
This file is part of BlueSky

Copyright (C) 2012 Piotr Polesiuk

Permission is hereby granted, free of charge, to any person obtaining a copy 
of this software and associated documentation files (the "Software"), to deal 
in the Software without restriction, including without limitation the rights 
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
copies of the Software, and to permit persons to whom the Software is 
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in 
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
THE SOFTWARE.
---------------------------------------------------------------------------- */

#ifndef BS_RAINBOWSELECTIVE_H_INCLUDED
#define BS_RAINBOWSELECTIVE_H_INCLUDED

#include "BSPhoton.h"
#include "BSAction.h"
#include "BSRefraction.h"

namespace BlueSky
{

//! Rainbow simulation, where you can select specific paths of light
/*! This Actions simulates interaction with perfectly spherical drops of water
 * or other liquids. You can also select specific family of light paths
 * that will be simulated, and remaining paths will be ignored.
 *
 * Paths are coded by binary number: When bit 0 is set, ouside reflections
 * are allowed. Bit (n+1) selects paths through drop with n internal
 * reflections. You can mask many paths. For example binary 1100 = 0xC selects
 * only rainbows of rank 1 and 2 (which are often observed).
 *
 * \remark This is a dynamic Object, so can be created only by new operator.
 * If you want to create local Objects of RainbowSelective, use ObjectManager
 * class.
 *
 * \sa Rainbow, Object, Object Manager
 */
class RainbowSelective : public Action
{
private:

	unsigned long long path;
	Refraction *refraction;

public:

	//! Create new RainbowSelective Action which allows only specified paths.
	/*! Default refraction model is RefractionPower with parameters for water
	 * in visible spectrum.
	 * \param[in] _path Path mask.
	 */
	explicit RainbowSelective(unsigned long long _path);
	
	//! Create new RainbowSelective with given refractive index and which allows only specified paths.
	/*!
	 * \param[in] _path Path mask.
	 * \param[in] _refractiveIndex Refractive index.
	 */
	RainbowSelective(unsigned long long _path, float _refractiveIndex);

	//! Create new RainbowSelective with given refraction model and which allows only specified paths.
	/*!
	 * \param[in] _path Path mask.
	 * \param[in] _refraction Refraction model.
	 */
	RainbowSelective(unsigned long long _path, Refraction *_refraction);

	//! Destructor of RainbowSelective.
	~RainbowSelective();

	//! Change direction and power of given photon.
	/*!
	 * \param[in] ph Photon before interaction.
	 *
	 * \return New Photon after interaction.
	 */
	Photon action(Photon ph);

	//! Change direction and power of given photon using given square excentricity.
	/*! Excentricity is a sine of angle of incidence.
	 *
	 * \param[in] ph Photon begore interaction.
	 * \param[in] param Excentricity (sine of angle of incidence).
	 *
	 * \return New Photon after interaction.
	 */
	Photon action(Photon ph, float param);
};

}

#endif
