/* ----------------------------------------------------------------------------
This file is part of BlueSky

Copyright (C) 2012 Piotr Polesiuk

Permission is hereby granted, free of charge, to any person obtaining a copy 
of this software and associated documentation files (the "Software"), to deal 
in the Software without restriction, including without limitation the rights 
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
copies of the Software, and to permit persons to whom the Software is 
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in 
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
THE SOFTWARE.
---------------------------------------------------------------------------- */

#ifndef BS_RANDOM_H_INCLUDED
#define BS_RANDOM_H_INCLUDED

#include <cstdlib>

#include "BSQuaternionf.h"

namespace BlueSky
{

//! Static class with random functions.
class Random
{
public:

	//! Gets random number from range [0, 1)
	/*!
	 * \return Random float from range [0, 1)
	 */
	inline static float randomFloat()
	{
		return (float)(rand()) / (float)RAND_MAX;
	}

	//! Gets random number from range [0, limit)
	/*!
	 * \param[in] limit Upper limit for random number.
	 *
	 * \return Random float from range [0, limit)
	 */
	inline static float randomFloat(float limit)
	{
		return limit * randomFloat();
	}

	//! Gets random number from range [min, max)
	/*!
	 * \param[in] min Lower limit of random number
	 * \param[in] max Upper limit of random number
	 *
	 * \return Random float from range [min, max)
	 */
	inline static float randomFloat(float min, float max)
	{
		return min + (max - min) * randomFloat();
	}

	//! Gets random point inside circle of radius 1.
	/*!
	 * \param[out] x X coordinate of selected point.
	 * \param[out] y Y coordinate of selected point.
	 */
	static void randomCircle(float& x, float& y);

	//! Gets random point on sphere of radius equal to 1 (normalsed Vector3f).
	/*!
	 * \return Random point on sphere.
	 */
	static Vector3f randomSphere();

	//! Gets random normalised Quaternionf.
	/*!
	 * \return Random normalised Quaternionf.
	 */
	static Quaternionf randomQuaternionf();

	//! Gets random float value with normal distribution.
	/*! This function uses algorithm based on Box-Muller transformation.
	 *
	 * \param[in] mean Mean value for distribution.
	 * \param[in] variance Variance for distribution.
	 *
	 * \return Random value with normal distribution with given parameters.
	 */
	static float randomFloatNormal(float mean, float variance);
};

}

#endif
