/* ----------------------------------------------------------------------------
This file is part of BlueSky

Copyright (C) 2012 Piotr Polesiuk

Permission is hereby granted, free of charge, to any person obtaining a copy 
of this software and associated documentation files (the "Software"), to deal 
in the Software without restriction, including without limitation the rights 
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
copies of the Software, and to permit persons to whom the Software is 
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in 
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
THE SOFTWARE.
---------------------------------------------------------------------------- */

#include "BSRefraction.h"

#include "BSRandom.h"

namespace BlueSky
{

bool Refraction::refract(Vector3f& direction, float refractiveIndex, const Vector3f& normal)
{
	float dot = normal.dot(direction);
	Vector3f normalDir = normal * dot;
	Vector3f tangentDir = (direction - normalDir) / refractiveIndex;
	
	if (tangentDir.lengthSQ() > 1.0f)
	{
		direction -= 2.0f * normalDir;
		return false;
	}
	
	Vector3f newNormalDir = 	
		Math::fsgn(dot) * Math::sqrt(1.0f - tangentDir.lengthSQ()) * normal;
	float ct1 = normalDir.length();
	float ct2 = newNormalDir.length();
	float pb = Math::sqr((refractiveIndex*ct1 - ct2) / (refractiveIndex*ct1 + ct2));

	if (Random::randomFloat() < pb)
	{
		direction -= 2.0f * normalDir;
		return false;
	}	

	direction = tangentDir + newNormalDir;
	return true;
}
	
bool Refraction::refract(
	Photon &photon, 
	float refractiveIndex, 
	const Vector3f& normal,
	bool tryRefract,
	bool tryReflect)
{
	if (!tryRefract && !tryReflect)
	{
		photon.color = Vector3f::ZERO;
		return false;
	}

	float dot = normal.dot(photon.dir);
	Vector3f normalDir = normal * dot;
	Vector3f tangentDir = (photon.dir - normalDir) / refractiveIndex;
	
	if (tangentDir.lengthSQ() > 1.0f)
	{
		if (!tryReflect)
			photon.color = Vector3f::ZERO;
		photon.dir -= 2.0f * normalDir;
		return false;
	}
	
	Vector3f newNormalDir = 	
		Math::fsgn(dot) * Math::sqrt(1.0f - tangentDir.lengthSQ()) * normal;
	float ct1 = normalDir.length();
	float ct2 = newNormalDir.length();
	float pb = Math::sqr((refractiveIndex*ct1 - ct2) / (refractiveIndex*ct1 + ct2));

	if (tryReflect)
	{
		if (tryRefract)
		{
			if (Random::randomFloat() < pb)
			{
				photon.dir -= 2.0f * normalDir;
				return false;
			}	

			photon.dir = tangentDir + newNormalDir;
			return true;
		}
		else
		{
			photon.color *= pb;
			photon.dir -= 2.0f * normalDir;
			return false;
		}
	}
	else
	{
		photon.color *= (1.0f - pb);
		photon.dir = tangentDir + newNormalDir;
		return true;
	}
}

}
