/* ----------------------------------------------------------------------------
This file is part of BlueSky

Copyright (C) 2012 Piotr Polesiuk

Permission is hereby granted, free of charge, to any person obtaining a copy 
of this software and associated documentation files (the "Software"), to deal 
in the Software without restriction, including without limitation the rights 
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
copies of the Software, and to permit persons to whom the Software is 
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in 
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
THE SOFTWARE.
---------------------------------------------------------------------------- */

#ifndef BS_REFRACTION_H_INCLUDED
#define BS_REFRACTION_H_INCLUDED

#include "BSObject.h"
#include "BSVector3f.h"
#include "BSPhoton.h"

namespace BlueSky
{

//! Abstract class representing refraction model.
/*! Objects of class Refraction are used to compute refractive index from
 * wavelength.
 */
class Refraction : public Object
{
public:

	//! Destructor of Refraction class.
	inline virtual ~Refraction()
	{
	}

	//! Gets refractive index from wavelength
	/*!
	 * \param[in] wavelength A wavelength.
	 *
	 * \return Refractive index.
	 */
	virtual float getRefractiveIndex(float wavelength) = 0;

	//! Refract or reflect given ray using given refractive index and normal vector
	/*! When refraction is not possibile ray is reflected (total internal 
	 * reflection), otherwise function randoms which situation will occur 
	 * (reflection has greater probability for angles closer to 90 degrees).
	 * 
	 * \param[in,out] direction A direaction of ray.
	 * \param[in] refractiveIndex A refractive index.
	 * \param[in] normal A normal vector of refracting surface.
	 *
	 * \return True when ray is refracted, false when ray is reflected.
	 */
	static bool refract(Vector3f& direction, float refractiveIndex, const Vector3f& normal);

	//! Refract or reflect given photon using given refractive index and normal vector.
	/*! When you set both of tryRefract and tryReflect flags function refract or 
	 * reflect given photon with proper probability. When you set only one of
	 * this flags, photon is reflected/refracted and its color is multiplied by
	 * probability of selected action. When no flags are set, color of photon
	 * is set to Vector3f::ZERO.
	 *
	 * \param[in,out] photon Photon which will be refracted/reflected.
	 * \param[in] refractiveIndex A refractive index.
	 * \param[in] normal A normal vector of refracting surface.
	 * \param[in] tryRefract a flag indicating whether you want to refract photon.
	 * \param[in] tryReflect a flag indicating whether you want to reflect photon.
	 *
	 * \return True when photon is refracted, false when when photon is reflected.
	 */
	static bool refract(
		Photon &photon, 
		float refractiveIndex, 
		const Vector3f& normal,
		bool tryRefract,
		bool tryReflect);
};

}

#endif
