/* ----------------------------------------------------------------------------
This file is part of BlueSky

Copyright (C) 2012 Piotr Polesiuk

Permission is hereby granted, free of charge, to any person obtaining a copy 
of this software and associated documentation files (the "Software"), to deal 
in the Software without restriction, including without limitation the rights 
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
copies of the Software, and to permit persons to whom the Software is 
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in 
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
THE SOFTWARE.
---------------------------------------------------------------------------- */

#ifndef BS_REFRACTIONPOWER_H_INCLUDED
#define BS_REFRACTIONPOWER_H_INCLUDED

#include "BSRefraction.h"

namespace BlueSky
{

//! Refraction model based on power function shifted by constant.
/*! The refractive index is calculated from the wavelength by the formula:
 * \f[ n = s + c\lambda^p
 * \f]
 * where n is a refractive index, s - shift, c - scale, p - power.
 *
 * For most materials, it is very good approximation, but not the fastest.
 * If you are using monochromatic light, use RefractionConst class.
 *
 * \remark This is a dynamic Object, so can be created only by new operator.
 * If you want to create local objects of RefractionPower, use ObjectManager
 * class.
 *
 * \sa RefractionConst, Object, ObjectManager
 */
class RefractionPower : public Refraction
{
private:

	float shift;
	float scale;
	float power;

public:

	//! Possibile predefine materials for PrefractionPower model.
	enum Material 
	{
		//! Diamond
		DIAMOND,
		//! Ice
		ICE,
		//! Clean water in room temperature. 
		WATER,
	};

	//! Create new RefractionPower model for given shift, scale and power.
	/*!
	 * \param[in] _shift Shift.
	 * \param[in] _scale Scale.
	 * \param[in] _power Power.
	 */
	RefractionPower(float _shift, float _scale, float _power);

	//! Create new RefractionPower model for given predefined material.
	/*!
	 * \param[in] material Predefined material.
	 */
	RefractionPower(Material material);

	//! Gets refractive index from wavelength
	/*!
	 * \param[in] wavelength A wavelength.
	 *
	 * \return Refractive index.
	 */
	float getRefractiveIndex(float wavelength);
};

}

#endif
