/* ----------------------------------------------------------------------------
This file is part of BlueSky

Copyright (C) 2012 Piotr Polesiuk

Permission is hereby granted, free of charge, to any person obtaining a copy 
of this software and associated documentation files (the "Software"), to deal 
in the Software without restriction, including without limitation the rights 
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
copies of the Software, and to permit persons to whom the Software is 
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in 
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
THE SOFTWARE.
---------------------------------------------------------------------------- */

#ifndef BS_ROTATIONALTERNATIVE_H_INCLUDED
#define BS_ROTATIONALTERNATIVE_H_INCLUDED

#include "BSRotation.h"

namespace BlueSky
{

//! Rotation model which selects selects other rotation model with given probablilty.
/*! When object of RotationAlternative contains list of rotation models
 * R1, R2, ... , Rn and list of probabilities P1, P2, ... , Pn, it gets rotation
 * from model Rk with probability Pk or rotation from default model with 
 * probability 1 - P1 - P2 - ... - Pn. Sum P1 + P2 + ... + Pn shouldn't be 
 * greater then 1.
 *
 * \remarks This is a dynamic Object, so it can be created only by new operator.
 * If you want to use local objects of RotationAlternative, use ObjectManager
 * class.
 */
class RotationAlternative : public Rotation
{
private:

	Rotation *defaultRotation;
	int maxRotationCount;
	int rotationCount;
	Rotation **rotations;
	float *probabilities;

public:

	//! Creates new RotationAlternative object.
	/*! After creation, object contains no rotation and default rotation is
	 * an identity rotaions (with probability 1).
	 */
	RotationAlternative();

	//! Creates new RotationAlternative object with given default rotation.
	/*! After creatios, rotation list is empty.
	 *
	 * \param[in] _defaultRotation Default rotation for created object.
	 */
	RotationAlternative(Rotation *_defaultRotation);

	//! Creates new RotationAlternative object with given rotation list.
	/*! Default rotation is set to identity rotation.
	 *
	 * \param[in] _rotationCount Number of rotations in rotation list.
	 * \param[in] _rotations Rotation list of length equal to _rotationCount.
	 * \param[in] _probabilities List of probabilities for rotations stored in 
	 * _rotations.
	 */
	RotationAlternative(
		int _rotationCount, 
		Rotation **_rotations, 
		float *_probabilities);

	//! Creates new RotationAlternative object with given rotation list and default rotation.
	/*!
	 * \param[in] _rotationCount Number of rotations in rotation list.
	 * \param[in] _rotations Rotation list of length equal to _rotationCount.
	 * \param[in] _probabilities List of probabilities for rotations stored in 
	 * _rotations
	 * \param[in] _defaultRotation A default rotation.
	 */
	RotationAlternative(
		int _rotationCount, 
		Rotation **_rotations, 
		float *_probabilities, 
		Rotation *_defaultRotation);

	//! Destructor of RotationAlternative.
	/*! Remove references for all contained rotation.
	 */
	~RotationAlternative();

	//! Adds rotation to rotation list.
	/*! New rotation is added on the end of list.
	 *
	 * \param[in] rotation Added rotation.
	 * \param[in] probability Probability for added rotation.
	 */
	void addRotation(Rotation *rotation, float probability);

	//! Sets default rotation.
	/*! This method removes reference to old default rotation.
	 *
	 * \param[in] rotation New default rotation.
	 */
	void setDefaultRotation(Rotation *rotation);

	//! Gets default rotation.
	/*!
	 * \return Rotation got as default.
	 */
	inline Rotation *getDefaultRotation()
	{
		return defaultRotation;
	}

	//! Gets rotation.
	/*!
	 * \return Quaternionf representing selected rotations.
	 * Rotation is got from rotation list with corresponded probabilities.
	 * When no rotation is selected from list, method gets quaternion from
	 * default rotation model.
	 */
	Quaternionf getRotation();
};

}

#endif
