/* ----------------------------------------------------------------------------
This file is part of BlueSky

Copyright (C) 2012 Piotr Polesiuk

Permission is hereby granted, free of charge, to any person obtaining a copy 
of this software and associated documentation files (the "Software"), to deal 
in the Software without restriction, including without limitation the rights 
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
copies of the Software, and to permit persons to whom the Software is 
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in 
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
THE SOFTWARE.
---------------------------------------------------------------------------- */

#include "BSSkyBoxCamera.h"

#include <cstring>

namespace BlueSky
{

SkyBoxCamera::SkyBoxCamera(int _resolution)
{
	resolution = _resolution;
	for (int i = 0; i < 6; i++)
	{
		data[i] = new Vector3f[resolution * resolution];
		memset(data[i], 0, resolution * resolution * sizeof(Vector3f));
	}

	photon_count = 0LL;
}

SkyBoxCamera::~SkyBoxCamera()
{
	for (int i = 0; i < 6; i++)
	{
		delete [] data[i];
	}
}

Vector3f *SkyBoxCamera::getMap(const Vector3f& direction, float &x, float &y) const
{
	if (Math::abs(direction.x) > Math::abs(direction.y))
	{
		if (Math::abs(direction.x) > Math::abs(direction.z))
		{
			if (direction.x > 0.0f)
			{
				x = direction.z / direction.x;
				y = direction.y / direction.x;
				return data[0];
			}
			else
			{
				x = - direction.z / direction.x;
				y = - direction.y / direction.x;
				return data[1];
			}
		}
		else
		{
			if (direction.z > 0.0f)
			{
				x = direction.x / direction.z;
				y = direction.y / direction.z;
				return data[2];
			}
			else
			{
				x = - direction.x / direction.z;
				y = - direction.y / direction.z;
				return data[3];
			}
		}
	}
	else
	{
		if (Math::abs(direction.y) > Math::abs(direction.z))
		{
			if (direction.y > 0.0f)
			{
				x = direction.x / direction.y;
				y = direction.z / direction.y;
				return data[4];
			}
			else
			{
				x = - direction.x / direction.y;
				y = - direction.z / direction.y;
				return data[5];
			}
		}
		else
		{
			if (direction.z > 0.0f)
			{
				x = direction.x / direction.z;
				y = direction.y / direction.z;
				return data[2];
			}
			else
			{
				x = - direction.x / direction.z;
				y = - direction.y / direction.z;
				return data[3];
			}
		}
	}
}

void SkyBoxCamera::collect(const Photon& ph)
{
	float x, y;
	Vector3f *map = getMap(ph.dir, x, y);
	int ix = (int)((float)resolution * 0.5f * (1.0f + x));
	int iy = (int)((float)resolution * 0.5f * (1.0f + y));
	if (ix < 0) ix = 0;
	else if (ix >= resolution) ix = resolution-1;
	if (iy < 0) iy = 0;
	else if (iy >= resolution) iy = resolution-1;

	map[resolution * iy + ix] += ph.color;
	photon_count++;
}
	
Vector3f SkyBoxCamera::getDataByDirection(const Vector3f& direction) const
{
	float x, y;
	Vector3f *map = getMap(direction, x, y);
	float pix_x = (float)resolution * 0.5f * (1.0f + x);
	float pix_y = (float)resolution * 0.5f * (1.0f + y);
	int ix = (int)pix_x;
	int iy = (int)pix_y;
	pix_x = Math::fractionalPart(pix_x);
	pix_y = Math::fractionalPart(pix_y);
	if (ix < 0) ix = 0;
	else if (ix >= resolution) ix = resolution-1;
	if (iy < 0) iy = 0;
	else if (iy >= resolution) iy = resolution-1;
	int ix2 = ix+1;
	int iy2 = iy+1;
	if (ix2 == resolution)
		ix2 = resolution-1;
	if (iy2 == resolution)
		iy2 = resolution-1;

	return 
		((1.0f - pix_y) * (map[resolution * iy + ix] * (1.0f - pix_x) 
			+ map[resolution * iy + ix2] * pix_x) +
		pix_y * (map[resolution * iy2 + ix] * (1.0f - pix_x) 
			+ map[resolution * iy2 + ix2] * pix_x))
		* (1.0f + x*x + y*y) // pixel position correction
		* (float)(resolution * resolution) // resolution correction
		/ (float)photon_count;
}

void SkyBoxCamera::clean()
{
	photon_count = 0LL;
	for (int i = 0; i < 6; i++)
	{
		memset(data[i], 0, sizeof(Vector3f) * resolution * resolution);
	}
}

}
