/* ----------------------------------------------------------------------------
This file is part of BlueSky

Copyright (C) 2012 Piotr Polesiuk

Permission is hereby granted, free of charge, to any person obtaining a copy 
of this software and associated documentation files (the "Software"), to deal 
in the Software without restriction, including without limitation the rights 
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
copies of the Software, and to permit persons to whom the Software is 
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in 
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
THE SOFTWARE.
---------------------------------------------------------------------------- */

#ifndef BS_SKYBOXCAMERA_H_INCLUDED
#define BS_SKYBOXCAMERA_H_INCLUDED

#include "BSCamera.h"

namespace BlueSky
{

//! Simple camera which registers photons from all directions.
/*! This camera uses six square matrices, each for one direction (north, 
 * south, west, east, up, down). If you want to render only phenomena with
 * axial symmetry, use LinearCamera.
 *
 * \remarks This is a dynamic object so can be created only by new constructor
 * If you want to ues local objects of LinearCamera use ObjectManager class.
 *
 * \sa Object, ObjectManager, LinearCamera
 */
class SkyBoxCamera : public Camera
{
private:

	int resolution;
	Vector3f *data[6];

	long long photon_count;

	Vector3f *getMap(const Vector3f& direction, float &x, float &y) const;

public:

	//! Creates new SkyBoxCamera.
	/*!
	 * \remarks This object can be created only by new operator.
	 *
	 * \param[in] _resolution Resolution of each of six square matrices of 
	 * created camera.
	 */
	SkyBoxCamera(int _resolution);

	//! Destructor of SkyBoxCamera.
	~SkyBoxCamera();

	//! Collects photon.
	/*!
	 * \param[in] ph A Photon which is collected.
	 */
	void collect(const Photon& ph);
	
	//! Gets color registred in given direction
	/*!
	 * \remarks A direction means direction of photons which activated reqested
	 * cell on camera. This direction is opposite to direction in which camera
	 * looks.
	 *
	 * \param[in] direction Requested direction.
	 *
	 * \return Color registered on given direction.
	 */
	Vector3f getDataByDirection(const Vector3f& direction) const;

	//! Gets a resolution of camera.
	/*!
	 * \return Resolution of each of six matrices of camera.
	 */
	inline int getResolution() const
	{
		return resolution;
	}

	//! Cleanup camera data.
	/*! Resets all camera data.
	 */
	void clean();
};

}

#endif
