/* ----------------------------------------------------------------------------
This file is part of BlueSky

Copyright (C) 2012 Piotr Polesiuk

Permission is hereby granted, free of charge, to any person obtaining a copy 
of this software and associated documentation files (the "Software"), to deal 
in the Software without restriction, including without limitation the rights 
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
copies of the Software, and to permit persons to whom the Software is 
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in 
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
THE SOFTWARE.
---------------------------------------------------------------------------- */

#ifndef BS_VECTOR3F_H_INCLUDED
#define BS_VECTOR3F_H_INCLUDED

#include "BSMath.h"

namespace BlueSky
{

//! Standart 3-dimensional vector of floats.
/*!
 * \remark Vector3f is also used to represent color, where the x, y and z
 * coordinates denote the r, g and b channels.
 */
class Vector3f
{
public:
	//! X coordinate of vector or R channel of color.
	float x;
	//! Y coordinate of vector or G channel of color.
	float y;
	//! Z coordinate of vector or B channel of color.
	float z;

	//! Creates new vector.
	inline Vector3f()
	{
	}

	//! Creates new vector with given coordinates.
	/*!
	 * \param[in] _x X coordinate of created vector.
	 * \param[in] _y Y coordinate of created vector.
	 * \param[in] _z Z coordinate of created vector.
	 */
	inline Vector3f(const float _x, const float _y, const float _z) :
		x(_x), y(_y), z(_z)
	{
	}

	//! Copy a vector.
	inline Vector3f& operator = (const Vector3f& v)
	{
		x = v.x;
		y = v.y;
		z = v.z;
		return *this;
	}

	//! Check for equality of vectors.
	inline bool operator == (const Vector3f& v) const
	{
		return (x == v.x && y == v.y && z == v.z);
	}

	//! Check for inequality of vectors.
	inline bool operator != (const Vector3f& v) const
	{
		return (x != v.x || y != v.y || z != v.z);
	}

	//! Compute sum of two vectors.
	inline Vector3f operator + (const Vector3f& v) const
	{
		return Vector3f(
			x + v.x,
			y + v.y,
			z + v.z);
	}

	//! Compute a difference of two vectors.
	inline Vector3f operator - (const Vector3f& v) const
	{
		return Vector3f(
			x - v.x,
			y - v.y,
			z - v.z);
	}

	//! Compute a product of vector and scalar.
	inline Vector3f operator * (const float c) const
	{
		return Vector3f(
			x * c,
			y * c,
			z * c);
	}

	//! Compute a quotient of vector and scalar.
	inline Vector3f operator / (const float c) const
	{
		float cinv = 1.0f / c;

		return Vector3f(
			x * cinv,
			y * cinv,
			z * cinv);
	}

	//! Unary plus.
	inline const Vector3f& operator + () const
	{
		return *this;
	}

	//! Compute the negation of vector.
	inline Vector3f operator - () const
	{
		return Vector3f(-x, -y, -z);
	}

	//! Compute the product of scalar and vector.
	inline friend Vector3f operator * (const float c, const Vector3f& v)
	{
		return Vector3f(
			c * v.x,
			c * v.y,
			c * v.z);
	}

	//! Add vector to another.
	inline Vector3f& operator += (const Vector3f& v)
	{
		x += v.x;
		y += v.y;
		z += v.z;

		return *this;
	}

	//! Subtract a vector from another.
	inline Vector3f& operator -= (const Vector3f& v)
	{
		x -= v.x;
		y -= v.y;
		z -= v.z;

		return *this;
	}

	//! Multiply vector by scalar.
	inline Vector3f& operator *= (const float c)
	{
		x *= c;
		y *= c;
		z *= c;

		return *this;
	}

	//! Divide vector by scalar.
	inline Vector3f& operator /= (const float c)
	{
		float cinv = 1.0f / c;

		x *= cinv;
		y *= cinv;
		z *= cinv;
		
		return *this;
	}

	//! Gets length of vector.
	/*!
	 * \return Length of vector.
	 */
	inline float length() const
	{
		return Math::sqrt(x*x + y*y + z*z);
	}

	//! Gets square of length of vector.
	/*!
	 * \return Squared lenght of vector.
	 */
	inline float lengthSQ() const
	{
		return x*x + y*y + z*z;
	}

	//! Compute dot product of two vectors.
	/*! 
	 * \param[in] v Second argument for dot product.
	 *
	 * \return Dot product of given vectors.
	 */
	inline float dot(const Vector3f& v) const
	{
		return x*v.x + y*v.y + z*v.z;
	}

	//! Compute cross product of two vectors.
	/*!
	 * \param[in] v Second argument for cross product.
	 *
	 * \return Cross product of given vectors.
	 */
	inline Vector3f cross(const Vector3f& v) const
	{
		return Vector3f(
			y*v.z - z*v.y,
			z*v.x - x*v.z,
			x*v.y - y*v.x);
	}

	//! Normalise a vector.
	/*!
	 * \return Length of vector before normalisation.
	 */
	inline float normalise()
	{
		float l = Math::sqrt(x*x + y*y + z*z);
		float linv = 1.0f / l;

		x *= linv;
		y *= linv;
		z *= linv;

		return l;
	}

	//! Gets normalised copy of vector.
	/*!
	 * \return Normalised copy of vector.
	 */
	inline Vector3f normalised() const
	{
		Vector3f res = *this;
		res.normalise();
		return res;
	}

	//! Finds two vectors, so that form orthonormal base with given vector.
	/*!
	 * \param[out] v1 First vector of found orthonormal base.
	 * \param[out] v2 Second vector of found orthonormal base.
	 */
	void orthoBase(Vector3f& v1, Vector3f& v2) const;

	//! Zero vector.
	static const Vector3f ZERO;
	//! Unit X vector.
	static const Vector3f UNIT_X;
	//! Unit Y vector.
	static const Vector3f UNIT_Y;
	//! Unit Z vector.
	static const Vector3f UNIT_Z;
};

}

#endif
